/*
* BlockIP2.c -	security channel exit to control connection name of incomming call to a channel,
*		This security exit will only allow connection if the connectionname, and username
*		match the specification on the channel definition or conform to the rules in the
*		specified file.
*
* --------------------------------------------------------------------------
* Description
*
* This exit will check the connection name and see if it matches the pattern of the one
* specified.
* This module was originally based on the IBM support-pack MS05.
*
* This version of the program is a development edition and is without any
* guarantee, and use of it is on your own risk.
*
* Author:
*	   Jrgen Pedersen (c)
*	   email: oz1ccg@mrmq.dk
*
* Copyright:
*	   This program may be used and changed, but may not be sold
*	   or incoperated in any softwarepackage without the authors written permission.
*	   I would like to receive a copy of the improved version, to gather ideas
*	   for further improvements for my exit suite.
*
* --------------------------------------------------------------------------
* History
*
* Written by Jrgen Pedersen								  - 12Dec02 DK	v1.00
*
* Changes
*	   Jrgen H. Pedersen								  - 06Jan04 DK	v1.10
*	   Improved due to inspiration from T.Rob, Sriniddhi and Peter Potkay
*	   Now we allow more patterns specified, seperated by semicolon (;), the number
*	   of patterns is limited ny size of SCYDATA field.
*	   Protection against JMS intruders, or users using mqm or MUSR_MQADMIN.
*
*	   Michael Dag									  - 20Jan04 NL	v1.11
*	   Changed / / comments to / * with ending ... as it wasn't suited for C
*	   compiler on AIX
*	   Changed IFDEF to find out WIN32 or UNIX as UNIX can have 1 CPU aswell
*	   Commented the #pragma statement as this wasn't suited for C compiler
*	   on AIX
*	   Removed * from string and wild in the wildcmp routine and added [0] as
*	   it didn't work on Solaris.
*
*	   Jrgen H. Pedersen								  - 22Jan04 DK	v1.12
*	   Problem with simple pattern (172.22.22.*) this failed due to a design
*	   mistake, this is now solved. Now should the Exit work as designed.
*	   Log improved using real date format, instead of internal formats...
*	   Conditional compilation added, so the same source can be compiled on
*	   differnt platforms. Some other improvements due to nature of UNIX/LINUX
*	   so the log list is fine.
*
*	   Jrgen H. Pedersen								  - 06Feb04 DK	v1.14
*	   Problem with missing timestamp in the log solved. Version 1.14.
*	   Problem with patternmatching without trailing asterix, this problem
*	   was introduced in 1.11.
*
*	   Jrgen H. Pedersen								  - 10Feb04 DK	v1.15
*	   Added funtionallity to verify the connecting userid, based on remote
*	   userid, this matching is also based on wild_cmp(), so wildcards is
*	   allowed. This pattern matching is case sensetive, so was51 is different
*	   from Was51. Check also added for right exit invocation. Any other exit
*	   will just be terminated.
*
*	   inspiration:
*		 SCYDATA('10.1.10.*;-n;userids=zz*,yy*')
*		 SCYDATA('10.1.10.*;10.2.11.*;userids=michael,zz*,yy*')
*
*	   Jrgen H. Pedersen								  - 15Feb04 DK	v1.16
*	   Added functionallity to allow specification rules to be passed in a file,
*	   this file is specified in SCYDATA() like (C:\path\rulefile.txt) or
*	   (/var/mqm/exits/rulefile.txt) depending on your platform.
*	   This allows you to thae a rulefile per channel, and this offers flexibility
*	   to implement various security schemas.
*	   Two extra parameters have been added to SCYDATA: -d & -q. -d is used for
*	   debuging purposes, while -q is q Quitemode, where the output is very
*	   reduced (only one line per attempt to connect).
*
*	   Jrgen H. Pedersen								  - 16Feb04 DK	v1.17
*	   A serious user-validation error was introduced in 1.16 which only allowed usage
*	   of FN= mode. The error is corrected (i hope), and my tests shows it works.
*	   Enhanced error reporting, and control of filters. And still be restrictive
*	   so a nRetCode don't open the connection.
*
*	   Jrgen H. Pedersen								  - 20Feb04 DK	v1.18
*	   The exit is now able to allow blank lines and empty lines.
*
*	   Jrgen H. Pedersen								  - 20Feb04 DK	v1.20
*	   The exit is extented to display SSL properties. Pattern matching for x503
*	   development started. Specifications are passed in configuration file.
*	   no test are done if options != "FN=" due to performance reasons.
*	   This filter option is combined with an DN->User generic mapping function
*	   to ease systems administration, so unknown users with valid certificates
*	   can be kept outside our world.
*	   The way the SSL-specs are searched is top down and with stop on first match,
*	   this gives the ablility to "shop" in the CN's, and just take the ones we
*	   like and block the rest.
*	   Following options are implemented:
*	   SSL=CN=ibmwebspheremqQSJHPT01,MCA=user;	 change MCAUSER for this CN
*	   SSL=CN=ibmwebspheremq*,MCA=*;			 allow all connections from ibmwebsphere*
*	   SSL=CN=*;BLOCK;							 Refuse all connections
*	   SSL=CN=*;MCA=NoBody; 					 set MCAUSER for CNs not matching
*
*	   Jrgen H. Pedersen								  - 05Mar04 DK	v1.21
*	   The logic of the exit have been changed so it's no longer possible to penetrate
*	   this exit using a exit on the client end, to send a sec_msg, to get a connection
*	   accepted status.
*	   This design mistake would leave the channel open to almost anybody, that was capable
*	   of creating a client security exit.
*
*	   Jrgen H. Pedersen								  - 20Mar04 DK	v1.22
*	   Added functions to specify connectionname/userid match, so we can combine
*	   the features of security. This matching feature only works on FN= mode, where
*	   the connectionname/userid is specified using the CON= keyword:
*	   CON=<conname>;<userids>[;MCA={*|userid}];
*	   CON=10.11.12.*;u*;MCA=sysoper;
*	   CON=192.162.1.14;zz*;
*	   The list is searched for first match of connectionname+userid, which means it's very
*	   important to specify the options in the right order.
*	   If you specify:
*	   CON=*;*;MCA=root;
*	   CON=162.12.*;peter;
*	   This will have the result that all incomming connection attemts will have the MCAUSER forced
*	   to "mqm", even "peter", because all connetions+userids will match*;*;. You see the point ?
*
*  2004-03-22  Sid Young Complete re-write and restructure of code.
*          Changed logic to deny all and only accept if all conditions required are met
*          Added enhanced logging capability
*          Changed BlockExit() to call seperate init, parse and test methods.
*
*  2004-03-29  Sid Young Added code to support chanel and user name stamping in log file name.
*	   Found type in ProcessLine(). Tested OK.
*
*	   Jrgen H. Pedersen								  - 30Mar04 DK	v2.10
*	   Retrofitted some code to create a windows version of the program that could
*	   be loaded. And to allow compability with old versions that works without a
*	   rule-file. Restrictions added so connections will be refused if failure in
*	   rule/parameter specifications, because my auto test passed many connection
*	   attempts that should be blocked due to rules.
*	   This is a valid rule spec:127*;userids=u*;-q; resulted in : Warning - Unknown Tag [127*;userids=u*;-q;]
*	   and gives connection accepted .... :o(
*	   Problem with blank rule lines -> gives connection blocked.
*
*	   Exit changed so it respects commented lines containg valid commands like:
*	   #BlockMqmUsers=Y; which in the redesigned version was taken as a command.
*	   This was caused by missing init of variables if the exit was redriven, so the state
*	   was saved between invocations on the same connection.
*
*	   Specification of logfiles changed to conform with the used standard where all specs.
*	   are terminated with a semicolon.
*	   Small changes where numeric constants have been changed to WMQ supplied constants.
*
*	   Jrgen H. Pedersen								  - 27May04 DK	v2.11
*	   Some editorial changes to allow compilation under AIX without problems.
*	   UNIX logging changed, so misspecified log information still will result in logging,
*	   where the spec. is bypassed. LogDirecty removed and is replaced with LogPath, for
*	   complience between environments.
*
*	   Jrgen H. Pedersen								  - 29May04 DK	v2.12
*	   Repporting of refused connections added to log, so we can see who is trying to get in,
*	   this was disapeared on Petterns= and CON= mismatch this is fixed so we can continue.
*	   UseridUpperLowerCase= Introduced. Default is case sensetive to be compatible with older
*	   versions. To ignore case use UseridUpperLowerCase=*. Any other is invalid.
*	   BlockUsers=, added, so we now can use a negativelist, but still with precedence of
*	   CON=, and SSL=, meaning that we can change an incomming user to a blocked user!
*
*	   Jrgen H. Pedersen								  - 07Jun04 DK	v2.13
*	   Validation of SCYDATA field is enhanced, so more errors are reported, for investigation.
*
*	   Neil Casey									  - 03Aug04 	v2.13nwc
*	   No change to functionality at all, but added Windows compiler information so
*	   that external link directives are not required, and added MVS directives to
*	   allow generation of an MVS targetted exit. Files specifications on MVS can be
*	   //DD:JCLNAME or UFS hierarchical names (ie unix names)
*	   Also fixed issues with ConnectionNames. The code was using the EXIT_NAME_LENGTH to
*	   get a value for the length of the ConName. This doesn't work on MVS where the
*	   exit name length is 8. Change all references to use the MQ defined constant
*	   MQ_CONN_NAME_LENGTH.
*
*	   Jrgen H. Pedersen								  - 05Aug04 DK	v2.14
*	   Validation of SCYDATA field is enhanced, so errors on specifying FN=file is
*	   enhanced. If KW FN= not specified, it's mandatory that SCYDATA contains either
*	   a asterix(*), question mark(?) or a pattern starting with 0-9. Due to the fact
*	   that the latest problems reported is in this area of bad specification.
*	   Hard-brackets ([]) is changed in z/OS implementation to (<>), so it becomes
*	   readable in non-english environments.
*
*	   Neil Casey									   - 10Aug04	v2.13nwc2
*	   Extended maximum pattern lengths to 256 characters. Changed logic for building
*	   the pattern strings so the multiple Pattern= or User= etc lines add to the
*	   previously built string, instead of overwriting it. This is a function change
*	   which makes this version behave differently when faced with multiple lines of data
*	   which previously just used the last data found.
*
*	   Jrgen H. Pedersen								  - 11Aug04 DK	v2.15
*	   just a small one... together with a small code change for solaris so that the
*	   timestamp is printed too. This is done using wcsftime...
*
*	   Neil Casey									  - 24Aug04 	v2.16
*	   Remove the wcsftime call. The parameter passed to it (char*) by the code did
*	   not match the expected parameter (wchar*) The timestamping works fine on Solaris
*	   without this mod. Remove some commented out code which was for debugging.
*          Reformat source indenting.
*
*	   Jrgen H. Pedersen								  - 08Feb05 DK	v2.17
*	   A small change about setting MCAUSER based on CON= control, z/OS
*	   will have a field filled with trailing spaces. Added RespectMCA keyword.
*
*	   Jrgen H. Pedersen								  - 21Apr05 DK	v2.18
*	   Handling of SEC_PARMS added as part of WMQ 6.0 support.
*
*	   Jrgen H. Pedersen								  - 24Apr05 DK	v2.20
*	   Added support for max connections on a given channel.
*	   new keyword: MAXCHL=ChannelName;MaxConnections; was added.
*	   This is currently not supported on WebSphere MQ for Z/OS.
*
*	   Jrgen H. Pedersen								  - 03May05 DK	v2.21
*	   Problem with userid check in CON= Fixed.
*	   Wildcard compare extented with character range, numeric, alpha patterns,
*	   to enhance filtering capabilities.
*	   AllowBlankUserID added, Default is changed so we don't allow blank userids
*	   anymore.
*
*	   Jrgen H. Pedersen								  - 17May05 DK	v2.22
*	   MAXCHL=ChannelName;MaxConnections; is now supported on
*	   WebSphere MQ for Z/OS prior to version 6.0. using MQSC:
*	   "DISPLAY CHS(<channel name>) CURRENT" command.
*	   Explanation on exit-reason and channel type enhanced, to give the user
*	   a better understanding of what is happening.
*	   We have fixed the known problem in the userid-list of the CON= specification
*	   so it now works as designed.
*	   On z/OS we're also blocking connection attempts using the STCID from the xxxxCHIN task,
*	   this should prevent unauth. access.
*
*	   Jrgen H. Pedersen								  - 1Jul05 DK	v2.30
*          Problem with reentant code changed, so it should be reentrant.
*          Syntax check of parameter specification is improved.
*
*	   Jrgen H. Pedersen								  - 11Sep05 DK	v2.31
*          strtok changed to strtok_r for non windows implementations to deal with
*          reentant code problems. This should help make it more stable.
*
*	   Jrgen H. Pedersen								  - 20Sep05 DK	v2.32
*          MAXCHL for unix changed to use shrmem for performance reasons.
*
*	   Jrgen H. Pedersen								  - 25Oct05 DK	v2.35
*          OS/400 support added (bypass strtok_r) and handling a new print model.
*          New OS/400 entrypoint (int main() added with conditional compile.
*
*	   Jrgen H. Pedersen								  - 14Nov05 DK	v2.36
*          Handling of semaphores in LockSpecSem() fixed.
*
*	   Jrgen H. Pedersen								  - 14Nov05 DK	v2.37
*          Connection refused message enhanced to conname and channelname
*
*	   Jrgen H. Pedersen								  - 12Dec05 DK	v2.38
*          Connection refused, Pattern string is too long, max. This message wasn't
*          showing the resulting message length, just the current one. So when concat
*          patterns we had a issue.  This was also fixed for Userids
*
*	   Jrgen H. Pedersen								  - 25Jan06 DK	v2.39
*          Porting for Linux AMD64. various definitions changed.
*
*	   Jrgen H. Pedersen								  - 5Feb06 DK	v2.40
*          Problem with semaphore addressing fixed. And a bit about logging on UNIX.
*          shmdt(shm); chaned to shmdt((char*)shm); due to changes on Solaris x86
*
*	   Jrgen H. Pedersen								  - 13Feb06 DK	v2.41
*          Problem when a config file contains garbage and MQXR_TERM is
*          invoked and it returns a failure .... This was seen WIN2K3.
*
*	   Jrgen H. Pedersen								  - 14Feb06 DK	v2.42
*          Logging to EventLog added on windows. Currently all events are logged.
*          Switch will be added to SCYDATA to control this feature.
*
*	   Jrgen H. Pedersen								  - 18Feb06 DK	v2.43
*          Needed cleanup procedure for getActualCurrentNumberOfRunningChannels to cleanup
*          allocated "databags" mqDeleteBag must be included in the windows edition to prevent
*          storage leak.
*
*	   Jrgen H. Pedersen								  - 1Mar06 DK	v2.44
*          Added support for shared memory on windows. This is implemented using a seperate
*          program BlockIP2S, that initializes the Shared Memory Segment(SMS). There are a
*          detatched BlockIP2S per queue manager running with BlockIP2. (when the channel limitter
*          is activated).
*          Storage Leak from pre. version 2.44 is also solved. This appled only to the delivered
*          windows DLL, due to a compilation problem. Serilasation was added in windows to handle
*          logfile contention. This is also included in the control of SMS and BlockIP2S.
*
*	   Jrgen H. Pedersen								  - 19Mar06 DK	v2.45
*          Shared memory naming changed to support HACMP and other MA special settings.
*          The shm name under *NIX is now based on /var/mqm/mqs.init
*
*	   Jrgen H. Pedersen								  - 4Apr06 DK	v2.46
*          Added logic to handle cycling of logfiles, to prevent fill up of filesystems.
*          Controlled by:
*          LogCount=nn; # of versions (between 3 and 99).
*          LogSize=nnnnnnn; Size og the logfile before switching. min 100KB.
*          Default LogDrive og LogPath log path is changed on windows to conformw with the
*          UNIX implementation, so we use the windows settings. And FileName is extented with "001"
*          to allow circular logging.
*
*	   Jrgen H. Pedersen								  - 5Apr06 DK	v2.48
*          Problem with Log file rotation solved. Error messages was enhanced.
*
*/


/*
* Needed changes:
*          Currently none.
*/


/*
* Planned changes
*	   Possibility of specifying MCAUSER, to help on JMS connection difficulties,
*	   done without SSL or other encrytion. This is just meant to be a small
*	   security imoprovement, not a 100% solution. Could be spec. like:
*	   JMS=<channel_name>;<conname>;<MCAUSER>;[DEF|ONLYMCA;]
*	   Implemented 80% in version 1.22
*
*	   Ideas for SSL security enhancements:
*	   SSL=[C=,][L=,][O=,][OU=,]CN=;[[MCA={userid|role|*};]|[BLOCK;]] some examples:
*	   SSL=CN=ibmwebspheremqQSJHPT01,MCA=user;	change MCAUSER for this DN
*	   SSL=O=CSC;CN=*;BLOCK;			Refuse connections from CSC
*	   SSL=CN=*;MCA=NoBody; 			set MCAUSER for DNs not matching
*							any previous specified patterns
*
*	   Improve performace, so we only reads the file once, and run on this information without
*	   having to reading, parsing, tokenizing it. 040522.
*/

/*
* Version no
*/

#define VERSION 	 "2.48"


/* --------------------------------------------------------------------------
* Build hints
*
*
*	For NT:
*	  cl /MT /LD BlockIP2.c
*	  linker def file:
*		LIBRARY QMCONN
*		DATA MULTIPLE
*		EXPORTS BlockExit
*
*	For Solaris:
*	  cc -c -KPIC BlockIP2.c
*	  ld -G BlockIP2.o	-o Block
*	  cp BlockIP2 /var/mqm/exits
*
*
* Compiled on Windows using Microsoft C:
*          cl BlockIP2.c /MT /DWIN32 /LD mqm.lib advapi32.lib /DWIN32_SHM
*
* Compiled on Linux using GNU C:
*	   gcc	-fPIC -shared  -o BlockIP2 BlockIP2.c -lmqm_r -DUNIX -D_RENTRANT
*
* Compiled on Linux AMD_64 using GNU C:
*          gcc -m64 -o BlockIP2 BlockIP2.c -fPIC -ansi -shared -D_REENTRANT -D AIX -D UNIX -D_XOPEN_SOURCE -L/opt/mqm/lib64
*
*
* Compilation parameters supplied by Michael Dag:
* Compiled on Solaris using GNU C	 : gcc -G -I/opt/include -I/opt/mqm/inc -L/opt/mqm/lib -o BlockIP2 BlockIP2.c -lmqm_r -D_REENTRANT -DUNIX
* Compiled on AIX 5.2 using C for AIX: cc  -G -I/usr/include -I/usr/mqm/inc -L/usr/mqm/lib -o BlockIP2 BlockIP2.c -lmqm_r -D_REENTRANT -DUNIX
*
* Compilation parameters supplied by techno:
* Compiled on HP-UX: cc -c +z BlockIP2.c -DUNIX
*	Linked on HP-UX: ld -b BlockIP2.o BlockIP2 -lmrm_r
*
* Compilation on OS400: CRTSRVPGM SRVPGM(BLOCKIP2) EXPORT(*ALL) BNDSRVPGM(LIBMQM_R)
*
* Compiled on Solaris X86_64 using GNU C:
*          gcc -c -m64 BlockIP2.c -ansi -fPIC -DPWEXIT -DUNIX -D_REENTRANT -I/opt/mqm/inc
*          /usr/ccs/bin/amd64/ld -64 -mt -G -o BlockIP2 BlockIP2.o -L/opt/mqm/lib64 -R/opt/mqm/lib64 -R/usr/lib/64 -lmqm -lmqmcs -lmqmzse -lsocket -lnsl -ldl
*
*
*
* --------------------------------------------------------------------------
*/

#define SEM_SHM 0
#define SEM_CHL 1

#if defined(__MVS__)
/* NWC Cater for MVS operation */
#define ENV "MVS"
#else
#define ENV "non-MVS"
#endif

/*
* standard headers
*/
#if !defined(WIN32)
#include <pthread.h>
#endif

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <ctype.h>

#if defined(WIN32)
#include <windows.h>
#include <sys\stat.h>
#define __EXT_LOG_FILENAME__ 1
#endif
/*
* MQ headers
*/
#include <cmqc.h>
#include <cmqxc.h>

/* start chl. inq */
#if !defined(__MVS__)
#include <cmqbc.h>
#include <cmqcfc.h>
#endif

#if defined(__MVS__)
#include <spc.h>
#endif

#ifndef MQXR_SEC_PARMS                /* Help version 5.x compilation to run on 6.0 qmgrs */
#define MQXR_SEC_PARMS 29
#endif

#if defined UNIX
#define __UNIX__ 1
#define __EXT_LOG_FILENAME__ 1
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <signal.h>
#include <unistd.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#include <sys/shm.h>        /* testing solaris */
#endif    /* end if unix like defines */


#if MQAT_DEFAULT==MQAT_OS400
#define _REENTRANT 1
#endif

/*
* Common Defines
*/

#define SEC_FAILURE 1
#define SEC_OK		2

#define MAX_SSL  16
#define SSL_LEN  80
#define MAX_CON  64
#define CON_LEN  80
#define MAX_PL	256
/*#define MAX_PL  128 */
#define MAX_LEN 400
#if defined (__MVS__)
#define LOGMODE "a"
#else
#define LOGMODE "a+"
#endif


/* character coding indicator */
#define CC_ASCII     1
#define CC_EBCDIC    2

/* Size of AgentBuffer */
#define WSIZE 80

/*
* Bit mask of Logging options
*/
#define LOGNAME    1	/* always used but defined for consistency */
#define LOGDATE    2
#define LOGCHANNEL 4
#define LOGUSER    8


#define LOGDATA  0				/* normal data */
#define LOGDEBUG 1				/* Debug information */

#define MXC_RC_NONE 0
#define MXC_RC_ERROR 1

#if defined (WIN32)
#define LOG_FILEONLY    1
#define LOG_EVTLOGONLY  2
#define LOG_BOTH        3
#endif

#if defined (__MVS__)		   /* define communication to BLOCKIPW */
#define WTOLENGTH 80

typedef struct _bi2wto
{							 /* communication structure */
    char msg[WTOLENGTH];
} bi2wto_type;

typedef struct _bi2dataspace
{							 /* communication structure */
    long version;
    char function[1];
    char channel[MQ_CHANNEL_NAME_LENGTH];
    long limit;
    long current;
    long maxrunning;
    char status[7];
} bi2ds_type;

#pragma linkage (BLOCKIPS,OS)
#pragma linkage (BLOCKIPW,OS)
#pragma linkage(csq4bax3,OS)

long BLOCKIPS(bi2ds_type *);
long BLOCKIPW(bi2wto_type *);
void csq4bax3(char *, char*, long *, long *);

#endif

#if defined UNIX
typedef struct _shmem_channels
{
    int  iNextChl;
    char szChannelName[MQ_CHANNEL_NAME_LENGTH+1];
    int  iActiveChannelLimit;
    int  iCurrentActiveConnections;
    int  iMaxActiveConnections;
} chlstruc;

#define SHMSZ 2048
#define SHMMAX_CHL ((SHMSZ-51) / sizeof(chlstruc)) -1
typedef struct _shmem_dataspace
{							 /* communication structure */
    int iPid;                            /* who created/owns the shmem */
    int iNumOfChannels;
    char Reserved[43];

    chlstruc chnl[SHMMAX_CHL];
} shmem_type;

#endif

#if defined WIN32
typedef struct _shmem_channels
{
    int  iNextChl;
    char szChannelName[MQ_CHANNEL_NAME_LENGTH+1];
    int  iActiveChannelLimit;
    int  iCurrentActiveConnections;
    int  iMaxActiveConnections;
} chlstruc;
#define SHMSZ 2048
#define SHMMAX_CHL ((SHMSZ-51) / sizeof(chlstruc)) -1

typedef struct _shmem_dataspace
{							 /* communication structure */
    int iPid;                            /* who created/owns the shmem */
    int iNumOfChannels;
    char Reserved[43];

    chlstruc chnl[SHMMAX_CHL];
} shmem_type;

#endif


typedef struct CURRCHLDATA
{
    long limCurrChl;
    long CurrChannels;
    long MaxChannels;
} currChl_type;

typedef struct WS
{
    int      bFirstTime;
    long     lMaxChannelActive;
#if defined (UNIX)
    char      szQmgrDirectory[255];
    char      szQmgrPrefix[255];
#endif

} ws_type;                             /* WS Buffer */

typedef struct WORKDATA
{
    /* start inq chl */
    MQHCONN  hConn;
    MQLONG   HobjStat;
    char     szStatusQ[MQ_OBJECT_NAME_LENGTH+1];
    char     szQMgr[MQ_Q_MGR_NAME_LENGTH+1];
    char     szChannelName[MQ_CHANNEL_NAME_LENGTH+1];
    long     lMaxChannelActive;
    char     szLogPath[128];
    char     szLogDrive[3]; 			/* only applicable to Windows systems */
    int      nLogFormat;				/* Bit mask 0-255 values */
    char     szLogChannel[64];
    char     szLogFileName[64];
    char     szLogFileExtension[32];
    char     szTmpBufff[256];
    char     szWTOBuffErr[256];
    char     szWTOBuffOk[256];
    /*
    * Flags
    */
    int      nDebugFlag;
    int      nZOSconv;						 /* convert brackets on z/OS to handle national chars */
    int      bRespectMCA;               /* we don't respect MCAsettings */
    int      bAllowBlankUserID;		/* We will not accept blank uids */
    int      bBlockMqmUsers;
    int      bDebugMode;
    int      SilentMode;
    int      bLogByDate;
    int      bUseridUpperLowerCase;		/* be case sensitive */
    int      ReportLogFileError;		/* only report logfileerror once */
    int      lExpireValue;		/* let channel record live for 10 minutes (in secs) */
    int      lRefuseReason;			/* send refuse reason back to client exit.. */
    int      bInteractive;			/* is the client requesting feedback ? */

    char     CONList[MAX_CON] [CON_LEN];
    char     SSLPeer[SSL_LEN];
    char     SSLList [MAX_SSL] [SSL_LEN];
    int      SSLCount;
    int      CONCount;

    char RemUID[MQ_USER_ID_LENGTH+1];  /* remote user identifier 		  */
    char ConName[MQ_CONN_NAME_LENGTH+1]; 		 /* Local copy of the connectionname */
    char Flags[64];
    char U2[MAX_PL];
    char Users[MAX_PL];	  /* list of userids */
    char BlockUsers[MAX_PL];	   /* list of Blocked userids */
    char Pattern[MAX_PL];	   /* The pattern to match against	  */
    char szClientUser[WSIZE];      /* user in the other end */
    int  bFileSizeCheck;           /* default is done=switched off to maintain transperacy */
    long lLogFileCount;            /* Number of logfiles to maintain default =3 */
    long lMaxlogFileSize;          /* max size of logfile before cycling defailt 200kb */

#if defined(__MVS__)
    MQLONG Hobj;
    char		szReplyQ[MQ_OBJECT_NAME_LENGTH+1];
#endif
#if defined (UNIX)
    int       iCurrentActiveConnections;
    key_t     KeySemaphores;
    key_t     KeyShmem;
    char      szQmgrDirectory[255];
    char      szQmgrPrefix[255];
#endif

#if defined (WIN32)
    int       iCurrentActiveConnections;
    int      lWinLog;
    HANDLE   hMutex;
    HANDLE   hMutexShm;
    char     szMutexKey[256];
    char     ExitPath[128];          /* where exit DLL are loaded from */
#endif

} workingStorage_type;

/*
* 22-03-2004 Sid Young Additional Defs for Enhanced Logging
*
* Add to Spec File specified in FN= parameter

 LogFormat=
 N	 - For name only or
 ND  - Name and Date or
 NDC - Name Date Channel format
 NCD - Name Channel Date format or any variation
 #
 # File Name to use (Default BlockIP2)
 #
 LogFileName=BlockLog
 LogExt=txt
 LogFormat=ND
 # example of use on Windows path
 #
 LogPath=\qml\log
 LogDrive=c:
 #
 # on Linux systems (LogDrive ignored)
 #
 LogPath=/var/spool/blockip2/logs
*/


/*
* Function Definitions
*/

int  wildcmp(char *wild, char *string, workingStorage_type *pWS);
int  wildcmplist(char *wild, char *string, workingStorage_type *pWS);
void LogData(int iMode, char * szData, workingStorage_type *pWS);
int  CheckInvalidUsers(PMQCD pChannelDefinition, workingStorage_type *pWS);
int  CheckBlankUser(PMQCD pChannelDefinition, workingStorage_type *pWS);
int  CheckUserId(PMQCD pChannelDefinition, workingStorage_type *pWS );
int  CheckBlockedUserId(PMQCD pChannelDefinition, workingStorage_type *pWS );
int  CheckConnectionPattern( PMQCD pChannelDefinition, workingStorage_type *pWS );
int  CheckCONList( MQCD *pChannelDefinition, MQLONG ExitReason, workingStorage_type *pWS);
int  CheckSSLList( MQCD *pChannelDefinition, workingStorage_type *pWS );
int  CopyUsers(char *p, workingStorage_type *pWS);


int  ProcessData(char *line, workingStorage_type *pWS );
int  ProcessFile( char *p, workingStorage_type *pWS);
int  InitialiseData(PMQCD pChannelDefinition, workingStorage_type *pWS, ws_type *pWS1, int ExitReason);
void i_strupr(char *p);
char * Strip (char * s);   /* 2.45 */
void translate (char *src, char *ny, char *old, workingStorage_type *pWS);
void translate2 (char *src, char *ny, char *old);
void WTODataE(char * szData, workingStorage_type *pWS);
void WTODataOk(char * szData, workingStorage_type *pWS);
long UpdateStatusQ(MQCD *pChannelDefinition,  long* what2do, workingStorage_type *pWS);
long deductStatusQ(MQCD *pChannelDefinition, workingStorage_type *pWS);

void nullDelimit(char[],char[],long);
long connectToQMgr(workingStorage_type *pWS);
int  CheckRunningChannels(MQCD *pChannelDefinition, workingStorage_type *pWS);
static void Xlate( PMQBYTE pData, MQLONG Length, MQLONG CharCode);
char * MQExitReason(PMQCXP pChannelExitParms);
char * MQExitId(PMQCXP pChannelExitParms);
char * MQChannelType(PMQCD pChannelDefinition);


/* start of inq chl defs */
#if !defined(__MVS__)
long bagSetup(MQHBAG*const,MQHBAG*const,MQHBAG*const, workingStorage_type *pWS);
long getActualCurrentNumberOfRunningChannels(long*, workingStorage_type *pWS);
long openStatusQ(workingStorage_type *pWS);
long closeStatusQ(workingStorage_type *pWS);
#endif

#if defined(__MVS__)
long chsSetup(workingStorage_type *pWS);
long getActualCurrentNumberOfRunningChannelszOS(long*, workingStorage_type *pWS);
long chsSendDisplayCHStatus(char * cmdq, char * replyq, workingStorage_type *pWS);
long closeReplyQ(workingStorage_type *pWS);
void GetZOSUserId(char *buffer);
#endif

#ifdef UNIX
int  LockAllSem(workingStorage_type *pWS);
int  FreeAllSem(workingStorage_type *pWS);
int  LockChannelSem(workingStorage_type *pWS);
int  FreeChannelSem(workingStorage_type *pWS);
int  FreeSpecSem(int sem_number, workingStorage_type *pWS);
int  LockSpecSem(int sem_number, workingStorage_type *pWS);
long  UpdateShrMem(MQCD *pChannelDefinition,  long* what2do, workingStorage_type *pWS);
int ProcessMQSiniFile(char *szQmgrName, workingStorage_type *pWS, ws_type *pWS1);
#endif


#if !defined (_REENTRANT)
char *strtok_r(char * s, const char * delim , char ** ptrptr );
#endif

#if defined (WIN32)
int  LockChannelSem(workingStorage_type *pWS);
int  FreeChannelSem(workingStorage_type *pWS);
long UpdateShrMem(MQCD *pChannelDefinition,  long* what2do, workingStorage_type *pWS);
void SendLogMessage (char *szData);
void OpenMTX(workingStorage_type *pWS, char *szLogFile);
int LoadRegistry(workingStorage_type *pWS);
void OpenMTXBlockIP2S(workingStorage_type *pWS, HANDLE *MutexBlockIP2S);
int LockBlockIP2S(workingStorage_type *pWS, HANDLE *MutexBlockIP2S);
int FreeBlockIP2S(workingStorage_type *pWS, HANDLE *MutexBlockIP2S);
#endif
void _Lsplitpath(const char* path, char* drv, char* dir, char* name, char* ext);
char * GenerateLogFileName(char * szFile, workingStorage_type *pWS);

#if defined (__EXT_LOG_FILENAME__)
long CycleFiles(workingStorage_type *pWS);
long getFileSize(char* szFileName, workingStorage_type *pWS);
long HandleCycleFiles(workingStorage_type *pWS);
#endif

/* ----------------------------------------------------------------------- */
/* -- global data -- */

/* simple, ascii/ebcdic conversion tables for queue manager name xlation */

/* ebcdic to ascii translation table */
static MQBYTE e2a[] =
    {
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, /*0*/
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, /*1*/
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, /*2*/
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, /*3*/
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
        0x20,0x00,0x00,0x00,0x40,0x00,0x00,0x00, /*4*/
        0x5c,0x00,0x00,0x2e,0x3c,0x28,0x2b,0x21,
        0x26,0x7b,0x00,0x00,0x7d,0x00,0x00,0x00, /*5*/
        0x00,0x00,0x00,0x24,0x2a,0x29,0x3b,0x5e,
        0x2d,0x2f,0x00,0x00,0x00,0x00,0x00,0x00, /*6*/
        0x00,0x00,0x00,0x2c,0x25,0x5f,0x3e,0x3f,
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, /*7*/
        0x00,0x00,0x3a,0x00,0x00,0x27,0x3d,0x22,
        0x00,0x61,0x62,0x63,0x64,0x65,0x66,0x67, /*8*/
        0x68,0x69,0x00,0x00,0x00,0x00,0x00,0x00,
        0x5b,0x6a,0x6b,0x6c,0x6d,0x6e,0x6f,0x70, /*9*/
        0x71,0x72,0x00,0x00,0x00,0x00,0x00,0x00,
        0x60,0x00,0x73,0x74,0x75,0x76,0x77,0x78, /*a*/
        0x79,0x7a,0x00,0x00,0x00,0x00,0x00,0x00,
        0x00,0x23,0x00,0x00,0x00,0x5d,0x00,0x00, /*b*/
        0x00,0x00,0x00,0x7c,0x00,0x7e,0x00,0x00,
        0x00,0x41,0x42,0x43,0x44,0x45,0x46,0x47, /*c*/
        0x48,0x49,0x00,0x00,0x00,0x00,0x00,0x00,
        0x00,0x4a,0x4b,0x4c,0x4d,0x4e,0x4f,0x50, /*d*/
        0x51,0x52,0x00,0x00,0x00,0x00,0x00,0x00,
        0x00,0x00,0x53,0x54,0x55,0x56,0x57,0x58, /*e*/
        0x59,0x5a,0x00,0x00,0x00,0x00,0x00,0x00,
        0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37, /*f*/
        0x38,0x39,0x00,0x00,0x00,0x00,0x00,0x00
    }; /* e2a */

/* ascii to ebcdic translation table */
static MQBYTE a2e[] =
    {
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, /*0*/
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, /*1*/
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
        0x40,0x4f,0x7f,0xb1,0x5b,0x6c,0x50,0x7d, /*2*/
        0x4d,0x5d,0x5c,0x4e,0x6b,0x60,0x4b,0x61,
        0xf0,0xf1,0xf2,0xf3,0xf4,0xf5,0xf6,0xf7, /*3*/
        0xf8,0xf9,0x7a,0x5e,0x4c,0x7e,0x6e,0x6f,
        0x44,0xc1,0xc2,0xc3,0xc4,0xc5,0xc6,0xc7, /*4*/
        0xc8,0xc9,0xd1,0xd2,0xd3,0xd4,0xd5,0xd6,
        0xd7,0xd8,0xd9,0xe2,0xe3,0xe4,0xe5,0xe6, /*5*/
        0xe7,0xe8,0xe9,0x90,0x48,0xb5,0x5f,0x6d,
        0xa0,0x81,0x82,0x83,0x84,0x85,0x86,0x87, /*6*/
        0x88,0x89,0x91,0x92,0x93,0x94,0x95,0x96,
        0x97,0x98,0x99,0xa2,0xa3,0xa4,0xa5,0xa6, /*7*/
        0xa7,0xa8,0xa9,0x51,0xbb,0x54,0xbd,0x00,
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, /*8*/
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, /*9*/
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, /*a*/
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, /*b*/
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, /*c*/
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, /*d*/
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, /*e*/
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, /*f*/
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
    }; /* a2e */

/* ----------------------------------------------------------------------- */
/*
 * Xlate - perform ascii/ebcdic translation depending on platform & char code
 *
 * FUNCTION:
 *    translate if
 *       on MVS (ebcdic) and current char set is ascii
 *    or
 *       not on MVS (ascii) and current char set is ebcdic
 *    else
 *       do nothing
 *
 * PARAMETERS:
 *    pData (PMQBYTE)   - input/output
 *       data to translate
 *
 *    Length (MQLONG)   - input
 *       length of data
 *
 *    CharCode (MQLONG) - input
 *       char set of incoming data
 *
 * RETURN:
 *    void
 *
 */

static void Xlate( PMQBYTE pData, MQLONG Length, MQLONG CharCode)
{

    /*?? query char set of current platform, rather than hard code it */

    /* see if, for current platform, the data needs converting */
    if( CharCode != CC_ASCII)
        /* on an ascii platform with ebcdic data */
    { /* need to do convertion */

        if( CharCode == CC_EBCDIC)
        { /* ebcdic to ascii */

            while(Length--)
            {
                *pData = (MQBYTE)e2a[*pData];
                ++pData;
            }
        }
        else
        { /* ascii to ebcdic */

            while(Length--)
            {
                *pData = (MQBYTE)a2e[*pData];
                ++pData;
            }
        }

        /* (assume data does not change in length during conversion) */

    } /* if - xlate needed */

} /* Xlate */


/**
* rlnts()
*
* Remove trailing spaces
*
*
* @return length, string has no trailing spaces
*/
static int rlnts(int l, char * s)
{
    while( l && s[l-1] == ' ') --l;
    return l;
}


/**
* InitialiseData()
*
*
* Setup required fields from Exit data and
* cleanup any lists.
*
*
* @pre	  Require exit parameters to be set by MQ
* @return int SEC_OK if if no failures else SEC_FAILURE
*/
int  InitialiseData(PMQCD pChannelDefinition, workingStorage_type *pWS, ws_type *pWS1, int ExitReason)
{
    int i;
    char *p;

    char Buffer[256];
    char szFile[256];
    long lFileSize;

    pWS->nDebugFlag=LOGDATA;

#if defined (WIN32)
    pWS->lWinLog=LOG_FILEONLY;
#endif

    /* Initialize working storage.. */
    strcpy(pWS->szStatusQ,"SYSTEM.ADMIN.BIP2.QUEUE");
    pWS->lMaxChannelActive=-1;
    memset(pWS->szQMgr,'\00',sizeof(pWS->szQMgr));
    memcpy(pWS->szQMgr,pChannelDefinition->QMgrName,rlnts(MQ_Q_MGR_NAME_LENGTH, pChannelDefinition->QMgrName));

    memset(pWS->szChannelName,'\00',sizeof(pWS->szChannelName));
    memcpy(pWS->szChannelName,pChannelDefinition->ChannelName,rlnts(MQ_CHANNEL_NAME_LENGTH, pChannelDefinition->ChannelName));

    pWS->nZOSconv=0;                  /* convert brackets on z/OS to handle national chars */
    pWS->bFileSizeCheck=1;            /* default is done=switched off to maintain transperacy with older releases */
    pWS->bFileSizeCheck=0;            /* We haven't played with fileSizeCHeck yet */
    pWS->lLogFileCount=3;             /* Number of logfiles to maintain default =3 */
    pWS->lMaxlogFileSize=200*1024;    /* max size of logfile before cycling defailt 200kb */

    pWS->bRespectMCA=0;               /* we don't respect MCAsettings */
    pWS->bAllowBlankUserID=0;         /* We will not accept blank uids */
    pWS->bBlockMqmUsers=0;
    pWS->bDebugMode=0;
    pWS->SilentMode=0;
    pWS->bLogByDate=0;
    pWS->bUseridUpperLowerCase=0;     /* be case sensitive */
    pWS->ReportLogFileError=0;        /* only report logfileerror once */
    pWS->lExpireValue=600*10;         /* let channel record live for 10 minutes (in secs) */
    pWS->lRefuseReason=0;             /* send refuse reason back to client exit.. */
    pWS->bInteractive=0;              /* is the client requesting feedback ? */

    pWS->SSLCount=0;
    pWS->CONCount = 0;

    strcpy(pWS->RemUID,"");           /* remote user identifier */
    strcpy(pWS->ConName,"");          /* Local copy of the connectionname */
    strcpy(pWS->Flags,"");
    strcpy(pWS->U2,"");
    strcpy(pWS->Users,"");            /* list of userids */
    strcpy(pWS->BlockUsers,"");	      /* list of Blocked userids */
    strcpy(pWS->Pattern,"");          /* The pattern to match against */
    strcpy(pWS->szClientUser,"");     /* user in the other end */

#if defined (UNIX)
    pWS->KeySemaphores=-1;
    pWS->KeyShmem=-1;
    strcpy(pWS->szQmgrPrefix,"");
    strcpy(pWS->szQmgrDirectory,"");
#endif

    /*
    * Setup logging information
    */
#if defined (__MVS__)
    /* NWC New defaults for MVS implementation */
    strcpy(pWS->szLogFileName,"//DD:SYSPRINT");
    strcpy(pWS->szLogFileExtension,"");
    strcpy(pWS->szLogDrive,"");
    pWS->nLogFormat=LOGNAME;
    strcpy(pWS->szLogChannel,"");
    strcpy(pWS->szLogPath,"");

#else /* not MVS */
    strcpy(pWS->szLogFileName,"BlockIP2");
    strcpy(pWS->szLogFileExtension,"log");
    strcpy(pWS->szLogDrive,"C:");
    pWS->nLogFormat=LOGNAME;
    strcpy(pWS->szLogChannel,"");
#if defined (WIN32)
    strcpy(pWS->szLogPath,"\\");
    pWS->hMutex = ERROR_INVALID_HANDLE;
    pWS->hMutexShm = ERROR_INVALID_HANDLE;
    strcpy(pWS->szMutexKey,"");
    strcpy(pWS->ExitPath,"");
    if (LoadRegistry(pWS) == 0) {
       memcpy(pWS->szLogDrive,pWS->ExitPath,2);
       memcpy(pWS->szLogPath,pWS->ExitPath+2,strlen(pWS->ExitPath)-1);  /* take '\00' too */
       }

#else	/* should be unix file system */
    strcpy(pWS->szLogPath,"/var/mqm/exits/");
#endif
#endif

#if MQAT_DEFAULT==MQAT_OS400
    strcpy(pWS->szLogFileName,"BLOCKIP2");
    strcpy(pWS->szLogFileExtension,"LOG");
    strcpy(pWS->szLogPath,"");
    strcpy(pWS->szLogDrive,"");
#endif

    switch(ExitReason)
    {
    case MQXR_INIT:
        pWS1->bFirstTime=1;
        pWS1->lMaxChannelActive=pWS->lMaxChannelActive;
        break;
    default:
        pWS->lMaxChannelActive=pWS1->lMaxChannelActive;
#if defined (UNIX)
        strcpy(pWS->szQmgrDirectory,pWS1->szQmgrDirectory);
        strcpy(pWS->szQmgrPrefix,pWS1->szQmgrPrefix);
#endif
    }

    /*
    * Empty CONList and SSList tables
    */
    for(i=0; i < MAX_SSL; i++)
        strcpy(pWS->SSLList[i], "");
    for(i=0; i < MAX_CON; i++)
        strcpy(pWS->CONList[i], "");

    pWS->CONCount = 0;
    pWS->SSLCount = 0;
    pWS->RemUID[0]=0x00;			/* remote user identifier 		  */
    pWS->ConName[0]= 0x00;		/* Local copy of the connectionname */
    pWS->szLogChannel[0]=0x00;
    pWS->bRespectMCA = 0;
    strncpy(pWS->ConName,pChannelDefinition->ConnectionName,MQ_CONN_NAME_LENGTH);/*	only take the first 32 bytes ! */
    for (i=0; i<MQ_CONN_NAME_LENGTH; i++)
    {
        if(pWS->ConName[i] == ' ')
        {
            pWS->ConName[i] = 0x00;
            break;
        }
    }

    strncpy(pWS->RemUID,pChannelDefinition->RemoteUserIdentifier,MQ_USER_ID_LENGTH);
    pWS->RemUID[MQ_USER_ID_LENGTH]=0x00;			/* force last char */

    for(i=0; i<MQ_USER_ID_LENGTH; i++)
    {
        if(pWS->RemUID[i] == ' ')
        {
            pWS->RemUID[i] = 0x00;
            break;
        }
    }

    strncpy(pWS->Pattern,pChannelDefinition->SecurityUserData,MQ_EXIT_DATA_LENGTH);
    pWS->Pattern[MQ_EXIT_DATA_LENGTH] = 0x00;	/* force end of string */
    for (i=0; i<MQ_EXIT_DATA_LENGTH; i++)
    {
        if(pWS->Pattern[i] == ' ')
        {
            pWS->Pattern[i] = 0x00;
            break;
        }
    }

    strncpy(pWS->szLogChannel, pChannelDefinition->ChannelName,MQ_CHANNEL_NAME_LENGTH);
    pWS->szLogChannel[MQ_CHANNEL_NAME_LENGTH]=0x00;
    for (i=0; i<MQ_CHANNEL_NAME_LENGTH; i++)
    {
        if(pWS->szLogChannel[i] == ' ')
        {
            pWS->szLogChannel[i] = 0x00;
            break;
        }
    }

#if defined (WIN32)
    p = strstr(pWS->Pattern,"-w");
    if( p != 0 ) pWS->lWinLog=LOG_EVTLOGONLY;
#endif
    /*
    * Process some debugging parameters first
    */
    p = strstr(pWS->Pattern,"-d");
    if( p != 0 ) pWS->nDebugFlag=LOGDEBUG;

    /* start JHP 040330 */
    p = strstr(pWS->Pattern,"FN=");	/* Filename specified .. */
    if( p == NULL )
    {
        /* check pattern syntax JHP 040805 */
        if (*pWS->Pattern != '*' && *pWS->Pattern != '?' && (*pWS->Pattern > '9' || *pWS->Pattern < '0' ))
        {
            sprintf(pWS->szTmpBufff,"Errors SCYDATA specification - Connection refused for Channel [%.*s]",
                    rlnts(MQ_CHANNEL_NAME_LENGTH, pChannelDefinition->ChannelName),
                    pChannelDefinition->ChannelName);
            LogData( LOGDATA, pWS->szTmpBufff, pWS);
            WTODataE("BLOCKIP-01E", pWS);

            return SEC_FAILURE;
        }
        p = strstr(pWS->Pattern,"-n");
        if( p != 0 ) pWS->bBlockMqmUsers=1;

        p = strstr(pWS->Pattern,"+b");
        if( p != 0 ) pWS->bAllowBlankUserID=1;

        p = strstr(pWS->Pattern,"userids=");
        if( p != 0 )
        {
            if( CopyUsers(p,pWS)==SEC_FAILURE)
            {
                return SEC_FAILURE;
            }
        }

    }

    return SEC_OK;
    /* END JHP 040330 */
}

void i_strupr(char *p)
{
    while (*p)
        *(p++) = toupper(*p);
}

/*#if defined (WIN32)*/
#if !defined (_REENTRANT)
char *strtok_r(char * s, const char * delim , char ** ptrptr )
{
    return(strtok(s, delim));
}
#endif

/*
* BlockExit - Channel security exit, entry point
*
* PARAMETERS:
*	 pChannelExitParams (PMQCXP)  - input/output
*		Channel exit parameter block
*
*	 pChannelDefinition (PMQCD)   - input/output
*		Channel definition
*
*	 pDataLength		(PMQLONG) - input/output
*		On input:  length of AgentBuffer
*		On output: length of returned data (AgentBuff/ErrorBuff)
*
*	 pAgentBufferLength (PMQLONG) - input
*		Agent Buffer length
*
*	 AgentBuffer		(PMQBYTE) - input/output
*		Agent buffer
*
*	 pExitBufferLength	(PMQLONG) - input/output
*		Exit buffer length
*
*	 pExitBufferAddr	(PMQPTR)  - input/output
*		Exit buffer
*
* RETURN:
*	 void
*
* NOTES:
*
*	 For more details on the operation of security exits, see:
*		'MQSeries - Distributed Queue Management Guide'
*
* --------------------------------------------------------------------------
* This security exit does NOT perform any security validation, it only solves
* is to show how to log connections made to a particular channel.
* --------------------------------------------------------------------------
*
*/
/**
*  BlockExit()
*
*
* main entry point for security exit
*
*
* @return void
*/

#if MQAT_DEFAULT==MQAT_OS400
int main (int argc, char * argv[])

#else

/* NWC MVS linkage requirements added */
#if defined(__MVS__)
/* Define code CSECT name */
#pragma csect(code,"BLOCK2")
/* Define static CSECT name */
#pragma csect(static,"BLOCK2D")

#pragma runopts(trap(off),PLIST(MVS))

#pragma environment(BLOCKIP2)

int BLOCKIP2 (
    PMQCXP	 pChannelExitParams,
    PMQCD 	 pChannelDefinition,
    PMQLONG	 pDataLength,
    PMQLONG	 pAgentBufferLength,
    PMQBYTE	 AgentBuffer,
    PMQLONG	 pExitBufferLength,
    PMQPTR	 pExitBufferAddr)

/*  MVS */
#else
/*
* dummy function used as entry point to exit, only needed for *IX boxes
*/

extern void MQENTRY MQStart(void) {;}

#if defined (WIN32)

__declspec (dllexport) void MQENTRY BlockExit (
    PMQCXP	  pChannelExitParams,
    PMQCD	  pChannelDefinition,
    PMQLONG	  pDataLength,
    PMQLONG	  pAgentBufferLength,
    PMQBYTE	  AgentBuffer,
    PMQLONG	  pExitBufferLength,
    PMQPTR	  pExitBufferAddr);
#endif

extern void MQENTRY BlockExit(
        PMQCXP	 pChannelExitParams,
        PMQCD 	 pChannelDefinition,
        PMQLONG	 pDataLength,
        PMQLONG	 pAgentBufferLength,
        PMQBYTE	 AgentBuffer,
        PMQLONG	 pExitBufferLength,
        PMQPTR	 pExitBufferAddr)

    /* Not MVS */
#endif
/* not OS400 */
#endif
{
    int i;
    char *p;

    workingStorage_type ws;
    workingStorage_type *pWS;

    ws_type ** ppExitUserArea = (ws_type**) &pChannelExitParams->ExitUserArea;
    ws_type *  pExitUserArea  = *ppExitUserArea;

#if MQAT_DEFAULT==MQAT_OS400
    PMQCXP	 pChannelExitParams;   /* Channel Exit Parameter pointer */
    PMQCD 	 pChannelDefinition;   /* Channel Definition pointer     */
    PMQLONG	 pDataLength;          /* Data Length                    */
    PMQLONG	 pAgentBufferLength;   /* Agent Buffer Length            */
    PMQBYTE	 AgentBuffer;          /* Agent Buffer Pointer           */
    PMQLONG	 pExitBufferLength;    /* Exit Buffer Length             */
    PMQPTR	 pExitBufferAddr;      /* Exit Buffer Address Pointer    */

    if (argc > 8)
    {
        fprintf(stdout,
                "\nBlockIP2 - ERROR wrong number of parameters\n");
    }
    /* Assign values on argv[] to the channel exit parameter list
       as defined in the WebSphere MQ Intercommunications manual  */
    pChannelExitParams  = (PMQCXP)   argv[1];
    pChannelDefinition  = (PMQCD)    argv[2];
    pDataLength         = (PMQLONG)  argv[3];
    pAgentBufferLength  = (PMQLONG)  argv[4];
    AgentBuffer         = (PMQBYTE)  argv[5];
    pExitBufferLength   = (PMQLONG)  argv[6];
    pExitBufferAddr     = (PMQPTR)   argv[7];
#endif

    pExitBufferAddr   = pExitBufferAddr;
    pExitBufferLength = pExitBufferLength;
    pWS=&ws;

    sprintf(pWS->szWTOBuffErr,"Connection refused CHL=%.*s CONN=%.*s USER=%.*s",
            rlnts(MQ_CHANNEL_NAME_LENGTH, pChannelDefinition->ChannelName),
            pChannelDefinition->ChannelName,
            rlnts(MQ_CONN_NAME_LENGTH, pWS->ConName),
            pWS->ConName,
            rlnts(MQ_USER_ID_LENGTH, pChannelDefinition->RemoteUserIdentifier),
            pChannelDefinition->RemoteUserIdentifier);
    sprintf(pWS->szWTOBuffOk,"Connection accepted CHL=%.*s CONN=%.*s USER=%.*s",
            rlnts(MQ_CHANNEL_NAME_LENGTH, pChannelDefinition->ChannelName),
            pChannelDefinition->ChannelName,
            rlnts(MQ_CONN_NAME_LENGTH, pWS->ConName),
            pWS->ConName,
            rlnts(MQ_USER_ID_LENGTH, pChannelDefinition->RemoteUserIdentifier),
            pChannelDefinition->RemoteUserIdentifier);

    /*
    * check if security exit has been invoked
    */
    if(pChannelExitParams->ExitId!=MQXT_CHANNEL_SEC_EXIT)
    {
        pChannelExitParams->ExitResponse = MQXCC_SUPPRESS_FUNCTION;
        sprintf(pWS->szTmpBufff,"ERROR!! - Unexpected invocation.  ExitId=[%ld]",   pChannelExitParams->ExitId);
        LogData(LOGDATA,pWS->szTmpBufff, pWS);
#if MQAT_DEFAULT==MQAT_OS400
        return 0;
#else
        return;
#endif
    }
    if (pChannelExitParams->ExitReason==MQXR_INIT) {
        /*       LogData(LOGDEBUG,"Before calloc", pWS);*/

#if !defined(__MVS__)
        pExitUserArea = (ws_type *)calloc( 1, sizeof(ws_type) );
#else
        pExitUserArea = (ws_type *)__4kmalc( sizeof(ws_type) );
#endif
        if (pExitUserArea)
        {
            LogData(LOGDEBUG,"after calloc", pWS);
            *ppExitUserArea = pExitUserArea;

            pExitUserArea->bFirstTime=0;

            LogData(LOGDEBUG,"Calloc success", pWS);

        }
        else
        {
            pChannelExitParams->ExitResponse = MQXCC_SUPPRESS_FUNCTION;
            LogData(LOGDATA,"Cmalloc failed", pWS);
            return;
        }
    }
    else
    {

        /*    sprintf(pWS->szTmpBufff,"szLogFileName [%s]",pExitUserArea->szLogFileName);
            LogData(LOGDATA,pWS->szTmpBufff, pWS);
        */
    }

    /*
    * initialize some structures
    */
    i=pWS->lMaxChannelActive;
    if (InitialiseData(pChannelDefinition, pWS, pExitUserArea, pChannelExitParams->ExitReason)==SEC_FAILURE)
    {
        if (pChannelExitParams->ExitReason == MQXR_TERM)		  /* JHP 0403030 */
            if (pExitUserArea->lMaxChannelActive!=-1) {                       /* Be ready to store shm info here */
                if (deductStatusQ(pChannelDefinition, pWS) == 0)
                {}
            }
        LogData( LOGDEBUG, "======= TERM ======", pWS);
        free( pExitUserArea );
        ppExitUserArea = 0;
        LogData(LOGDEBUG,"Free success", pWS);
        pChannelExitParams->ExitResponse  = MQXCC_OK;      /* Allways ok on MQXR_TERM JHP 060214 */
#if MQAT_DEFAULT==MQAT_OS400
        return 0;
#else
        return;
#endif
    }


#if defined UNIX
    /* functions added under 2.45 */
    if (pChannelExitParams->ExitReason == MQXR_INIT) {
        if (ProcessMQSiniFile(pWS->szQMgr, pWS, pExitUserArea) !=SEC_OK) {
            pChannelExitParams->ExitResponse = MQXCC_SUPPRESS_FUNCTION;
            LogData(LOGDEBUG,"ProcessMQSiniFile() requested connection broken.", pWS);
            return;
        }
    }
#endif


/*    sprintf( pWS->szTmpBufff,"lMaxChannelActive was %ld and is now %ld ",
             i, pWS->lMaxChannelActive);
    LogData( LOGDEBUG, pWS->szTmpBufff, pWS);*/

    /*
    *  Only accept z/os option on z/OS implementation
    */
#if defined(__MVS__)
    p = strstr(pWS->Pattern,"-z");
    if( p != 0 ) pWS->nZOSconv=1;
#endif
    p = strstr(pWS->Pattern,"-q");
    if( p != 0 ) pWS->SilentMode=1;

    p = strstr(pWS->Pattern,"FN=");	/* Filename specified .. */
    if( p != NULL )
    {
        p=p+3;
        if(ProcessFile(p, pWS)==SEC_FAILURE)
        {
            sprintf(pWS->szTmpBufff,"Errors in parameter file - Connection refused for Channel [%.*s]",
                    rlnts(MQ_CHANNEL_NAME_LENGTH, pChannelDefinition->ChannelName),
                    pChannelDefinition->ChannelName);
            LogData( LOGDATA, pWS->szTmpBufff, pWS);
            WTODataE("BLOCKIP-02E", pWS);
            pChannelExitParams->ExitResponse = MQXCC_SUPPRESS_FUNCTION; /* JHP 040329 */
#if MQAT_DEFAULT==MQAT_OS400
            return 0;
#else
            return;
#endif
        }
        pExitUserArea->lMaxChannelActive=pWS->lMaxChannelActive;               /* Update flag */
    }


    /*
    * set default exit responses at this point
    */
    pChannelExitParams->ExitResponse  = MQXCC_OK;
    if (pChannelExitParams->ExitReason == MQXR_TERM)		  /* JHP 0403030 */
    {
        LogData( LOGDEBUG, "======= TERM ======", pWS);
        if (pExitUserArea->lMaxChannelActive!=-1) {                       /* Be ready to store shm info here */
            if (deductStatusQ(pChannelDefinition, pWS) == 0)
            {}
        }

        if (pWS->lMaxChannelActive != -1)       /* Max active channels was specified, check it.. */
        {
            /*           if (deductStatusQ(pChannelDefinition, pWS) == 0)
                       {}*/
        }
        LogData(LOGDEBUG,"Before Free", pWS);
        free( pExitUserArea );
        ppExitUserArea = 0;
        LogData(LOGDEBUG,"Free success", pWS);
#if MQAT_DEFAULT==MQAT_OS400
        return 0;
#else
        return;
#endif
    }

/* begin 2.46 */
#if defined (__EXT_LOG_FILENAME__)
#if defined (WIN32)
    if (pWS->lWinLog!=LOG_EVTLOGONLY) {
#endif
       if (pWS->bFileSizeCheck==0) {
           HandleCycleFiles(pWS);
           pWS->bFileSizeCheck=1;
       }
#if defined (WIN32)
    }
#endif
#endif   /* __EXT_LOG_FILENAME__ */
/* end 2.46 */


    /*
    * switch on ExitReason to see why exit was invoked
    */
    switch(pChannelExitParams->ExitReason)
    {
    case MQXR_INIT: 	   /* Initialization (all exits) */
        LogData( LOGDEBUG, "======= INIT ======", pWS);
        /*#if !defined(__MVS__)*/
        sprintf( pWS->szTmpBufff,"lMaxChannelActive is now %ld (before channel check) ",
                 pWS->lMaxChannelActive);
        LogData( LOGDEBUG, pWS->szTmpBufff, pWS);
        if (pWS->lMaxChannelActive != -1)       /* Max active channels was specified, check it.. */
        {
            if (CheckRunningChannels(pChannelDefinition, pWS) != SEC_OK)
            {
                LogData( LOGDEBUG, "CheckRunningChannels - refused...", pWS);
                WTODataE("BLOCKIP-32E", pWS);
                pChannelExitParams->ExitResponse = MQXCC_SUPPRESS_FUNCTION;
#if MQAT_DEFAULT==MQAT_OS400
                return 0;
#else
                return;
#endif
            }
        }
        /*#endif*/
        break;

    case MQXR_TERM: 	   /* Termination  (all exits) */
        LogData( LOGDEBUG, "======= TERM ======", pWS);
        free( pExitUserArea );
        ppExitUserArea = 0;
        LogData(LOGDEBUG,"Free success", pWS);
        break;

#ifdef MQXR_SEC_PARMS
    case MQXR_SEC_PARMS: 	   /* Security message exchange  (SCY exits) */
        LogData( LOGDEBUG, "======= SEC_PARMS ======", pWS);
        break;
        /* we handle both INIT_SEC and SEC_MSG the same way to keep connecting clients with security exits secure */
#endif

    case MQXR_INIT_SEC:    /* Initiate security exchange (SCTY exits only) */
    case MQXR_SEC_MSG:	   /* Init security message  */
        if(pChannelExitParams->ExitReason==MQXR_INIT_SEC)
        {
            LogData( LOGDEBUG, "======= Start INIT_SEC ======", pWS);
        }
        else
        {
            LogData( LOGDEBUG, "======= Start SEC_MSG ======", pWS);
        }
        /*
        * this stmt is placed here to help testing.
        */
        /*
        printf( "BlockExit QMgr=[%.*s] ChannelName=[%.*s] ConnName=[%.*s] Uid=[%.*s]\n",
        rlnts(MQ_Q_MGR_NAME_LENGTH, pChannelDefinition->QMgrName),
        pChannelDefinition->QMgrName,
        rlnts(MQ_CHANNEL_NAME_LENGTH, pChannelDefinition->ChannelName),
        pChannelDefinition->ChannelName,
        rlnts(MQ_CONN_NAME_LENGTH, pChannelDefinition->ConnectionName),
        pChannelDefinition->ConnectionName,
        rlnts(MQ_USER_ID_LENGTH, pChannelDefinition->RemoteUserIdentifier),
        pChannelDefinition->RemoteUserIdentifier);
        */

        /*
        * Record debug info
        */
        if(pWS->SilentMode==0)
        {
            /*			sprintf( pWS->szTmpBufff,"ver=%s env=%s ExitId=%ld ExitReason=%ld ChannelType=%ld",
            				VERSION, ENV,
            				pChannelExitParams->ExitId,
            				pChannelExitParams->ExitReason,
            				pChannelDefinition->ChannelType);
            			LogData( LOGDEBUG, pWS->szTmpBufff);*/

            sprintf( pWS->szTmpBufff,"ver=%s env=%s ExitId=%s ExitReason=%s ChannelType=%s",
                     VERSION, ENV,
                     MQExitId(pChannelExitParams),
                     MQExitReason(pChannelExitParams),
                     MQChannelType(pChannelDefinition));
            LogData( LOGDEBUG, pWS->szTmpBufff, pWS);

            sprintf(pWS->szTmpBufff, "BlockExit QMgr=[%.*s] ChannelName=[%.*s] ConnName=[%.*s] Uid=[%.*s] pDataLength=%ld",
                    rlnts(MQ_Q_MGR_NAME_LENGTH, pChannelDefinition->QMgrName),
                    pChannelDefinition->QMgrName,
                    rlnts(MQ_CHANNEL_NAME_LENGTH, pChannelDefinition->ChannelName),
                    pChannelDefinition->ChannelName,
                    rlnts(MQ_CONN_NAME_LENGTH, pChannelDefinition->ConnectionName),
                    pChannelDefinition->ConnectionName,
                    rlnts(MQ_USER_ID_LENGTH, pChannelDefinition->RemoteUserIdentifier),
                    pChannelDefinition->RemoteUserIdentifier,
                    *pDataLength);
            LogData( LOGDEBUG, pWS->szTmpBufff, pWS);


#ifdef MQCD_VERSION_7										  /* for compilers */
            if(pChannelDefinition->Version == MQCD_VERSION_7)
            {  /* version 5.3 */
                if(pChannelDefinition->SSLPeerNameLength > 0 )
                {  /* eval in two.. */
                    sprintf(pWS->szTmpBufff,"QMgr=[%.*s] Channel=[%.*s] SSLPeerName=[%.*s] SSLCipherSpec=[%.*s] SSLClientAuth=[%ld]",
                            rlnts(MQ_Q_MGR_NAME_LENGTH, pChannelDefinition->QMgrName),
                            pChannelDefinition->QMgrName,
                            rlnts(MQ_CHANNEL_NAME_LENGTH, pChannelDefinition->ChannelName),
                            pChannelDefinition->ChannelName,
                            pChannelDefinition->SSLPeerNameLength,
                            pChannelDefinition->SSLPeerNamePtr,
                            rlnts(MQ_SSL_CIPHER_SPEC_LENGTH, pChannelDefinition->SSLCipherSpec),
                            pChannelDefinition->SSLCipherSpec,
                            pChannelDefinition->SSLClientAuth);
                    LogData( LOGDEBUG, pWS->szTmpBufff, pWS);
                }
            }
#endif
        }
        /*
        * Deny all connections automatically
        */
        pChannelExitParams->ExitResponse = MQXCC_SUPPRESS_FUNCTION;
        pChannelExitParams->ExitResponse2 = MQXR2_USE_AGENT_BUFFER;

        if (pWS->bUseridUpperLowerCase==1)		   /* jhp 2.12 */
        {
            sprintf(pWS->szTmpBufff,"RemUID Before strupr [%s]",pWS->RemUID);
            LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
            i_strupr(pWS->RemUID);
            sprintf(pWS->szTmpBufff,"RemUID After strupr [%s]",pWS->RemUID);
            LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
        }


        /*
        * If FN specified - parse file or parse exit data
        */
        p = strstr(pWS->Pattern,"FN=");	/* Filename specified .. */
        if( p != NULL )
        {
            p=p+3;
            if(ProcessFile(p, pWS)==SEC_FAILURE)
            {
                sprintf(pWS->szTmpBufff,"Errors in parameter file - Connection refused");
                LogData( LOGDEBUG, pWS->szTmpBufff, pWS);
            }
        }
        /*
        * this part is removed to allow compliance with the previous version of the exit
        */
        /*						  else
        if (ProcessData(Pattern)==SEC_FAILURE)
        {
        sprintf(pWS->szTmpBufff,"Errors in parameters - Connection refused");
        LogData( LOGDEBUG, pWS->szTmpBufff);
        						};*/
        /*								  ProcessData(Pattern);*/
        /*
        * file parsed OK (or was not specified) - cleanup "Pattern" data
        */
        for (i=0; i<MAX_PL; i++)
        {
            if(pWS->Pattern[i] == ',')
            {
                pWS->Pattern[i] = ';';
            }
            if(pWS->Pattern[i] == ' ')
            {
                pWS->Pattern[i] = 0x00;
                break;
            }
        }
        if(pWS->Pattern[0] != 0)
        {
            if(CheckConnectionPattern(pChannelDefinition, pWS)==SEC_FAILURE)
            {
                LogData(LOGDEBUG,"Connection failure!", pWS);
#if MQAT_DEFAULT==MQAT_OS400
                return 0;
#else
                return;
#endif
            }
        }
        /*
        * Copy and report on user list extracted from file
        */
        strcpy(pWS->U2,pWS->Users);
        sprintf(pWS->szTmpBufff,"Users: [%s] len [%d]", pWS->U2,strlen(pWS->U2));
        LogData( LOGDEBUG, pWS->szTmpBufff, pWS);
        /*
        * If a Blocked user list was given, we must check it now
        */
        if(strlen(pWS->BlockUsers) > 0)
        {
            if(CheckBlockedUserId(pChannelDefinition, pWS)==SEC_FAILURE)
            {
                sprintf(pWS->szTmpBufff,"Connection refused, Channel [%.*s] ConName [%.*s] User [%.*s] is in negative list [%s]",
                        rlnts(MQ_CHANNEL_NAME_LENGTH, pChannelDefinition->ChannelName),
                        pChannelDefinition->ChannelName,
                        rlnts(MQ_CONN_NAME_LENGTH, pWS->ConName),	/* NWC */
                        pWS->ConName,
                        rlnts(MQ_USER_ID_LENGTH, pChannelDefinition->RemoteUserIdentifier),
                        pChannelDefinition->RemoteUserIdentifier,
                        pWS->BlockUsers);
                LogData( LOGDATA, pWS->szTmpBufff, pWS );
                WTODataE("BLOCKIP-03E", pWS);
#if MQAT_DEFAULT==MQAT_OS400
                return 0;
#else
                return;
#endif
            }
        }
        /*
        * If a user list was given, we must check it now
        */
        if(strlen(pWS->Users) > 0)
        {
            if(CheckUserId(pChannelDefinition, pWS)==SEC_FAILURE)
            {
                sprintf(pWS->szTmpBufff,"Connection refused, Channel [%.*s] ConName [%.*s] User [%.*s] not in positive list [%s]",
                        rlnts(MQ_CHANNEL_NAME_LENGTH, pChannelDefinition->ChannelName),
                        pChannelDefinition->ChannelName,
                        rlnts(MQ_CONN_NAME_LENGTH, pWS->ConName),	/* NWC */
                        pWS->ConName,
                        rlnts(MQ_USER_ID_LENGTH, pChannelDefinition->RemoteUserIdentifier),
                        pChannelDefinition->RemoteUserIdentifier,
                        pWS->Users);
                LogData( LOGDATA, pWS->szTmpBufff, pWS );
                WTODataE("BLOCKIP-04E", pWS);
#if MQAT_DEFAULT==MQAT_OS400
                return 0;
#else
                return;
#endif
            }
        }
        /*
        * Check the CON= command, if not specified we return safely
        */
        if(CheckCONList(pChannelDefinition,pChannelExitParams->ExitReason,pWS) == SEC_FAILURE)
        {
            sprintf(pWS->szTmpBufff,"Connection refused, Channel [%.*s] ConName [%.*s] User [%.*s] was not accepted in CON=",
                    rlnts(MQ_CHANNEL_NAME_LENGTH, pChannelDefinition->ChannelName),
                    pChannelDefinition->ChannelName,
                    rlnts(MQ_CONN_NAME_LENGTH, pWS->ConName),
                    pWS->ConName,
                    rlnts(MQ_USER_ID_LENGTH, pChannelDefinition->RemoteUserIdentifier),
                    pChannelDefinition->RemoteUserIdentifier);
            LogData( LOGDATA, pWS->szTmpBufff,pWS );
            WTODataE("BLOCKIP-05E",pWS);
#if MQAT_DEFAULT==MQAT_OS400
            return 0;
#else
            return;
#endif
        }

#ifdef MQCD_VERSION_7
        /*
        * Check the SSL= command, if not specified we return safely
        */

        if( CheckSSLList(pChannelDefinition, pWS)==SEC_FAILURE)
        {
            sprintf(pWS->szTmpBufff,"Connection refused, Channel [%.*s] ConName [%.*s] User [%.*s] SSLPeer [%s] requested by CheckSSLList()",
                    rlnts(MQ_CHANNEL_NAME_LENGTH, pChannelDefinition->ChannelName),
                    pChannelDefinition->ChannelName,
                    rlnts(MQ_CONN_NAME_LENGTH, pWS->ConName),	/* NWC */
                    pWS->ConName,
                    rlnts(MQ_USER_ID_LENGTH, pChannelDefinition->RemoteUserIdentifier),
                    pChannelDefinition->RemoteUserIdentifier,
                    pWS->SSLPeer);
            LogData( LOGDATA, pWS->szTmpBufff, pWS );
            WTODataE("BLOCKIP-06E", pWS);
#if MQAT_DEFAULT==MQAT_OS400
            return 0;
#else
            return;
#endif
        }
#endif


        /*
        * Now check for excluded user account names
        */
        if	(pWS->bAllowBlankUserID==0)
        {
            if(CheckBlankUser(pChannelDefinition, pWS)==SEC_FAILURE)
            {
                sprintf(pWS->szTmpBufff,"Connection refused, Channel [%.*s] ConName [%.*s] User [%.*s]",
                        rlnts(MQ_CHANNEL_NAME_LENGTH, pChannelDefinition->ChannelName),
                        pChannelDefinition->ChannelName,
                        rlnts(MQ_CONN_NAME_LENGTH, pWS->ConName),	/* NWC */
                        pWS->ConName,
                        rlnts(MQ_USER_ID_LENGTH, pChannelDefinition->RemoteUserIdentifier),
                        pChannelDefinition->RemoteUserIdentifier);
                LogData( LOGDATA, pWS->szTmpBufff, pWS );
#if MQAT_DEFAULT==MQAT_OS400
                return 0;
#else
                return;
#endif
            }
        }
        if	(pWS->bBlockMqmUsers==1)
        {
            if(CheckInvalidUsers(pChannelDefinition, pWS)==SEC_FAILURE)
            {
                sprintf(pWS->szTmpBufff,"Connection refused, Channel [%.*s] ConName [%.*s] User [%.*s]",
                        rlnts(MQ_CHANNEL_NAME_LENGTH, pChannelDefinition->ChannelName),
                        pChannelDefinition->ChannelName,
                        rlnts(MQ_CONN_NAME_LENGTH, pWS->ConName),	/* NWC */
                        pWS->ConName,
                        rlnts(MQ_USER_ID_LENGTH, pChannelDefinition->RemoteUserIdentifier),
                        pChannelDefinition->RemoteUserIdentifier);
                LogData( LOGDATA, pWS->szTmpBufff, pWS );
#if MQAT_DEFAULT==MQAT_OS400
                return 0;
#else
                return;
#endif
            }
        }
        strcpy(pWS->Flags,"");
        if(pWS->bBlockMqmUsers==1) strcpy(pWS->Flags,"BlockMqmUsers=Y ");
        if(pWS->bAllowBlankUserID==1) strcat(pWS->Flags,"AllowBlankUserID=Y ");
        if(pWS->bUseridUpperLowerCase==1) strcat(pWS->Flags,"UseridUpperLowerCase=* ");

        if(pWS->SilentMode==0)
        {
            sprintf(pWS->szTmpBufff,"Connection accepted, Channel [%.*s] ConName [%.*s] Pattern [%.*s] Flags [%s] User [%s]",
                    rlnts(MQ_CHANNEL_NAME_LENGTH, pChannelDefinition->ChannelName),
                    pChannelDefinition->ChannelName,
                    rlnts(MQ_CONN_NAME_LENGTH, pWS->ConName),	/* NWC */
                    pWS->ConName,
                    rlnts(MQ_EXIT_DATA_LENGTH,pWS->Pattern),
                    pWS->Pattern,
                    pWS->Flags,
                    pWS->RemUID);
            LogData(LOGDATA, pWS->szTmpBufff, pWS);
            WTODataOk("BLOCKIP-50I", pWS);
        }
        /*
        * If we get here then all required parameters were correctly met - so set exit to OK
        */
        /*
        sprintf(pWS->szTmpBufff,"debug: Connection accepted QMgr=[%.*s] Channel=[%.*s] ConName=[%.*s] Uid=[%.*s]",
        	rlnts(MQ_Q_MGR_NAME_LENGTH, pChannelDefinition->QMgrName),
        	pChannelDefinition->QMgrName,
        	rlnts(MQ_CHANNEL_NAME_LENGTH, pChannelDefinition->ChannelName),
        	pChannelDefinition->ChannelName,
        	rlnts(MQ_CONN_NAME_LENGTH, pChannelDefinition->ConnectionName),
        	pChannelDefinition->ConnectionName,
        	rlnts(MQ_USER_ID_LENGTH, pChannelDefinition->RemoteUserIdentifier),
        	pChannelDefinition->RemoteUserIdentifier);
        LogData(LOGDEBUG, pWS->szTmpBufff);
        */

        pChannelExitParams->ExitResponse  = MQXCC_OK;
        sprintf( pWS->szTmpBufff,"Return status Exitresponse=%ld",
                 pChannelExitParams->ExitResponse);
        LogData( LOGDEBUG, pWS->szTmpBufff, pWS);
#if MQAT_DEFAULT==MQAT_OS400
        return 0;
#else
        return;
#endif
        break;
        /*
        * Unsupported exit reason - abort security exchange
        */
    default:
        pChannelExitParams->ExitResponse = MQXCC_SUPPRESS_FUNCTION;
        LogData(LOGDEBUG,"error - unsupported exit reason", pWS);
        sprintf( pWS->szTmpBufff,"ver=%s env=%s ExitId=%ld ExitReason=%ld ChannelType=%ld",
                 VERSION, ENV,
                 pChannelExitParams->ExitId,
                 pChannelExitParams->ExitReason,
                 pChannelDefinition->ChannelType);
        LogData( LOGDEBUG, pWS->szTmpBufff, pWS);
#if MQAT_DEFAULT==MQAT_OS400
        return 0;
#else
        return;
#endif
        break;
    }  /* switch - ExitReason */
    sprintf( pWS->szTmpBufff,"Return status Exitresponse=%ld",
             pChannelExitParams->ExitResponse);
    LogData( LOGDEBUG, pWS->szTmpBufff, pWS);
#if MQAT_DEFAULT==MQAT_OS400
    return 0;
#else
    return;
#endif
} /* BlockExit */




/**
* CheckUserId()
*
*
* Check List of Users to see if RemUID is among them
*
* @pre	List of users already filled
* @return int SEC_OK if user found else SEC_FAILURE
*/
int CheckUserId(PMQCD pChannelDefinition, workingStorage_type *pWS )
{
    char *p;
    char *u;
    char *strtok_cb;
    char szUserList[33];
    int i,j;
    /*
    * Scan through list of user ID's
    */

    LogData(LOGDEBUG,"CheckUserId()", pWS);

    p = pWS->U2;
    u = (char*)strtok_r(pWS->U2 , "," , &strtok_cb);

    sprintf(pWS->szTmpBufff,"Users=[%s] first u=[%s]", pWS->U2,u);
    LogData(LOGDEBUG, pWS->szTmpBufff, pWS);

    memset(szUserList,0,sizeof(szUserList));
    do
    {
        j=0;
        for(i=0; i<(int)strlen(u); i++)
        {
            if(u[i] != ' ')
            {
                szUserList[j]=u[i];
                j=j+1;
            }
        }
        szUserList[j] = (char)NULL;	  /* Added (char) as without it, it didn't work on Solaris MD 20-JAN-2004 */

        sprintf(pWS->szTmpBufff,"Users=[%s] extract=[%s]  u=[%s]", pWS->U2, szUserList,u);
        LogData(LOGDEBUG, pWS->szTmpBufff, pWS);

        if(wildcmp((char *)&szUserList,(char *)&pWS->RemUID, pWS))
        {
            sprintf(pWS->szTmpBufff,"User accepted [%s]",szUserList);
            LogData(LOGDEBUG,pWS->szTmpBufff, pWS);
            return SEC_OK;
        }
    } while (u = (char*)strtok_r(NULL, ",", &strtok_cb));

    sprintf(pWS->szTmpBufff,"Connection refused, User [%.*s] not in list [%s]",
            rlnts(MQ_USER_ID_LENGTH, pChannelDefinition->RemoteUserIdentifier),
            pChannelDefinition->RemoteUserIdentifier,
            pWS->U2);
    LogData(LOGDEBUG,pWS->szTmpBufff, pWS);
    return SEC_FAILURE;
}

/**
* CheckBlockedUserId()
*
*
* Check List of BlockedUsers to see if RemUID is among them
*
* @pre	List of users already filled
* @return int SEC_OK if user found else SEC_FAILURE
*/
int CheckBlockedUserId(PMQCD pChannelDefinition, workingStorage_type *pWS )
{
    char *p;
    char *u;
    char *strtok_cb;
    char szUserList[33];
    int i,j;
    /*
    * Scan through list of user ID's
    */

    LogData(LOGDEBUG,"CheckBlockedUserId()", pWS);

    p = pWS->BlockUsers;
    u = (char*)strtok_r(pWS->BlockUsers,",", &strtok_cb);

    sprintf(pWS->szTmpBufff,"BlockedUsers=[%s] first u=[%s]", pWS->BlockUsers,u);
    LogData(LOGDEBUG, pWS->szTmpBufff, pWS);

    memset(szUserList,0,sizeof(szUserList));
    do
    {
        j=0;
        for(i=0; i<(int)strlen(u); i++)
        {
            if(u[i] != ' ')
            {
                szUserList[j]=u[i];
                j=j+1;
            }
        }
        szUserList[j] = (char)NULL;	  /* Added (char) as without it, it didn't work on Solaris MD 20-JAN-2004 */

        sprintf(pWS->szTmpBufff,"Users=[%s] extract=[%s]  u=[%s]", pWS->BlockUsers, szUserList,u);
        LogData(LOGDEBUG, pWS->szTmpBufff, pWS);

        if(wildcmp((char *)&szUserList,(char *)&pWS->RemUID, pWS))
        {
            sprintf(pWS->szTmpBufff,"User Blocked [%s]",szUserList);
            LogData(LOGDEBUG,pWS->szTmpBufff, pWS);
            return SEC_FAILURE;
        }
    } while (u = (char*)strtok_r(NULL, ",", &strtok_cb));

    sprintf(pWS->szTmpBufff,"User not found in BlockUsers [%s]",pWS->BlockUsers);
    LogData(LOGDEBUG,pWS->szTmpBufff, pWS);

    return SEC_OK;
}



/**
* CheckConnectionPattern()
*
*
* Check the list of Patterns against the Connname
*
* @pre	All , changed to ; in list so we tokenise correctly
* @return int - SEC_OK or SEC_FAILURE
*/
int CheckConnectionPattern( PMQCD pChannelDefinition, workingStorage_type *pWS )
{
    int j;
    int i;
    char ipstring[33];

    strncpy(pWS->ConName,pChannelDefinition->ConnectionName,MQ_CONN_NAME_LENGTH);
    for (i=0; i<MQ_CONN_NAME_LENGTH; i++)
    {
        if(pWS->ConName[i] == ' ')
        {
            pWS->ConName[i] = 0x00;
            break;
        }
    }

    LogData(LOGDEBUG,"CheckConnectionPattern()", pWS);

    j=0;
    /*		for(i=0;i<=32;i++)			 JHP 040522 */
    for (i=0;i <= MAX_PL; i++)
    {
        if((pWS->Pattern[i] != ';') && (pWS->Pattern[i] != 0x00))
        { 		  /* jhp 04-01-21 single pattern incl. */
            /* NWC separated by ';' because ',' altered to ';' earlier */
            ipstring[j] = pWS->Pattern[i];
            j=j+1;
        }
        else
        {
            ipstring[j] = (char)NULL; 	/* Added (char) as without it, it didn't work on Solaris MD 20-JAN-2004 */
            j=0;
            sprintf(pWS->szTmpBufff,"Pattern [%s]  ip[%s]", pWS->Pattern, ipstring);
            LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
            if (wildcmp((char *)&ipstring,(char *)&pWS->ConName,pWS))
            {
                sprintf(pWS->szTmpBufff,"pattern [%s], ConName [%s] passed test..",
                        ipstring, pWS->ConName);
                LogData(LOGDEBUG,pWS->szTmpBufff, pWS);
                return SEC_OK;
            }
            if (pWS->Pattern[i] == 0x00) /* NWC terminate at end of string JHP040811 */
            { 	i = MAX_PL;
            }
        }
    }
    sprintf(pWS->szTmpBufff,"Connection refused for pattern [%s] ChannelName=[%s] user=[%.*s] ConnName=[%s]",
            pWS->Pattern,
            pWS->szChannelName,
            rlnts(MQ_USER_ID_LENGTH, pChannelDefinition->RemoteUserIdentifier),
            pChannelDefinition->RemoteUserIdentifier,
            pWS->ConName);
    LogData(LOGDATA,pWS->szTmpBufff, pWS);
    WTODataE("BLOCKIP-07E", pWS);
    return SEC_FAILURE;
}



/**
* wildcmplist()
*
*
* Usage:
* This is a lightweight and simple pattern matching function.
*
* if(wildcmplist("sans*,bl?h.*", "blah.jpg")) {
* 	 //we have a match!
* } else {
* 	 //no match =(
*}
*
* @return  int  0=nRetCode to match  1=match
*/
int wildcmplist(char *wild, char *string, workingStorage_type *pWS)
{
    char work[255];
    int  i,j,words=0;
    int  wtab[32];

    i=0;
    memset(work,'\00',sizeof(work));
    do
    {
        work[i++] = *wild;
        if(*wild == ',' | *wild == '\00')
        {
            work[--i]='\00';
            if (wildcmp((char *)work,string, pWS)) {
                return 1;
            }
            memset(work,'\00',sizeof(work));
            i=0;
        }
        wild = wild + 1;
    }  while (*wild != '\00')  ;
    if (i >0)
        if (wildcmp((char *)work,string,pWS)) {
            return 1;
        }
    return 0;
}


/**
* wildcmp()
*
*
* Usage:
* This is a lightweight and simple pattern matching function.
*
* if(wildcmp("bl?h.*", "blah.jpg")) {
* 	 //we have a match!
* } else {
* 	 //no match =(
*}
*
* @return  int  0=nRetCode to match  1=match
*/
int wildcmp(char *wild, char *string, workingStorage_type *pWS)
{
    int  ichfrom,ichto;

    while ((*string != 0x00) && (*wild != '*'))	/* Changed due to nRetCode.. in v.1.12-1.13 */
    {
        switch(*wild)
        {
        case '?': 	   /* single wildcard position */
            wild = wild + 1;
            string = string + 1;
            break;
        case '#': 	   /* Single numeric position */
            if (isdigit(*string) == 0)
            {
                return 0;
            }
            else
                wild = wild + 1;
            string = string + 1;
            break;
        case '$': 	   /* Single alphabetic position */
            if (isalpha(*string) == 0)
            {
                return 0;
            }
            else
                wild = wild + 1;
            string = string + 1;
            break;
        case '[': 	   /* specified range */
            if ((*(wild+2) != '-') | (*(wild+4) != ']') | (isalnum(*(wild+1)) ==0) | (isalnum(*(wild+3)) ==0))
            {
                return 0;
            }
            else
            {
                ichfrom=*(wild+1);
                ichto=*(wild+3);
                sprintf(pWS->szTmpBufff,"Testing..... from %i to %i string %i",ichfrom, ichto, *string);
                LogData(LOGDATA,pWS->szTmpBufff, pWS);
#if defined (__MVS__off)
                if (((isdigit(ichfrom) != 0) & (isdigit(ichto) != 0)) | (isalpha(ichfrom) != 0) & (isalpha(ichto) != 0))
                {                                                                /* same type... */
                    if (((int)*string < ichfrom) | ((int)*string > ichto))
                        return 0;
                }
                else
                {
                    sprintf(pWS->szTmpBufff,"wildcmp don't support mixed patterns yet. (0-J) J-0");
                    LogData(LOGDATA,pWS->szTmpBufff);
                    WTODataE("BLOCKIP-99E");
                    return 0;
                }
#else
                if (((int)*string < ichfrom) | ((int)*string > ichto))
                    return 0;
#endif
            }
            wild = wild + 5;
            string = string + 1;
            break;
        case '(': 	   /* specified range */
            if ((*(wild+2) != '-') | (*(wild+4) != ')') | (isalnum(*(wild+1)) ==0) | (isalnum(*(wild+3)) ==0))
            {
                return 0;
            }
            else
            {
                ichfrom=*(wild+1);
                ichto=*(wild+3);
                sprintf(pWS->szTmpBufff,"Testing..... from %i to %i string %i",ichfrom, ichto, *string);
                LogData(LOGDATA,pWS->szTmpBufff, pWS);
#if defined (__MVS__off)
                if (((isdigit(ichfrom) != 0) & (isdigit(ichto) != 0)) | (isalpha(ichfrom) != 0) & (isalpha(ichto) != 0))
                {                                                                /* same type... */
                    if (((int)*string < ichfrom) | ((int)*string > ichto))
                        return 0;
                }
                else
                {
                    sprintf(pWS->szTmpBufff,"wildcmp don't support mixed patterns yet. (0-J) J-0");
                    LogData(LOGDATA,pWS->szTmpBufff);
                    WTODataE("BLOCKIP-99E");
                    return 0;
                }
#else
                if (((int)*string < ichfrom) | ((int)*string > ichto))
                    return 0;
#endif
            }
            wild = wild + 5;
            string = string + 1;
            break;
        default:
            if(*wild != *string)
            {
                return 0;
            }
            else
            {
                string = string + 1;
                wild = wild + 1;
            }
        }
    }
    /* we have to make shure that the string is not shorter than wild */
    if ((*string == 0x00) && (*wild != '*') && (*wild != 0x00))
    {
        return 0;
    }
    else
    {
        return 1;
    }
}

/*	Strip white space off a string
**	------------------------------
**	Return value points to first non-white character, or to 0 if none.
**	All trailing white space is OVERWRITTEN with zero.
*/
char * Strip (char * s)
{
    if (s) {
	char * p=s;
	for(p=s;*p;p++);		/* Find end of string */
	for(p--;p>=s;p--) {
	    if (isspace((int) *p))
		*p=0;			/* Zap trailing blanks */
	    else
		break;
	}
	while (isspace((int) *s)) s++;	/* Strip leading blanks */
    }
    return s;
}


#ifdef MQCD_VERSION_7

/**
* CheckSSLList()
*
*
* Check the SSL= commands and process accordingly
* If none specified then exit with SEC_OK
*
*
* @return int SEC_OK	SEC_FAILURE
*/
int CheckSSLList( MQCD *pChannelDefinition, workingStorage_type *pWS )
{
    char work[33];
    int i;
    char *p;
    char *u;
    char *strtok_cb;

    LogData(LOGDEBUG,"CheckSSLList()", pWS);

    if(pWS->SSLCount == 0) return SEC_OK;

    if(pChannelDefinition->Version == MQCD_VERSION_7)
    {  /* version 5.3 */
        if(pChannelDefinition->SSLPeerNameLength > 0 && pWS->SSLCount > 0)
        {  /* eval in two.. */
            if(pChannelDefinition->SSLPeerNameLength >= SSL_LEN)
            {
                sprintf(pWS->szTmpBufff,"Connection refused, SSLPeerName too long max [%ld] was [%ld]",
                        SSL_LEN,pChannelDefinition->SSLPeerNameLength);
                LogData(LOGDATA,pWS->szTmpBufff, pWS);
                WTODataE("BLOCKIP-08E", pWS);
                return SEC_FAILURE;
            } 					 /* spec too long */
            memcpy(pWS->SSLPeer,pChannelDefinition->SSLPeerNamePtr,pChannelDefinition->SSLPeerNameLength);
            pWS->SSLPeer[pChannelDefinition->SSLPeerNameLength]=0x00;
            p = strstr(pWS->SSLPeer,"CN=");		/*	allow CN patterns */
            if(p != NULL)
            {
                strcpy(pWS->SSLPeer,p);			/* we're just taking CN */
                pWS->SSLPeer[strlen(p)]=0x00;
                sprintf(pWS->szTmpBufff,"SSLPeer [%s]",p);
                LogData(LOGDEBUG,pWS->szTmpBufff, pWS);
                for(i=0; i<pWS->SSLCount; i++)
                {   /* run thru SSL spec */
                    sprintf(pWS->szTmpBufff,"SSLSpec [%s]",pWS->SSLList[i]);
                    LogData(LOGDEBUG,pWS->szTmpBufff, pWS);
                    p = strstr(pWS->SSLList[i],"CN=");	/*  allow CN patterns */
                    if(p != NULL)
                    {
                        u=(char*)strtok_r(p,";", &strtok_cb);			/* Just take the CN */
                        if(u != NULL )
                        { 					/* It's  time to ask wildcard.. */
                            if(wildcmp(u,(char *)&pWS->SSLPeer,pWS))
                            {
                                if(pWS->SilentMode ==0 )
                                {
                                    sprintf(pWS->szTmpBufff,"SSL Pattern matched [%s]",u);
                                    LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
                                }
                                i=pWS->SSLCount;				/* We're thru patter matching */
                                u=(char*)strtok_r(NULL,";", &strtok_cb);		/* Now we have to find the result */
                                if(u != NULL )
                                { 			  /* Specified ?? */
                                    sprintf(pWS->szTmpBufff,"SSL Action [%s]", u);
                                    LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
                                    if(!strcmp(u,"BLOCK"))
                                    {
                                        LogData(LOGDATA, "Connection refused, CN requested blocked", pWS);
                                        WTODataE("BLOCKIP-09E", pWS);
                                        return SEC_FAILURE;
                                    }
                                    if(!strncmp(u,"MCA=",4))
                                    {
                                        u=u+4;
                                        memset(work,' ',sizeof(work));
                                        strcpy(work,u);
                                        if(strcmp(work,"*"))
                                        {
                                            sprintf(pWS->szTmpBufff,"SSL Set MCA userid to: [%s] work [%s]",u,work);
                                            LogData(LOGDATA, pWS->szTmpBufff, pWS);
                                            WTODataOk("BLOCKIP-51I", pWS);
#if defined (__MVS__)
                                            work[strlen(work)] = ' ';
                                            memset(pChannelDefinition->MCAUserIdentifier,' ',MQ_MCA_USER_ID_LENGTH);
#endif
                                            strncpy(pChannelDefinition->MCAUserIdentifier,work,MQ_MCA_USER_ID_LENGTH);
                                            return SEC_OK;
                                        }
                                    }
                                }
                            }
                        }
                    }
                } /* for()*/
                return SEC_FAILURE;
            }
        }
    }
    return SEC_OK;
}
#endif







/**
* CheckCONList()
*
* Check the CON= parameters
* If no CON= spec was supplied, so return just "OK"
*
* @return int 0=ignore 1=error
*/
int CheckCONList(MQCD *pChannelDefinition,MQLONG ExitReason, workingStorage_type *pWS)
{
    int  i;
    char *u;
    char *strtok_cb;
    char work[33]; /* just working storage */

    LogData(LOGDEBUG,"CheckCONList()", pWS);

    if(pWS->CONCount == 0) return SEC_OK;

    for(i=0; i<pWS->CONCount; i++)
    {
        sprintf(pWS->szTmpBufff,"CONList[i] = [%s]",pWS->CONList[i]);
        LogData(LOGDEBUG,pWS->szTmpBufff, pWS);
        u=(char*)strtok_r(pWS->CONList[i],";", &strtok_cb);		/* Just take the connection pattern	*/
        if(u != NULL )
        { 								/* It's  time to ask wildcard.. */
            sprintf(pWS->szTmpBufff,"CON Pattern: [%s]",u);
            LogData(LOGDEBUG,pWS->szTmpBufff, pWS);
            if(wildcmp(u,(char *)&pWS->ConName,pWS))
            {
                sprintf(pWS->szTmpBufff,"CON Pattern matched [%s] CON name [%s]", u,&pWS->ConName);
                LogData(LOGDEBUG,pWS->szTmpBufff, pWS);
                u=(char*)strtok_r(NULL,";", &strtok_cb);		/* Now we have to find the userid */
                if(u != NULL )
                { 			  /* Specified userid ?? */
                    if (pWS->bUseridUpperLowerCase==1) 			/* JHP 2.12 */
                    {
                        sprintf(pWS->szTmpBufff,"ConUserid Before strupr [%s]",u);
                        LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
                        i_strupr(u);
                        sprintf(pWS->szTmpBufff,"ConUserid After strupr [%s]",u);
                        LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
                    }
                    sprintf(pWS->szTmpBufff,"CON Userid: [%s]",u);
                    LogData(LOGDEBUG,pWS->szTmpBufff, pWS);
                    /* we need to check the userid as a list (CON=127.*;peter,post,sonny;MCA=mrmqo1;) */

                    if(wildcmplist(u,(char *)&pWS->RemUID,pWS))
                    {
                        sprintf(pWS->szTmpBufff,"CON/RemUid Pattern matched [%s] RemUID [%s]",u, &pWS->RemUID);
                        LogData(LOGDEBUG,pWS->szTmpBufff, pWS);
                        i=pWS->CONCount;					/* We're thru pattern matching */
                        u=(char*)strtok_r(NULL,";", &strtok_cb);		/* Now we have to find the result */
                        if(u != NULL )
                        { 		 /* Specified what keyword do we have ? */
                            if(!strcmp(u,"BLOCK"))   /* Block this combination ? */
                            {
                                LogData(LOGDATA, "Connection refused, CON requested blocked", pWS);
                                WTODataE("BLOCKIP-09E", pWS);
                                return SEC_FAILURE;
                            }
                            if(u != NULL )
                            { 			  /* Specified userid ?? */
                                if(!strncmp(u,"MCA=",4))
                                {
                                    LogData(LOGDEBUG, "CON MCA specified", pWS);
                                    u=u+4;
                                    memset(work,' ',sizeof(work));
                                    strcpy(work,u);
                                    if (pWS->bRespectMCA==1)
                                    {
                                        sprintf(pWS->szTmpBufff,"work [%s] MCA [%s]",work,
                                                pChannelDefinition->MCAUserIdentifier);
                                        LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
                                        LogData(LOGDEBUG, "RespectMCA was specified..", pWS);
                                    }
                                    else
                                    {
                                        LogData(LOGDEBUG, "RespectMCA was not specified...", pWS);
                                    }
                                    if(strcmp(work,"*"))
                                    {
                                        if ((pWS->bRespectMCA==1) && (memcmp(pChannelDefinition->MCAUserIdentifier,
                                                                             work,
                                                                             sizeof(pChannelDefinition->MCAUserIdentifier))) != 0 )
                                        {
                                            LogData(LOGDATA, "Connection refused, RespectMCA specified", pWS);
                                            WTODataE("BLOCKIP-23E", pWS);
                                            return SEC_FAILURE;
                                        }
                                        sprintf(pWS->szTmpBufff,"CON Set MCA userid to [%s] from [%s]",u,pWS->RemUID);
                                        LogData(LOGDATA,pWS->szTmpBufff, pWS);
#if defined (__MVS__)
                                        work[strlen(work)] = ' ';
                                        memset(pChannelDefinition->MCAUserIdentifier,' ',MQ_MCA_USER_ID_LENGTH);
#endif
                                        strncpy(pChannelDefinition->MCAUserIdentifier,work,MQ_MCA_USER_ID_LENGTH);
                                    }
                                }
                            }
                        }
                        LogData(LOGDEBUG,"CheckCONList leave return 0 (OK)", pWS);
                        return SEC_OK;	 /* Successful check */
                    }  /* if check userid .. */
                }
            }
        }
    }
    LogData(LOGDEBUG, "CheckCONList leave return 1 (Failed)", pWS);
    return SEC_FAILURE; /* Check failed */
}





/**
* CheckInvalidUsers()
*
*
* Run through a list of possible invalid user names and fail if present
* otherwise drop through on success.
*
* Expansion Possibilities
* -----------------------
* Simply add additional names to end of list
*
*
* @return int SEC_OK or SEC_FAILUR
*/
int CheckInvalidUsers(PMQCD pChannelDefinition, workingStorage_type *pWS)
{
    char szZosUserId[9];

    LogData(LOGDEBUG,"CheckInvalidUsers()", pWS);

    if (pWS->bAllowBlankUserID==0)
    {
        if(pChannelDefinition->RemoteUserIdentifier[0] == ' ')
        {
            LogData(LOGDATA,"Connection refused for blank user identifier", pWS);
            WTODataE("BLOCKIP-10E", pWS);
            return SEC_FAILURE;
        }
    }
    /*
    * Check for system uids
    */
    if(!strcmp(pWS->RemUID,"mqm"))
    {
        LogData(LOGDATA,"Connection refused for system user identifier -1", pWS);
        WTODataE("BLOCKIP-11E", pWS);
        return SEC_FAILURE;
    }
    if(!strcmp(pWS->RemUID,"MUSR_MQADMIN"))
    {
        LogData(LOGDATA,"Connection refused for system user identifier.-2", pWS);
        WTODataE("BLOCKIP-12E", pWS);
        return SEC_FAILURE;
    }
    if(!strcmp(pWS->RemUID,"musr_mqadmin"))
    {
        LogData(LOGDATA,"Connection refused for system user identifier.-3", pWS);
        WTODataE("BLOCKIP-13E", pWS);
        return SEC_FAILURE;
    }
    if(!strcmp(pWS->RemUID,"QMQM"))
    {
        LogData(LOGDATA,"Connection refused for system user identifier.-4", pWS);
        WTODataE("BLOCKIP-24E", pWS);
        return SEC_FAILURE;
    }
#if defined (__MVS__)
    memset(szZosUserId,'\00',sizeof(szZosUserId));
    GetZOSUserId((char*)&szZosUserId);
    if(!strcmp(pWS->RemUID,szZosUserId))
    {
        LogData(LOGDATA,"Connection refused for system user identifier(CHIN task).-5", pWS);
        WTODataE("BLOCKIP-25E", pWS);
        return SEC_FAILURE;
    }
#endif
    return SEC_OK;
}

/**
* CheckBlankUser()
*
*
* Check if userid is present and fail if not present
* otherwise drop through on success.
*
*
* @return int SEC_OK or SEC_FAILUR
*/
int CheckBlankUser(PMQCD pChannelDefinition, workingStorage_type *pWS)
{

    LogData(LOGDEBUG,"CheckBlankUser()", pWS);

    if(pChannelDefinition->RemoteUserIdentifier[0] == ' ')
    {
        LogData(LOGDATA,"Connection refused for blank user identifier", pWS);
        WTODataE("BLOCKIP-10E", pWS);
        return SEC_FAILURE;
    }
    return SEC_OK;
}



/**
* ProcessData()
*
*
*
* Process a line of data for all possible tags and setup required structures.
* Can be called from ProcessFile() method or direct from exit data field.
* If tag is unknow then return OK anyway (no harm in doing so).
*
*
* Expansion Possibilities
* -----------------------
* Add additional commands to the end of the method.
*
*
* @return int SEC_OK if line was processed OK else SEC_FAILURE if an error occured
*/
int ProcessData(char *pszData, workingStorage_type *pWS)
{
    int bFound=0;
    char *p;
    char *u;
    char *strtok_cb;
    int i;
    long lCurrChl;
    char work[49];  /* max obj len + 1 */

    /*
    * Test for command termination
    */
    p = (char*)strtok_r(pszData," ", &strtok_cb);
    if(p != NULL)
    {
        /*       p=p-1;*/
        if(p[strlen(p)-1] != ';')
            /*       if(p[0] != ';')*/
        {
            sprintf(pWS->szTmpBufff,"Connection refused, invalid specification, parm not terminated with \";\" [%s]" ,pszData);
            LogData(LOGDATA, pWS->szTmpBufff, pWS);
            return SEC_FAILURE;
        }

    }

    /*
    * Test for logging command
    */
    p = strstr(pszData,"LogFormat=");
    if(p != NULL)
    {
        pWS->bLogByDate=1;
        bFound=1;
        p=p+10;
        u=(char*)strtok_r(p,";", &strtok_cb);
        if(u!=NULL)
        {
            for(i=0;i<(int)strlen(u);i++)
            {
                if(*(u+i) == 'N') pWS->nLogFormat |= LOGNAME;
                if(*(u+i) == 'D') pWS->nLogFormat |= LOGDATE;
                if(*(u+i) == 'C') pWS->nLogFormat |= LOGCHANNEL;
                if(*(u+i) == 'U') pWS->nLogFormat |= LOGUSER;
            }
        }
    }
    p = strstr(pszData,"LogFileName=");
    if((p!=NULL) && (bFound==0))
    {
        bFound=1;
        p=p+12;
        u=(char*)strtok_r(p,";", &strtok_cb);
        if(u != NULL)
        {
            strcpy(pWS->szLogFileName,u);
        }
    }
    p = strstr(pszData,"LogExt=");
    if((p!=NULL) && (bFound==0))
    {
        bFound=1;
        p=p+7;
        u=(char*)strtok_r(p,";", &strtok_cb);
        if(u != NULL)
        {
            strcpy(pWS->szLogFileExtension,u);
        }
    }
    p = strstr(pszData,"LogPath=");
    if((p!=NULL) && (bFound==0))
    {
        bFound=1;
        p=p+8;
        u=(char*)strtok_r(p,";", &strtok_cb);
        if(u != NULL)
        {
            strcpy(pWS->szLogPath,u);
        }
    }
    p = strstr(pszData,"LogDrive=");
    if((p !=NULL) && (bFound==0))
    {
        bFound=1;
        p=p+9;
        u=(char*)strtok_r(p,";", &strtok_cb);
        if(u != NULL)
        {
            strcpy(pWS->szLogDrive,u);
        }
    }
/* 2.46 start */
#if defined (__EXT_LOG_FILENAME__)             /* Only UNIX, LINUX and Windows */
       p = strstr(pszData,"LogCount=");
       if((p !=NULL) && (bFound==0))
       {
           bFound=1;
           p=p+9;
           u=(char*)strtok_r(p,";", &strtok_cb);
           if(u != NULL)
           {
               pWS->lLogFileCount=atol(u);
               sprintf(pWS->szTmpBufff,"Number of logfiles set [%ld]",pWS->lLogFileCount);
               LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
               if ((pWS->lLogFileCount < 1) | (pWS->lLogFileCount > 99)) {
                  pWS->lLogFileCount=3;
                  sprintf(pWS->szTmpBufff,"Invalid value was specified for LogCount, value reset to default [%ld]",pWS->lLogFileCount);
                  LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
               }
           }
       }
       p = strstr(pszData,"LogSize=");
       if((p !=NULL) && (bFound==0))
       {
           bFound=1;
           p=p+8;
           u=(char*)strtok_r(p,";", &strtok_cb);
           if(u != NULL)
           {
               pWS->lMaxlogFileSize=atol(u);
               sprintf(pWS->szTmpBufff,"Size of logfiles set [%ld]",pWS->lMaxlogFileSize);
               LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
               if (pWS->lMaxlogFileSize < 100*1024) {
                  pWS->lMaxlogFileSize=200*1024;
                  sprintf(pWS->szTmpBufff,"Invalid value was specified for LogSize, value reset to default [%ld]",pWS->lMaxlogFileSize);
                  LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
               }
           }
       }
#endif /* __EXT_LOG_FILENAME__ defined */
/* 2.46 end */

    /*
    * Test for Block system accounts command
    */
    p = strstr(pszData,"BlockMqmUsers=Y");
    if(p != NULL)
    {
        pWS->bBlockMqmUsers=1;
        bFound=1;
    }
    /*
    * Save pattern structure away if applicable
    */
    p = strstr(pszData,"Patterns=");
    if((p != NULL) && (bFound==0))
    {
        bFound=1;
        p=p+9;
        u=(char*)strtok_r(p,";", &strtok_cb);
        if(u != NULL)
        {
            if( (strlen(u) + 1 + strlen(pWS->Pattern)) >= MAX_PL) /*NWC allow multiple lines */
            {
                sprintf(pWS->szTmpBufff,"Connection refused, Pattern string is too long, max[%d] was [%d]",MAX_PL-1,(strlen(u) + 1 + strlen(pWS->Pattern)));
                LogData(LOGDATA, pWS->szTmpBufff, pWS);
                WTODataE("BLOCKIP-14E", pWS);
                return SEC_FAILURE;
            }
            strcat(pWS->Pattern,u);
            strcat(pWS->Pattern,";");	/* NWC put ; on the end to allow multiple lines */
        }
    }
    /*
    * Save Userids list away if applicable
    */
    p = strstr(pszData,"Userids=");
    if((p != NULL) && (bFound==0))
    {
        bFound=1;
        p=p+8;
        u=(char*)strtok_r(p,";", &strtok_cb);
        if(u != NULL)
        {
            if( (strlen(u) +1 + strlen(pWS->Users)) >= MAX_PL) /* NWC allow multiple lines */
            {
                sprintf(pWS->szTmpBufff,"Connection refused, Users string is too long, max[%d], was [%d]",MAX_PL-1,(strlen(u) +1 + strlen(pWS->Users)));
                LogData(LOGDATA, pWS->szTmpBufff, pWS);
                WTODataE("BLOCKIP-15E", pWS);
                return SEC_FAILURE;
            }
            strcat(pWS->Users,u);
            strcat(pWS->Users,";"); /* NWC allow multiple lines */
            pWS->Users[strlen(u)] = 0x00;
            if (pWS->bUseridUpperLowerCase==1) 			/* JHP 2.12 */
            {
                sprintf(pWS->szTmpBufff,"Users Before strupr [%s]",pWS->Users);
                LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
                i_strupr(pWS->Users);
                sprintf(pWS->szTmpBufff,"Users After strupr [%s]",pWS->Users);
                LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
            }
        }
    }
    /*
    * Save Connection list (CON=) options (if applicable).
    */
    p = strstr(pszData,"CON=");
    if((p != NULL) && (bFound==0))
    {
        bFound=1;
        p=p+4;
        u=(char*)strtok_r(p," ", &strtok_cb);						  /* Just take the data, no imbedded blanks */
        if(u != NULL)
        {
            if(u[strlen(u)-1] != ';')
            {
                sprintf(pWS->szTmpBufff,"Connection refused, CON Spec invalid, not terminated with \";\" [%s]" ,p-4);
                LogData(LOGDATA, pWS->szTmpBufff, pWS);
                return SEC_FAILURE;
            }
            if(strlen(u) >= CON_LEN)
            {
                sprintf(pWS->szTmpBufff,"Connection refused, CON Spec string is too long, max [%d], was [%d]" ,SSL_LEN-1,strlen(u));
                LogData(LOGDATA, pWS->szTmpBufff, pWS);
                WTODataE("BLOCKIP-16E", pWS);
                return SEC_FAILURE;
            }
            /*
            sprintf(pWS->szTmpBufff,"CON spec added [%s]",u);
            LogData(LOGDEBUG, pWS->szTmpBufff);
            */
            strcpy(pWS->CONList[pWS->CONCount++],u);
        }
    }
    /*
    * Test for SSL= pattern and save parameters in List
    */
    p = strstr(pszData,"SSL=");
    if((p != NULL) && (bFound==0))
    {
        bFound=1;
        p=p+4;
        u=(char*)strtok_r(p," ", &strtok_cb);						  /* Just take the data, no imbedded blanks */
        if(u != NULL)
        {
            if(strlen(u) >= SSL_LEN)
            {
                sprintf(pWS->szTmpBufff,"Connection refused, SSL Spec string is too long, max[%d], was [%d]",SSL_LEN-1,strlen(u));
                LogData(LOGDATA, pWS->szTmpBufff, pWS);
                WTODataE("BLOCKIP-17E", pWS);
                return SEC_FAILURE;
            }
            /*
            sprintf(pWS->szTmpBufff,"SSL CN spec added [%s]",u);
            LogData(LOGDEBUG, pWS->szTmpBufff);
            */
            strcpy(pWS->SSLList[pWS->SSLCount++],u);
        }
    }

    /*
    * Test for Case sensitivity flag
    */
    p = strstr(pszData,"UseridUpperLowerCase=*");
    if(p != NULL)
    {
        pWS->bUseridUpperLowerCase=1;
        bFound=1;
    }


    /*
    * Save BlockUsers list away if applicable
    */
    p = strstr(pszData,"BlockUsers=");
    if((p != NULL) && (bFound==0))
    {
        bFound=1;
        p=p+11;
        u=(char*)strtok_r(p,";", &strtok_cb);
        if(u != NULL)
        {
            if( (strlen(u) + 1 + strlen(pWS->BlockUsers)) >= MAX_PL)
            {
                sprintf(pWS->szTmpBufff,"Connection refused, BlockUsers string is too long, max[%d], was [%d]",MAX_PL-1,strlen(u));
                LogData(LOGDATA, pWS->szTmpBufff, pWS);
                WTODataE("BLOCKIP-18E", pWS);
                return SEC_FAILURE;
            }
            strcat(pWS->BlockUsers,u);
            strcat(pWS->BlockUsers,";");
            pWS->BlockUsers[strlen(u)] = 0x00;
            if (pWS->bUseridUpperLowerCase==1) 			/* JHP 2.12 */
            {
                sprintf(pWS->szTmpBufff,"BlockUsers Before strupr [%s]",pWS->BlockUsers);
                LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
                i_strupr(pWS->BlockUsers);
                sprintf(pWS->szTmpBufff,"BlockUsers After strupr [%s]",pWS->BlockUsers);
                LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
            }
        }
    }

    /*
    * Test for Respect channel definition MCA userid
    */
    p = strstr(pszData,"RespectMCA=Y");
    if((p != NULL) && (bFound==0))
    {
        pWS->bRespectMCA=1;
        bFound=1;
    }

    /* Start inq chl */
    /*
    * Test for MAX active channels
    */
    p = strstr(pszData,"MAXCHL=");
    if((p != NULL) && (bFound==0))
    {
        bFound=1;
        p=p+7;
        p=(char*)strtok_r(p," ", &strtok_cb);
        u=(char*)strtok_r(p,";", &strtok_cb);
        if(u != NULL)
        {
            if (wildcmp((char *)u, (char *)&pWS->szChannelName, pWS))
            {
                strcpy(work,u);
                u=(char*)strtok_r(NULL,";", &strtok_cb);		/* Now we have to find max number of channels */
                if(u == NULL )
                {
                    sprintf(pWS->szTmpBufff,"MAX active channels set on [%s], number missing",pWS->szChannelName);
                    LogData(LOGDATA, pWS->szTmpBufff, pWS);
                    return SEC_FAILURE;
                }
                pWS->lMaxChannelActive=atol(u);
                sprintf(pWS->szTmpBufff,"MAX active channels set on [%s] set to [%ld] Pattern used[%s]",pWS->szChannelName,pWS->lMaxChannelActive, work);
                LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
            }
            else
            {
                sprintf(pWS->szTmpBufff,"MAX active channels set on [%s], was skipped.",u);
                LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
            }
        }
    }
    /* End inq chl */
    /*
    * Test for Allow blank userid
    */
    p = strstr(pszData,"AllowBlankUserID=Y");
    if(p != NULL)
    {
        pWS->bAllowBlankUserID=1;
        bFound=1;
    }

    /*
    * Test for Expiry settings
    */
    p = strstr(pszData,"Expiry=");
    if((p != NULL) && (bFound==0))
    {
        bFound=1;
        p=p+7;
        p=(char*)strtok_r(p," ", &strtok_cb);
        u=(char*)strtok_r(p,";", &strtok_cb);
        if(u != NULL)
        {
            pWS->lExpireValue=atol(u)*10;
            sprintf(pWS->szTmpBufff,"Expiry for channel rescan [%ld]",pWS->lExpireValue);
            LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
        }
    }

    /*
    * Test for Expiry settings
    */
    p = strstr(pszData,"ConfigQueue=");
    if((p != NULL) && (bFound==0))
    {
        bFound=1;
        p=p+12;
        p=(char*)strtok_r(p," ", &strtok_cb);
        u=(char*)strtok_r(p,";", &strtok_cb);
        if(u != NULL)
        {
            if (strlen(u) < MQ_OBJECT_NAME_LENGTH) {
                strcpy(pWS->szStatusQ,u);
                sprintf(pWS->szTmpBufff,"Configuration queue is set to [%s]",pWS->szStatusQ);
                LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
            }
            else {
                sprintf(pWS->szTmpBufff,"Configuration queue is invalid [%s] name too long(%ld). max %ld",u,strlen(u),MQ_OBJECT_NAME_LENGTH);
                LogData(LOGDATA, pWS->szTmpBufff, pWS);
                WTODataE("BLOCKIP-99E", pWS);
                return SEC_FAILURE;			 /* Newer ever accept bad parameters */

            }
        }
    }


#if defined (WIN32)
    /*
    * Test for Log control
    */
    p = strstr(pszData,"WinLog=ONLY");
    if(p != NULL)
    {
        pWS->lWinLog=LOG_EVTLOGONLY;
        bFound=1;
    }
    p = strstr(pszData,"WinLog=FILE");
    if(p != NULL)
    {
        pWS->lWinLog=LOG_FILEONLY;
        bFound=1;
    }
    p = strstr(pszData,"WinLog=BOTH");
    if(p != NULL)
    {
        pWS->lWinLog=LOG_BOTH;
        bFound=1;
    }
#endif

    /*
    * Add additional tags here

     ---->

    */

    /*
    * Record a warning - lets not worry if parameter is wrong
    */
    if(bFound==0)
    {
        sprintf(pWS->szTmpBufff, "Error - Unknown Tag [%s]", pszData);
        LogData(LOGDATA, pWS->szTmpBufff, pWS);
        WTODataE("BLOCKIP-19E", pWS);
        return SEC_FAILURE;			 /* Newer ever accept bad parameters */
    }
    return SEC_OK;
}



/**
*  ProcessFile()
*
*
*
* Open file and call the ProcessData() method to process the options.
* Return failure for file errors or data processing errors.
*
*
* @return int  SEC_OK or SEC_FAILURE
*/
int ProcessFile( char *p, workingStorage_type *pWS)
{
    int  bFound;
    int  nRetCode=SEC_OK;	/* parameters are OK */
    int  i;
    char *u;
    char *strtok_cb;
    char szFileName[32];	/* Input Data file name */
    FILE *phInFile;			/* Declare file handles for input and output   */
    char *result;
    char line[MAX_LEN];
    char cline[MAX_LEN];

    /*	sprintf(pWS->szTmpBufff,"ProcessFile() Invoked.");
    	LogData(LOGDEBUG, pWS->szTmpBufff);
    */

    szFileName[0]  = 0;
    if( strchr(p,';') == 0 )
    {
        sprintf(pWS->szTmpBufff,"Connection refused, invalid file name pattern specified: FN=<filename...>; [%s], trailing ';' missing.", pWS->Pattern);
        LogData( LOGDATA, pWS->szTmpBufff, pWS );
        WTODataE("BLOCKIP-20E", pWS);
        return SEC_FAILURE;
    }

    u=(char*)strtok_r(p,";", &strtok_cb);
    memcpy(szFileName,u,strlen(u));
    szFileName[strlen(u)] = 0x00;
    phInFile = fopen( szFileName, "r" );
    if( phInFile == NULL )
    {
        sprintf(pWS->szTmpBufff,"Connection refused, Specified file [%s] was not found", szFileName);
        LogData( LOGDATA, pWS->szTmpBufff, pWS);
        WTODataE("BLOCKIP-21E", pWS);
        fclose( phInFile );
        return SEC_FAILURE;
    }
    pWS->Pattern[0] = 0x00;		/* NWC start with empty pattern, not *; */
    pWS->Users[0] = 0x00;
    pWS->BlockUsers[0] = 0x00;	/* NWC allow for multiple lines */
    do
    {
        result = fgets(line,MAX_LEN,phInFile);
        if(result != NULL)
        {
            bFound=0;
            i=strlen(line);
            if(line[i-1] == '\n')				/* last char is a new-line */
            {
                line[i-1]  = '\0';				/* replace new-line with null */
                --i;							/* reduce buffer length */
            }
            memset(cline,' ',i);
            cline[i]=0x00;
            /*
            * Skip comment and empty lines - just process for possible data
            */
            if((line[0] != '#') && (strcmp(line,cline) != 0))
            {
                if(ProcessData( line, pWS )==SEC_FAILURE)
                    nRetCode=SEC_FAILURE;
            }
            else
                bFound=1;				/* Comment lines are allways processed */
        }
    } while (result != NULL);
    if (pWS->Pattern[0] == 0x00)				/* NWC If pattern is NULL, use *; */
    {
        strcpy(pWS->Pattern,"*;");
    }
    fclose( phInFile );
    return nRetCode;
}


#ifdef UNIX
/**
*  ProcessMQSiniFile()
*
*
*
* Open MQS.ini file and process the configuration specifications.
* Return failure for file errors or data processing errors.
*
*
* @return int  SEC_OK or SEC_FAILURE
* function added under 2.45
*/
int ProcessMQSiniFile(char *szQmgrName, workingStorage_type *pWS, ws_type *pWS1)
{
    int  bFound;
    int  nRetCode=SEC_OK;	/* parameters are OK */
    int  i;
    char *u;
    char *strtok_cb;
    char szFileName[32];	/* Input Data file name */
    FILE *phInFile;			/* Declare file handles for input and output   */
    char *result;
    char line[255];
    char sline[255];
    char cline[255];
    char szQmgrDirectory[255] = "";
    char szQmgrPrefix[255] = "";
    char * p;
    int  groupid=0;
    int  bfoundQmgr=0;

#define MQINI_GROUP_NotDefined       0
#define MQINI_GROUP_AllQueueManagers 1
#define MQINI_GROUP_ClientExitPath   0
#define MQINI_GROUP_QueueManager     4
#define MQINI_GROUP_LogDefaults      0

    LogData(LOGDEBUG, "ProcessMQSiniFile() Invoked.", pWS);

    strcpy(szFileName,"/var/mqm/mqs.ini");
    phInFile = fopen( szFileName, "r" );
    if( phInFile == NULL )
    {
        sprintf(pWS->szTmpBufff,"Connection refused, Specified file [%s] was not found.", szFileName);
        LogData( LOGDATA, pWS->szTmpBufff, pWS);
        fclose( phInFile );
        return SEC_FAILURE;
    }
    do
    {
        result = fgets(sline,MAX_LEN,phInFile);
        if(result != NULL)
        {
            p=Strip(sline);
            strcpy(line,p);
            bFound=0;
            i=strlen(line);
            if(line[i-1] == '\n')				/* last char is a new-line */
            {
                line[i-1]  = '\0';				/* replace new-line with null */
                --i;							/* reduce buffer length */
            }
            memset(cline,' ',i);
            cline[i]=0x00;
            /*
            * Skip comment and empty lines - just process for possible data
            */
            if((line[0] != '#') && (strcmp(line,cline) != 0))
            {
               if (line[strlen(line)-1]==':')
                  {
                  bFound=0;
                  bfoundQmgr=0;
                  groupid=MQINI_GROUP_NotDefined;
                  printf("Keyword..... %s\n",line);
                  }
               if (!strcmp(line,"AllQueueManagers:"))
                  {
                     bFound=1;
                     groupid=MQINI_GROUP_AllQueueManagers;
                   }
               if (!strcmp(line,"ClientExitPath:"))
                  {
                     bFound=1;
                     groupid=MQINI_GROUP_ClientExitPath;
                   }
               if (!strcmp(line,"LogDefaults:"))
                  {
                     bFound=1;
                     groupid=MQINI_GROUP_LogDefaults;
                   }
               if (!strcmp(line,"QueueManager:"))
                  {
                     bFound=1;
                     groupid=MQINI_GROUP_QueueManager;
                   }

               sprintf(pWS->szTmpBufff,"Line read %i %s",groupid, line);
               LogData( LOGDEBUG, pWS->szTmpBufff, pWS);

               if (groupid==MQINI_GROUP_QueueManager && bfoundQmgr==1) {
                  p = strstr(line,"Prefix=");
                  if(p != NULL )
                     {
                        p=p+7;
                        strcpy(szQmgrPrefix,p);
                        LogData(LOGDEBUG,"Wanted QmgrPrefix found",pWS);
                     }
                  p = strstr(line,"Directory=");
                  if(p != NULL )
                     {
                        p=p+10;
                        strcpy(szQmgrDirectory,p);
                        LogData(LOGDEBUG,"Wanted QmgrDirectory found",pWS);
                     }
               }

               if (groupid==MQINI_GROUP_QueueManager && bFound==0) {
                  LogData(LOGDEBUG,"Now we have the queueManager token...",pWS);
                  p = strstr(line,"Name=");
                  if(p != NULL )
                     {
                        p=p+5;
                        if (!strcmp(p,szQmgrName)) {
                           bfoundQmgr=1;
                           LogData(LOGDEBUG,"Wanted qmgr found",pWS);
                           }
                     }
                  }
            }
            else
                bFound=1;				/* Comment lines are allways processed */
        }
    } while (result != NULL);
    fclose( phInFile );
    sprintf(pWS->szTmpBufff,"resulting path: %s/qmgrs/%s\n",szQmgrPrefix,szQmgrDirectory);
    LogData( LOGDEBUG, pWS->szTmpBufff, pWS);
    strcpy(pWS->szQmgrPrefix,szQmgrPrefix);
    strcpy(pWS->szQmgrDirectory,szQmgrDirectory);
    strcpy(pWS1->szQmgrPrefix,szQmgrPrefix);
    strcpy(pWS1->szQmgrDirectory,szQmgrDirectory);
    return nRetCode;
}

#endif

/**
*  CopyUsers()
*  added JHP 2004-03-30
*
*
* Copy userids from the passed pointer into Users[], this is to help
*	   with backward compability
* Return failure if problem with parameter.
*
*
* @return int  SEC_OK or SEC_FAILURE
*/

int CopyUsers(char *p, workingStorage_type *pWS)
{
    char *u;
    char *strtok_cb;

    p=p+8;
    u=(char*)strtok_r(p,";", &strtok_cb);
    if(u != NULL)
    {
        if(strlen(u) >= MAX_PL)
        {
            sprintf(pWS->szTmpBufff,
                    "Connection refused, Users string is too long,",
                    " max[%d], was [%d]",MAX_PL-1,strlen(u));
            LogData(LOGDATA, pWS->szTmpBufff, pWS);
            WTODataE("BLOCKIP-22E", pWS);
            return SEC_FAILURE;
        }
        strcpy(pWS->Users,u);
        pWS->Users[strlen(u)] = 0x00;
        if (pWS->bUseridUpperLowerCase==1)				   /* 2.12 */
        {
            sprintf(pWS->szTmpBufff,"Users Before strupr [%s]",pWS->Users);
            LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
            i_strupr(pWS->Users);
            sprintf(pWS->szTmpBufff,"Users After strupr [%s]",pWS->Users);
            LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
        }
    }
    return SEC_OK ;
}


/**
* GenerateLogFileName(char * Buffer)
*
*
* Method to geterate logfile name
*
*
* @return char *
*/
char * GenerateLogFileName(char * szFile, workingStorage_type *pWS)
{
    struct tm *timenow;
    time_t longtime;

    char szBuff1[64];
    char szDate[32];
    /*    char szFile[256];*/

    char *p = NULL;

    time(&longtime);
    timenow = localtime(&longtime);
    sprintf(szDate,"%04d-%02d-%02d", timenow->tm_year+1900, timenow->tm_mon+1,timenow->tm_mday);
    sprintf(szBuff1,"%s", pWS->szLogFileName);

#if ! defined (__MVS__)
    if(pWS->nLogFormat & LOGCHANNEL)
    {
        strcat(szBuff1, "_");
        strcat(szBuff1,pWS->szLogChannel);
    }
    if(pWS->nLogFormat & LOGUSER)
    {
        strcat(szBuff1, "_");
        strcat(szBuff1,pWS->RemUID);
    }
    if(pWS->nLogFormat & LOGDATE)
    {
        strcat(szBuff1, "_");
        strcat(szBuff1,szDate);
    }
#if defined (__EXT_LOG_FILENAME__)             /* Only UNIX, LINUX and Windows */
    strcat(szBuff1,"001");                     /* make room for file rename */
#endif
    strcat(szBuff1, ".");
    strcat(szBuff1, pWS->szLogFileExtension);
#endif
    /*
    * Now have file to save to - now check path
    */
    strcpy(szFile,"");

#if defined (WIN32)
    strcat(szFile,pWS->szLogDrive);
    if( szFile[1] != ':') strcat(szFile, ":");
    strcat(szFile,pWS->szLogPath);
    strcat(szFile,"\\");
#elif defined (__MVS__)
    /* NWC No drive or path required on MVS */
    strcpy(szFile,"");
#else
    if (pWS->szLogPath[0] != '/')					/* Fix bad spec JHP 040522 */
    {
        strcpy(szFile,"/");
    }
    strcat(szFile,pWS->szLogPath);
    if (szFile[strlen(szFile)-1] != '/')		/* Fix bad spec JHP 040522 */
    {
        strcat(szFile,"/");
    }
#endif

    strcat(szFile,szBuff1);

#if MQAT_DEFAULT==MQAT_OS400
    strcat(szFile,pWS->szLogPath);
    if (szFile[strlen(szFile)-1] != '/')		/* insert seperator */
    {
        strcat(szFile,"/");
    }
    strcat(szFile, ".");
    strcat(szFile, pWS->szLogFileExtension);
#endif

    p=szFile;
    return p;
}


/**
* LogData()
*
*
* Method to log data in a specific way
*
*	  iMode 0 = Normal 1= Debug String
*	 szData ASCII data to report
*
*
* @return void
*/
void LogData(int iMode, char * szData, workingStorage_type *pWS)
{
    FILE *fptr;
    struct tm *timenow;
    time_t longtime;

    char   szBuff1[64];

    char szDate[32];
    char szTime[32];
    char szFile[256];
    char szFile2[256];

#if defined (WIN32)
    DWORD dwWaitResult;
    long lCount;
    long lFileSize;
#endif

    /* skip debug messages if not in debug mode*/
    if((iMode==LOGDEBUG) &&
            (pWS->nDebugFlag!=LOGDEBUG)) return;

    /* Start JHP060215 */
#if defined (WIN32)
    /* Send the message to the event viewer */
    if ((pWS->lWinLog==LOG_BOTH) | (pWS->lWinLog==LOG_EVTLOGONLY)) {
        SendLogMessage(szData);
        if (pWS->lWinLog==LOG_EVTLOGONLY) {
            return;
        }
    }
#endif
    /*End JHP060214 */

#if defined (__MVS__)
    translate (szData, "","", pWS);	   /* remove [] from the list JHP 040805 */
#endif

    /*
    * Process normal string or debug string with debug flag active
    */

    time(&longtime);
    timenow = localtime(&longtime);
    sprintf(szDate,"%04d-%02d-%02d", timenow->tm_year+1900, timenow->tm_mon+1,timenow->tm_mday);
    sprintf(szTime,"%02d:%02d:%02d",timenow->tm_hour,timenow->tm_min,timenow->tm_sec);
    sprintf(szBuff1,"%s", pWS->szLogFileName);

#if MQAT_DEFAULT==MQAT_OS400  /* test added */
    fprintf(stderr,szDate);
    fprintf(stderr,"|");
    fprintf(stderr,szTime);
    fprintf(stderr,"|");
    fprintf(stderr,szData);
    fprintf(stderr,"\n");

#else  /* End of OS400 */

    strcpy(szFile,GenerateLogFileName(szFile2,pWS));

#if defined (WIN32)
    OpenMTX(pWS,szFile);           /* See if the Mutex is in place */
    /* Wait for the mutex before writing the logevent, to prevent problems with the file. Just one is allowed.*/
    lCount=0;
    while( ( dwWaitResult = WaitForSingleObject( pWS->hMutex, 5L ) ) == WAIT_TIMEOUT)
        lCount++;
#endif
    if((fptr = fopen( szFile,LOGMODE))!=NULL)
    {
        fprintf(fptr,szDate);
        fprintf(fptr, "|");
        fprintf(fptr,szTime);
        fprintf(fptr, "|");
        fprintf(fptr, szData);
#if defined (WIN32)
        if (lCount >0 )
            fprintf(fptr, " Waited %ld ms on the logfile",lCount*5);
#endif
        fprintf(fptr, "\n");
        fclose(fptr);
    }
    else							 /* JHP 040523 */
        /* If a failure in specified path, use default specs. */
    {
#if defined (WIN32)
        strcpy(szFile2,"c:\\BlockIP2.log");
#elif ! defined (__MVS__)
    strcpy(szFile2,"/var/mqm/exits/BlockIP2.log");
#else
    /* NWC Use DD statement format to hook to JCL defined files on MVS */
    strcpy(szFile2,"//DD:BLOCKLOG");
#endif
#if MQAT_DEFAULT==MQAT_OS400
        strcpy(szFile2,"QIBM/UserData/BLOCKIP2.LOG");
#endif

        if((fptr = fopen( szFile2,LOGMODE))!=NULL)
        {
            if (pWS->ReportLogFileError==0)
            {
                pWS->ReportLogFileError=1;	   /* not first anymore */
                fprintf(fptr,szDate);
                fprintf(fptr, "|");
                fprintf(fptr,szTime);
                fprintf(fptr, "|");
#if ! defined (__MVS__)
                fprintf(fptr, "BlockIP2 failed to open the specified logfile [%s], used default.\n",szFile);
#else
    if (pWS->nZOSconv==1)
        fprintf(fptr, "BlockIP2 failed to open the specified logfile <%s>, used default.\n",szFile);
    else
        fprintf(fptr, "BlockIP2 failed to open the specified logfile [%s], used default.\n",szFile);
#endif
            }

            fprintf(fptr,szDate);
            fprintf(fptr, "|");
            fprintf(fptr,szTime);
            fprintf(fptr, "|");
            fprintf(fptr, szData);
#if defined (WIN32)
            fprintf(fptr, " Waited %ld ms on the logfile",lCount*5);
#endif
            fprintf(fptr, "\n");
            fclose(fptr);
        }
        else							 /* JHP 040523 */
            /* If a failure in default path, use stdout specs. */
        {
            if (pWS->ReportLogFileError==0)
            {
                pWS->ReportLogFileError=1;	   /* not first anymore */
                fprintf(fptr,szDate);
                fprintf(fptr, "|");
                fprintf(fptr,szTime);
                fprintf(fptr, "|");
#if ! defined (__MVS__)
                fprintf(fptr, "BlockIP2 failed to open the specified logfile [%s], used default.\n",szFile);
#else
    if (pWS->nZOSconv==1)
        fprintf(fptr, "BlockIP2 failed to open the specified logfile <%s>, used default.\n",szFile);
    else
        fprintf(fptr, "BlockIP2 failed to open the specified logfile [%s], used default.\n",szFile);
#endif
            }

            printf(szDate);
            printf("|");
            printf(szTime);
            printf("|");
            printf(szData);
            printf("\n");
            fclose(fptr);
        }
    }
#if defined (WIN32)
    ReleaseMutex( pWS->hMutex );
#endif
#endif    /* NOT AS/400 */
}

/**
* WTODataE()
*
*
* Method to log data to WTO
*
*	 szData ASCII data to report
*
*
* @return void
*
* added JHP 040806
*/
void WTODataE(char * szData, workingStorage_type *pWS)
{
    char * ptr;
    char szData1[512];

#if defined(__MVS__)
    bi2wto_type  wtomsg;		 /* jhp 040805 */
    bi2wto_type  * pwtomsg; 	 /* jhp 040805 */
#endif


    ptr = strcpy(szData1,szData);
    ptr = strcat(szData1, " ");
    ptr = strcat(szData1, pWS->szWTOBuffErr );

#if defined(__MVS__)
    pwtomsg = &wtomsg;			 /* jhp 040805 */

    memset(pwtomsg -> msg,' ',WTOLENGTH);
    if (strlen(szData1) > WTOLENGTH)
        memcpy(pwtomsg -> msg,
               szData1,
               WTOLENGTH) ;
    else
        memcpy(pwtomsg -> msg,
               szData1,
               strlen(szData1));
    BLOCKIPW(pwtomsg);
#endif
}

/**
* WTODataOk()
*
*
* Method to log data to WTO
*
*	 szData ASCII data to report
*
*
* @return void
*
* added JHP 040806
*/
void WTODataOk(char * szData, workingStorage_type *pWS)
{
    char * ptr;
    char szData1[512];

#if defined(__MVS__)
    bi2wto_type  wtomsg;		 /* jhp 040805 */
    bi2wto_type  * pwtomsg; 	 /* jhp 040805 */
#endif

    ptr = strcpy(szData1,szData);
    ptr = strcat(szData1, " ");
    ptr = strcat(szData1, pWS->szWTOBuffOk );

#if defined(__MVS__)
    pwtomsg = &wtomsg;			 /* jhp 040805 */

    memset(pwtomsg -> msg,' ',WTOLENGTH);
    if (strlen(szData1) > WTOLENGTH)
        memcpy(pwtomsg -> msg,
               szData1,
               WTOLENGTH) ;
    else
        memcpy(pwtomsg -> msg,
               szData1,
               strlen(szData1));
    BLOCKIPW(pwtomsg);
#endif
}

/**
* SendLogMessage()
*
*
* Method to log data to EventList on windows
*
*	 szData ASCII data to report
*
*
* @return void
*
* added JHP 060214
*/
#if defined (WIN32)
void SendLogMessage (char *szData)
{
    HANDLE  hSource;
    char    *szList[1];

    szList[0]=szData;
    hSource = RegisterEventSource(NULL, "WMQ BlockIP2");
    if (hSource != NULL)  {
        ReportEvent(hSource,EVENTLOG_INFORMATION_TYPE,
                    0,1,NULL,1,0,szList,NULL);
        DeregisterEventSource(hSource);
    }
}
#endif



void translate (char *src, char *ny, char *old, workingStorage_type *pWS)
{
    if (pWS->nZOSconv==1)
    {
        while (*src)
        {
            if (*src=='[')
                *src='<';
            if (*src==']')
                *src='>';
            src++;
        }
    }
}

void translate2 (char *src, char *ny, char *old)
{
    while (*src)
    {
        if (*src==':')
            *src='_';
        if (*src=='\\')
            *src='_';
        if (*src==' ')
            *src='_';
        src++;
    }
}

#if !defined(__MVS__)
/* Check the active connections */
int CheckRunningChannels(MQCD *pChannelDefinition, workingStorage_type *pWS) {
    long lCurrChl;

    sprintf(pWS->szTmpBufff,"Entered: CheckRunningChannels");
    LogData(LOGDEBUG, pWS->szTmpBufff, pWS);

#if defined UNIX
    lCurrChl = UpdateShrMem(pChannelDefinition,(long*)1,pWS);
    if (lCurrChl < 0 )  /* number of channels over limit */
    {
        sprintf(pWS->szTmpBufff,"Connection refused, Channel [%.*s] ConName [%.*s] User [%.*s] requested by CheckRunningChannels()",
                rlnts(MQ_CHANNEL_NAME_LENGTH, pChannelDefinition->ChannelName),
                pChannelDefinition->ChannelName,
                rlnts(MQ_CONN_NAME_LENGTH, pWS->ConName),	/* NWC */
                pWS->ConName,
                rlnts(MQ_USER_ID_LENGTH, pChannelDefinition->RemoteUserIdentifier),
                pChannelDefinition->RemoteUserIdentifier);
        LogData( LOGDATA, pWS->szTmpBufff, pWS);
        sprintf(pWS->szTmpBufff,"MAX active channels set on [%s] was exceeded, limit was [%ld] Active was [%ld]. Connection was refused.",
                pWS->szChannelName,pWS->lMaxChannelActive,pWS->iCurrentActiveConnections);
        LogData(LOGDATA, pWS->szTmpBufff, pWS);


        return SEC_FAILURE;
    }
    if (lCurrChl == 0 ) /* Number of channels below limit, just return */
    {
        return SEC_OK;
    }

#else   /* ! UNIX (and not MVS) */
#if defined WIN32_SHM
    lCurrChl = UpdateShrMem(pChannelDefinition,(long*)1,pWS);
    if (lCurrChl < 0 )  /* number of channels over limit */
    {
        sprintf(pWS->szTmpBufff,"Connection refused, Channel [%.*s] ConName [%.*s] User [%.*s] requested by CheckRunningChannels()",
                rlnts(MQ_CHANNEL_NAME_LENGTH, pChannelDefinition->ChannelName),
                pChannelDefinition->ChannelName,
                rlnts(MQ_CONN_NAME_LENGTH, pWS->ConName),	/* NWC */
                pWS->ConName,
                rlnts(MQ_USER_ID_LENGTH, pChannelDefinition->RemoteUserIdentifier),
                pChannelDefinition->RemoteUserIdentifier);
        LogData( LOGDATA, pWS->szTmpBufff, pWS);
        sprintf(pWS->szTmpBufff,"MAX active channels set on [%s] was exceeded, limit was [%ld] Active was [%ld]. Connection was refused.",
                pWS->szChannelName,pWS->lMaxChannelActive,pWS->iCurrentActiveConnections);
        LogData(LOGDATA, pWS->szTmpBufff, pWS);


        return SEC_FAILURE;
    }
    if (lCurrChl == 0 ) /* Number of channels below limit, just return */
    {
        return SEC_OK;
    }

#else
    if (connectToQMgr(pWS) != MXC_RC_NONE)
    {
        return SEC_FAILURE;
    }
    sprintf(pWS->szTmpBufff,"Connected to QueueManager [%s]",pWS->szQMgr);
    LogData(LOGDEBUG, pWS->szTmpBufff, pWS);

    if (openStatusQ(pWS) != MXC_RC_NONE)
    {
        return SEC_FAILURE;
    }
    lCurrChl =  UpdateStatusQ(pChannelDefinition,(long*)-1,pWS);
    if (lCurrChl < 0 )  /* number of channels over limit */
    {
        if (closeStatusQ(pWS) != MXC_RC_NONE)
        {
            return SEC_FAILURE;
        }
        return SEC_FAILURE;
    }
    if (lCurrChl == 0 ) /* Number of channels below limit, just return */
    {
        if (closeStatusQ(pWS) != MXC_RC_NONE)
        {
            return SEC_FAILURE;
        }
        return SEC_OK;
    }
#endif
#endif

    /* Else it's time to update.. */
    LogData(LOGDEBUG,"CheckRunningChannels() Get... update", pWS);
#if !defined(__MVS__)
    if (getActualCurrentNumberOfRunningChannels(&lCurrChl, pWS) != MXC_RC_NONE)
#else
    if (getActualCurrentNumberOfRunningChannelszOS(&lCurrChl, pWS) != MXC_RC_NONE)
#endif
    {
        sprintf(pWS->szTmpBufff,"Connection refused, Channel [%.*s] ConName [%.*s] User [%.*s] requested by CheckRunningChannels()",
                rlnts(MQ_CHANNEL_NAME_LENGTH, pChannelDefinition->ChannelName),
                pChannelDefinition->ChannelName,
                rlnts(MQ_CONN_NAME_LENGTH, pWS->ConName),	/* NWC */
                pWS->ConName,
                rlnts(MQ_USER_ID_LENGTH, pChannelDefinition->RemoteUserIdentifier),
                pChannelDefinition->RemoteUserIdentifier);
        LogData( LOGDATA, pWS->szTmpBufff, pWS);
        sprintf(pWS->szTmpBufff,"getActualCurrentNumberOfRunningChannels failed.");
        LogData(LOGDATA, pWS->szTmpBufff, pWS);

        lCurrChl =  UpdateStatusQ(pChannelDefinition,(long*)lCurrChl,pWS);
        if (closeStatusQ(pWS) != MXC_RC_NONE)
        {
            return SEC_FAILURE;
        }
        return SEC_FAILURE;
    }
    else
    {
        if (lCurrChl+1 > pWS->lMaxChannelActive)
        {
            sprintf(pWS->szTmpBufff,"Connection refused, Channel [%.*s] ConName [%.*s] User [%.*s] requested by CheckRunningChannels()",
                    rlnts(MQ_CHANNEL_NAME_LENGTH, pChannelDefinition->ChannelName),
                    pChannelDefinition->ChannelName,
                    rlnts(MQ_CONN_NAME_LENGTH, pWS->ConName),	/* NWC */
                    pWS->ConName,
                    rlnts(MQ_USER_ID_LENGTH, pChannelDefinition->RemoteUserIdentifier),
                    pChannelDefinition->RemoteUserIdentifier);
            LogData( LOGDATA, pWS->szTmpBufff, pWS);
            sprintf(pWS->szTmpBufff,"MAX active channels set on [%s] was exceeded, limit was [%ld] Active was [%ld]. Connection was refused.",
                    pWS->szChannelName,pWS->lMaxChannelActive,lCurrChl);
            LogData(LOGDATA, pWS->szTmpBufff, pWS);
            lCurrChl =  UpdateStatusQ(pChannelDefinition,(long*)lCurrChl,pWS);
            if (closeStatusQ(pWS) != MXC_RC_NONE)
            {
                return SEC_FAILURE;
            }
            return SEC_FAILURE;
        }
        lCurrChl =  UpdateStatusQ(pChannelDefinition,(long*)lCurrChl,pWS);
    }
    if (closeStatusQ(pWS) != MXC_RC_NONE)
    {
        return SEC_FAILURE;
    }
    sprintf(pWS->szTmpBufff,"Max Active channels check passed. Number of active channels [%ld]",lCurrChl);
    LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
    return SEC_OK;
}
#endif


#if defined(__MVS__)
/* Check the active connections */
int CheckRunningChannels(MQCD *pChannelDefinition, workingStorage_type *pWS) {
    long lCurrChl;
    lCurrChl =  UpdateStatusQ(pChannelDefinition,(long*)lCurrChl,pWS);
    sprintf(pWS->szTmpBufff,"UpdateStatusQ gave [%ld]",lCurrChl);
    LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
    if (lCurrChl < 0) {
        return SEC_FAILURE;
    }
    else {
        return SEC_OK;
    }
}
#endif



#if !defined(__MVS__)

/* start of inquire channel usage
**********************************************************************************************************
* Use the admin interface to query the channel status of instances of the channel in question, and count the number with MQCHS_RUNNING
*
* arg_plNumberOfChannels			number of extant channels with the relevant SvrConn name
**********************************************************************************************************
*/
long	getActualCurrentNumberOfRunningChannels( long* arg_plNumberOfChannels, workingStorage_type *pWS){

    MQLONG	reasonCode;
    MQLONG	completionCode;
    long	lRc=MXC_RC_NONE;

    MQHBAG adminBag = MQHB_UNUSABLE_HBAG;
    MQHBAG responseBag = MQHB_UNUSABLE_HBAG;
    MQHBAG qAttrsBag = MQHB_UNUSABLE_HBAG;


    sprintf(pWS->szTmpBufff,"Entered: getActualCurrentNumberOfRunningChannels()");
    LogData(LOGDEBUG, pWS->szTmpBufff, pWS);

    *arg_plNumberOfChannels = 0;

    do{
        lRc = bagSetup(&adminBag,&responseBag,&qAttrsBag, pWS);
        if (lRc != MXC_RC_NONE){
            sprintf(pWS->szTmpBufff,"bagSetup gave. Reason=%ld",lRc);
            LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
            break;
        }

        mqAddString(adminBag, MQCACH_CHANNEL_NAME, MQBL_NULL_TERMINATED, pWS->szChannelName, &completionCode, &reasonCode);
        if (reasonCode != MQRC_NONE){
            sprintf(pWS->szTmpBufff,"Error with mqAddString for MQCACH_CHANNEL_NAME of %s. Reason=%ld",pWS->szChannelName,reasonCode);
            LogData(LOGDATA, pWS->szTmpBufff, pWS);
            lRc = MXC_RC_ERROR;
            break;
        }

        mqAddInquiry(adminBag, MQIACH_CHANNEL_STATUS, &completionCode, &reasonCode);
        if (reasonCode != MQRC_NONE){
            sprintf(pWS->szTmpBufff,"Error with mqAddInquiry for MQIACH_CHANNEL_STATUS. Reason=%ld",reasonCode);
            LogData(LOGDATA, pWS->szTmpBufff, pWS);
            lRc = MXC_RC_ERROR;
            break;
        }

        mqExecute(pWS->hConn,
                  MQCMD_INQUIRE_CHANNEL_STATUS,
                  MQHB_NONE,
                  adminBag,
                  responseBag,
                  MQHO_NONE,
                  MQHO_NONE,
                  &completionCode,
                  &reasonCode);

        if (reasonCode != MQRC_NONE){
            if (reasonCode == MQRCCF_COMMAND_FAILED){
                MQLONG mqExecuteCC;
                MQLONG mqExecuteRC;
                MQHBAG errorBag;

                mqInquireBag(responseBag, MQHA_BAG_HANDLE, 0, &errorBag, &completionCode, &reasonCode);
                if (reasonCode != MQRC_NONE){
                    sprintf(pWS->szTmpBufff,"Error with mqInquireBag . Reason=%ld",reasonCode);
                    LogData(LOGDATA, pWS->szTmpBufff, pWS);
                    lRc = MXC_RC_ERROR;
                    break;
                }
                mqInquireInteger(errorBag,
                                 MQIASY_COMP_CODE,
                                 MQIND_NONE,
                                 &mqExecuteCC,
                                 &completionCode,
                                 &reasonCode );
                if (reasonCode != MQRC_NONE){
                    sprintf(pWS->szTmpBufff,"Error with mqInquireInteger (CC). Reason=%ld",reasonCode);
                    LogData(LOGDATA, pWS->szTmpBufff, pWS);
                    lRc = MXC_RC_ERROR;
                    break;
                }

                mqInquireInteger(errorBag,
                                 MQIASY_REASON,
                                 MQIND_NONE,
                                 &mqExecuteRC,
                                 &completionCode,
                                 &reasonCode);
                if (reasonCode != MQRC_NONE){
                    sprintf(pWS->szTmpBufff,"Error with mqInquireInteger (RC). Reason=%ld",reasonCode);
                    LogData(LOGDATA, pWS->szTmpBufff, pWS);
                    lRc = MXC_RC_ERROR;
                    break;
                }

                if (mqExecuteRC == MQRCCF_CHL_STATUS_NOT_FOUND){
                    *arg_plNumberOfChannels = 0;
                }else{
                    sprintf(pWS->szTmpBufff,"Error with mqExecute. ExecuteReason=%ld",mqExecuteRC);
                    LogData(LOGDATA, pWS->szTmpBufff, pWS);
                    lRc = MXC_RC_ERROR;
                    break;
                }
            }else{
                sprintf(pWS->szTmpBufff,"Error with mqExecute. Reason=%ld",reasonCode);
                LogData(LOGDATA, pWS->szTmpBufff, pWS);
                lRc = MXC_RC_ERROR;
                break;
            }
        }

        if (lRc == MXC_RC_NONE){
            MQLONG	lNumberOfResponseBags;
            mqCountItems(responseBag, MQHA_BAG_HANDLE, &lNumberOfResponseBags, &completionCode, &reasonCode);
            if (reasonCode != MQRC_NONE){
                sprintf(pWS->szTmpBufff,"Error with mqCountItems. Reason=%ld",reasonCode);
                LogData(LOGDATA, pWS->szTmpBufff, pWS);
                lRc = MXC_RC_ERROR;
                break;
            }
            if (lNumberOfResponseBags){
                long	l1;
                MQLONG	lItemCount;

                /* one bag per channel instance - loop thru them */
                for (l1=0;l1< lNumberOfResponseBags;l1++){
                    mqInquireBag(responseBag, MQHA_BAG_HANDLE, l1, &qAttrsBag, &completionCode, &reasonCode);
                    if (reasonCode != MQRC_NONE){
                        sprintf(pWS->szTmpBufff,"Error with mqInquireBag. Reason=%ld",reasonCode);
                        LogData(LOGDATA, pWS->szTmpBufff, pWS);
                        lRc = MXC_RC_ERROR;
                        break;
                    }
                    mqCountItems(qAttrsBag,MQSEL_ALL_SELECTORS ,&lItemCount,&completionCode, &reasonCode);
                    if (reasonCode != MQRC_NONE){
                        sprintf(pWS->szTmpBufff,"Error with mqCountItems . Reason=%ld",reasonCode);
                        LogData(LOGDATA, pWS->szTmpBufff, pWS);
                        lRc = MXC_RC_ERROR;
                        break;
                    }
                    if (lItemCount){
                        MQLONG	lStatus;
                        mqInquireInteger(qAttrsBag, MQIACH_CHANNEL_STATUS, MQIND_NONE,&lStatus,
                                         &completionCode, &reasonCode);
                        if (reasonCode != MQRC_NONE){
                            sprintf(pWS->szTmpBufff,"Error with mqInquireInteger MQIACH_CHANNEL_STATUS. Reason=%ld",reasonCode);
                            LogData(LOGDATA, pWS->szTmpBufff, pWS);
                            lRc = MXC_RC_ERROR;
                            break;
                        }

                        /* If the status is RUNNING, then increment counter */
                        if (lStatus == MQCHS_RUNNING){
                            (*arg_plNumberOfChannels)++;
                        }
                    }
                }
            }
        }
    }while(0);

    /* delete the alloacted dataBags JHP 060218 */
    if (adminBag != MQHB_UNUSABLE_HBAG) {
        mqDeleteBag (&adminBag, &completionCode, &reasonCode);
        if (reasonCode != MQRC_NONE){
            sprintf(pWS->szTmpBufff,"Error mqDeleteBag(adminBag). Reason=%ld",reasonCode);
            LogData(LOGDATA, pWS->szTmpBufff, pWS);
        }
    }
    if (responseBag != MQHB_UNUSABLE_HBAG) {
        mqDeleteBag (&responseBag, &completionCode, &reasonCode);
        if (reasonCode != MQRC_NONE){
            sprintf(pWS->szTmpBufff,"Error mqDeleteBag(responseBag). Reason=%ld",reasonCode);
            LogData(LOGDATA, pWS->szTmpBufff, pWS);
        }
    }
    /* end of change delete the alloacted dataBags JHP 060218 */
    return lRc;
}

/*
**********************************************************************************************************
* setup admin interface objects
**********************************************************************************************************
*/
long	bagSetup(MQHBAG*const arg_pAdminBag,MQHBAG*const arg_pResponseBag,MQHBAG*const arg_pQAttrsBag, workingStorage_type *pWS){

    long	lRc = MXC_RC_NONE;

    MQLONG reasonCode;
    MQLONG completionCode;

    do{
        if (*arg_pAdminBag != MQHB_UNUSABLE_HBAG){
            mqClearBag(*arg_pAdminBag, &completionCode, &reasonCode);
            if (reasonCode != MQRC_NONE){
                sprintf(pWS->szTmpBufff,"Error with mqClearBag . Reason=%ld",reasonCode);
                LogData(LOGDATA, pWS->szTmpBufff, pWS);
                lRc = MXC_RC_ERROR;
                break;
            }
        }
        mqCreateBag(MQCBO_ADMIN_BAG, arg_pAdminBag, &completionCode, &reasonCode);
        if (reasonCode != MQRC_NONE){
            sprintf(pWS->szTmpBufff,"Error with mqCreateBag . Reason=%ld",reasonCode);
            LogData(LOGDATA, pWS->szTmpBufff, pWS);
            lRc = MXC_RC_ERROR;
            break;
        }

        if (*arg_pResponseBag != MQHB_UNUSABLE_HBAG){
            mqClearBag(*arg_pResponseBag, &completionCode, &reasonCode);
            if (reasonCode != MQRC_NONE){
                sprintf(pWS->szTmpBufff,"Error with mqClearBag . Reason=%ld",reasonCode);
                LogData(LOGDATA, pWS->szTmpBufff, pWS);
                lRc = MXC_RC_ERROR;
                break;
            }
        }
        mqCreateBag(MQCBO_ADMIN_BAG, arg_pResponseBag, &completionCode, &reasonCode);
        if (reasonCode != MQRC_NONE){
            sprintf(pWS->szTmpBufff,"Error with mqCreateBag . Reason=%ld",reasonCode);
            LogData(LOGDATA, pWS->szTmpBufff, pWS);
            lRc = MXC_RC_ERROR;
            break;
        }

        if (*arg_pQAttrsBag != MQHB_UNUSABLE_HBAG){
            mqClearBag(*arg_pQAttrsBag, &completionCode, &reasonCode);
            if (reasonCode != MQRC_NONE){
                sprintf(pWS->szTmpBufff,"Error with mqClearBag . Reason=%ld",reasonCode);
                LogData(LOGDATA, pWS->szTmpBufff, pWS);
                lRc = MXC_RC_ERROR;
                break;
            }
            *arg_pQAttrsBag = MQHB_UNUSABLE_HBAG;
        }
    }while(0);

    return lRc;

}


#endif

#if defined(__MVS__)

/* start of inquire channel usage
**********************************************************************************************************
* Use the admin interface to query the channel status of instances of the channel in question, and count the number with MQCHS_RUNNING
*
* arg_plNumberOfChannels			number of extant channels with the relevant SvrConn name
**********************************************************************************************************
*/
long	getActualCurrentNumberOfRunningChannelszOS( long* arg_plNumberOfChannels, workingStorage_type *pWS){

    MQLONG	reasonCode=0;
    MQLONG	completionCode;

    MQGMO	gmo = {MQGMO_DEFAULT};
    MQMD	md = {MQMD_DEFAULT};

    long	lOpenRc,lRc=MXC_RC_NONE;
    long	buflen, messlen;
    char	buffer[500];
    char *p;
    char *u;
    char *strtok_cb;

    sprintf(pWS->szTmpBufff,"Entered: getActualCurrentNumberOfRunningChannelszOS()",pWS);
    LogData(LOGDEBUG, pWS->szTmpBufff,pWS);

    *arg_plNumberOfChannels = 0;

    do{
        lRc = chsSetup(pWS);
        lOpenRc=lRc;

        /*	LogData(LOGDEBUG, "Back from chlSetup()");*/

        if (lRc == MXC_RC_NONE)
        {
            lRc = chsSendDisplayCHStatus("SYSTEM.COMMAND.INPUT",pWS->szReplyQ,pWS);
        }
        if (lRc == MXC_RC_NONE)
        {
            gmo.Options = MQGMO_WAIT +     /* wait for new messages           */
                          MQGMO_NO_SYNCPOINT;  /* no cyncpointing.. */
            gmo.WaitInterval = 500;        /* .5 second limit for waiting     */

            do
            {
                memcpy(md.MsgId, MQMI_NONE, sizeof(md.MsgId));
                memcpy(md.CorrelId, MQCI_NONE, sizeof(md.CorrelId));
                buflen = sizeof(buffer) - 1; /* buffer size available for GET   */

                MQGET(pWS->hConn,          /* connection handle                 */
                      pWS->Hobj,           /* object handle                     */
                      &md,                 /* message descriptor                */
                      &gmo,                /* get message options               */
                      buflen,              /* buffer length                     */
                      buffer,              /* message buffer                    */
                      &messlen,            /* message length                    */
                      &completionCode,     /* completion code                   */
                      &reasonCode);        /* reason code                       */
                if (reasonCode == 0)
                {
                    if (memcmp(buffer,"CSQ9022I",8)==0)  /* NORMAL COMPLETION -> we're done */
                    {
                        continue;
                    }
                    if (memcmp(buffer,"CSQM425I",8)==0)  /* Server-connection Channel status */
                    {
                        buffer[messlen] = '\00';
                        sprintf(pWS->szTmpBufff,"message received %s",buffer);
                        LogData(LOGDEBUG, pWS->szTmpBufff,pWS);
                        p = strstr(buffer," STATUS(");
                        if (p != NULL)
                        {
                            p=p+8;
                            u=(char*)strtok_r(p,")",&strtok_cb);
                            if(u != NULL)
                            {
                                if (strlen(u) > 6)
                                {
                                    if (memcmp(u,"RUNNING",7) == 0)
                                        (*arg_plNumberOfChannels)++;
                                }
                            }

                        }
                        /* If the status is RUNNING, then increment counter */
                        /*						if (lStatus == MQCHS_RUNNING){*/
                        /*							(*arg_plNumberOfChannels)++;*/
                        /*						}*/
                    }
                }

            }  while (reasonCode == 0);
        }
        if (lOpenRc == MXC_RC_NONE)
        {
            lRc = closeReplyQ(pWS);
        }

    } while(0);

    return lRc;
}


/*
**********************************************************************************************************
* setup Reply queue.
**********************************************************************************************************
*/
long	chsSetup(workingStorage_type *pWS) {

    long	lRc = MXC_RC_NONE;

    MQLONG reasonCode;
    MQLONG completionCode;
    MQLONG O_options;               /* MQOPEN options                */
    MQOD	od = {MQOD_DEFAULT};       /* Object Descriptor             */

    O_options = MQOO_INPUT_SHARED	/* open queue for input     */
                | MQOO_FAIL_IF_QUIESCING /* but not if MQM stopping   */
                ;
    /*   LogData(LOGDEBUG, "Before MQOPEN");
    */
    strcpy(od.ObjectName,"SYSTEM.COMMAND.REPLY.MODEL");
    /*	strcpy(odReplyQ.DynamicQName,"SYSTEM.BlockIP2.*"); */
    strcpy(od.DynamicQName,"SYSTEM.CSQOREXX.*");   /* test only */

    MQOPEN(pWS->hConn,			/* connection handle            */
           &od,					/* object descriptor for queue  */
           O_options,			/* open options                 */
           &pWS->Hobj, 			/* object handle                */
           &completionCode,	/* MQOPEN completion code       */
           &reasonCode);		/* reason code                  */

    /* report reason, if any; stop if failed      */
    if (reasonCode != MQRC_NONE)
    {
        sprintf(pWS->szTmpBufff,"MQOPEN ended with reason code %d", reasonCode);
        LogData(LOGDATA, pWS->szTmpBufff,pWS);
        lRc = MXC_RC_ERROR ;
    }

    if (completionCode == MQCC_FAILED)
    {
        sprintf(pWS->szTmpBufff,"unable to open queue for output");
        LogData(LOGDATA, pWS->szTmpBufff,pWS);
        lRc = MXC_RC_ERROR ;
    }
    /*    nullDelimit(pWS->szReplyQ,od.ObjectName,MQ_OBJECT_NAME_LENGTH+1);*/
    memset(pWS->szReplyQ,'\00',sizeof(pWS->szReplyQ));
    memcpy(pWS->szReplyQ,od.ObjectName,rlnts(MQ_OBJECT_NAME_LENGTH, od.ObjectName));

    /*  	sprintf(pWS->szTmpBufff,"chsSetup() Hobj is %ld ReplyQ %s",Hobj,szReplyQ);
    	LogData(LOGDEBUG, pWS->szTmpBufff);
    */
    return lRc;

}

/*
**********************************************************************************************************
* setup Reply queue.
**********************************************************************************************************
*/
long chsSendDisplayCHStatus(char * cmdq, char * replyq, workingStorage_type *pWS) {

    MQLONG	reason;
    MQLONG	compCode;

    MQOD	od = {MQOD_DEFAULT};
    MQMD	md = {MQMD_DEFAULT};
    MQPMO   pmo = {MQPMO_DEFAULT};

    long	BufferLength;
    char	Buffer[200];
    long	lRc=MXC_RC_NONE;

    memcpy(od.ObjectName,cmdq,MQ_OBJECT_NAME_LENGTH);
    memcpy(md.ReplyToQ,replyq,MQ_OBJECT_NAME_LENGTH);

    memcpy(md.Format,           /* character string format            */
           MQFMT_STRING, (size_t)MQ_FORMAT_LENGTH);
    md.MsgType=MQMT_REQUEST;

    strcpy(Buffer,"DISPLAY CHSTATUS(");
    strcat(Buffer,pWS->szChannelName);
    strcat(Buffer,") CURRENT");
    BufferLength = strlen(Buffer);
    /*
    	strcpy(Buffer,"DISPLAY CHSTATUS(TO.QM.QSAT) CURRENT");
    	BufferLength = strlen(Buffer);*/

    pmo.Options= MQPMO_NO_SYNCPOINT;

    sprintf(pWS->szTmpBufff,"MQPUT1 Sending [%s]", Buffer);
    LogData(LOGDEBUG, pWS->szTmpBufff,pWS);

    MQPUT1 (pWS->hConn,
            &od,
            &md,
            &pmo,
            BufferLength,
            Buffer,
            &compCode,
            &reason);
    /* report reason, if any; stop if failed      */
    if (reason != MQRC_NONE)
    {
        sprintf(pWS->szTmpBufff,"MQPUT1 ended with reason code %d", reason);
        LogData(LOGDATA, pWS->szTmpBufff,pWS);
        lRc = MXC_RC_ERROR ;
    }

    if (compCode == MQCC_FAILED)
    {
        sprintf(pWS->szTmpBufff,"unable to put on COMMAND queue");
        LogData(LOGDATA, pWS->szTmpBufff,pWS);
        lRc = MXC_RC_ERROR ;
    }
    return lRc;

}
/*
**********************************************************************************************************
* Close reply queue
*
**********************************************************************************************************
*/
long	closeReplyQ(workingStorage_type *pWS){
    long	lRc = MXC_RC_NONE;

    MQLONG	reason;
    MQLONG	compCode;
    MQLONG   C_options;              /* MQCLOSE options                */

    /*	sprintf(pWS->szTmpBufff,"Close and delete the Reply queue");
    	LogData(LOGDEBUG, pWS->szTmpBufff);
    */
    C_options = MQCO_DELETE_PURGE;   /* delete the queue and its data */

    MQCLOSE(pWS->hConn,
            &pWS->Hobj,
            C_options,
            &compCode,
            &reason);

    if (reason == MQRC_NONE){
    }else{
        lRc = MXC_RC_ERROR;
        sprintf(pWS->szTmpBufff,"Error closing reply queue %s. Reason=%ld",pWS->szReplyQ,reason);
        LogData(LOGDATA, pWS->szTmpBufff,pWS);
    }
    /*	LogData(LOGDEBUG, "Close reply Q done....");*/

    return lRc;
}

#endif

/*
**********************************************************************************************************
* Connect
*
**********************************************************************************************************
*/
long	connectToQMgr(workingStorage_type *pWS){
    long	lRc = MXC_RC_NONE;

    MQLONG	reason;
    MQLONG	compCode;

    sprintf(pWS->szTmpBufff,"Connect to default Qmgr %s.",pWS->szQMgr);
    LogData(LOGDEBUG, pWS->szTmpBufff, pWS);

    MQCONN(pWS->szQMgr,
           &pWS->hConn,
           &compCode,
           &reason);

    sprintf(pWS->szTmpBufff,"Debug connecting to %s. Reason=%ld hConn=%ld",pWS->szQMgr,reason,pWS->hConn);
    LogData(LOGDEBUG, pWS->szTmpBufff, pWS);

    if (reason == MQRC_ALREADY_CONNECTED || reason == MQRC_NONE){
    }else{
        lRc = MXC_RC_ERROR;
        sprintf(pWS->szTmpBufff,"Error connecting to %s. Reason=%ld",pWS->szQMgr,reason);
        LogData(LOGDATA, pWS->szTmpBufff, pWS);
    }
    /*	LogData(LOGDEBUG, "Connect to QMgr done....");*/

    return lRc;
}


#if !defined(__MVS__)

/*
**********************************************************************************************************
* Open Status queue.
**********************************************************************************************************
*/
long	openStatusQ(workingStorage_type *pWS) {

    long	lRc = MXC_RC_NONE;

    MQLONG reasonCode;
    MQLONG completionCode;
    MQLONG O_options;              /* MQOPEN options                */
    MQOD	od = {MQOD_DEFAULT};    /* Object Descriptor             */

    LogData(LOGDEBUG,"In openStatusQ", pWS);

    O_options = MQOO_INPUT_EXCLUSIVE /*SHARED*/	/* open queue for input, exclusive, and output */
                | MQOO_OUTPUT
                | MQOO_FAIL_IF_QUIESCING /* but not if MQM stopping   */
                ;
    LogData(LOGDEBUG, "Before MQOPEN status Q", pWS);

    strcpy(od.ObjectName,pWS->szStatusQ);
    do {
        MQOPEN(pWS->hConn,			/* connection handle            */
               &od,		/* object descriptor for queue  */
               O_options,		/* open options                 */
               &pWS->HobjStat,		/* object handle                */
               &completionCode,	/* MQOPEN completion code       */
               &reasonCode);		/* reason code                  */
    } while (reasonCode==MQRC_OBJECT_IN_USE);

    /* report reason, if any; stop if failed      */
    if (reasonCode != MQRC_NONE)
    {
        sprintf(pWS->szTmpBufff,"MQOPEN status Q ended with reason code %d on %s hConn=%ld", reasonCode,pWS->szStatusQ,pWS->hConn);
        LogData(LOGDATA, pWS->szTmpBufff, pWS);
        lRc = MXC_RC_ERROR ;
    }
    LogData(LOGDEBUG,"In leave openStatusQ", pWS);

    return lRc;
}

/*
**********************************************************************************************************
* Close status queue
*
**********************************************************************************************************
*/
long	closeStatusQ(workingStorage_type *pWS){
    long	lRc = MXC_RC_NONE;

    MQLONG	reason;
    MQLONG	compCode;
    MQLONG   C_options;              /* MQCLOSE options                */

    sprintf(pWS->szTmpBufff,"Close Status queue");
    LogData(LOGDEBUG, pWS->szTmpBufff, pWS);

    C_options = MQCO_NONE;   /* just close the queue */

    MQCLOSE(pWS->hConn,
            &pWS->HobjStat,
            C_options,
            &compCode,
            &reason);

    if (reason == MQRC_NONE){
    }else{
        lRc = MXC_RC_ERROR;
        sprintf(pWS->szTmpBufff,"Error closing status queue %s. Reason=%ld",pWS->szStatusQ,reason);
        LogData(LOGDATA, pWS->szTmpBufff, pWS);
    }
    LogData(LOGDEBUG, "Close status Q done....", pWS);

    return lRc;
}
#endif

#if !defined(__MVS__)

/*
**********************************************************************************************************
* Update status queue
*
**********************************************************************************************************
*/
long	UpdateStatusQ(MQCD *pChannelDefinition,  long* what2do, workingStorage_type *pWS){
    long	lRc = -1;
    MQLONG      buflen;
    MQLONG      messlen;

    MQLONG	reason;
    MQLONG	compCode;
    char        buffer[30];
    currChl_type *b;

    MQGMO	gmo = {MQGMO_DEFAULT};
    MQPMO	pmo = {MQPMO_DEFAULT};
    MQMD	md = {MQMD_DEFAULT};

    LogData(LOGDEBUG, "Update status Q entered....", pWS);


    gmo.Options = MQGMO_WAIT |         /* wait for new messages           */
                  MQGMO_NO_SYNCPOINT;  /* no cyncpointing.. */
    gmo.WaitInterval = 50;             /* nil second limit for waiting     */

    memcpy(md.MsgId, pChannelDefinition->ChannelName, sizeof(pChannelDefinition->ChannelName));
    memcpy(md.CorrelId, MQCI_NONE, sizeof(md.CorrelId));
    buflen = sizeof(buffer) - 1; /* buffer size available for GET   */

    LogData(LOGDEBUG, "Before MQ Get....", pWS);
    MQGET(pWS->hConn,          /* connection handle                 */
          pWS->HobjStat,       /* object handle                     */
          &md,                 /* message descriptor                */
          &gmo,                /* get message options               */
          buflen,              /* buffer length                     */
          buffer,              /* message buffer                    */
          &messlen,            /* message length                    */
          &compCode,           /* completion code                   */
          &reason);            /* reason code                       */
    if (reason == 0)
    {
        sprintf(pWS->szTmpBufff,"MQGET status queue. Reason=%ld",reason);
        LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
    }
    else
    {
        if (reason == MQRC_NO_MSG_AVAILABLE)
        {
            if (what2do== (long*)-1) return 1;
            lRc=1;  /* insert status record */
        }
        else
        {
            sprintf(pWS->szTmpBufff,"Error MQGET status queue Reason=%ld",reason);
            LogData(LOGDATA, pWS->szTmpBufff, pWS);
            return -1;
        }
    }
    b=(struct CURRCHLDATA*)&buffer;
    sprintf(pWS->szTmpBufff,"Buffer was lim=%ld curr=%ld (Update)",b->limCurrChl,b->CurrChannels);
    LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
    if (what2do != (long*)-1 && reason == MQRC_NO_MSG_AVAILABLE) { /* init structure */
        LogData(LOGDEBUG, "Init structure....", pWS);
        memset(buffer,'\00',sizeof(buffer));
        b->limCurrChl   = pWS->lMaxChannelActive;
        b->CurrChannels = (long)what2do;  /* insert monitored value */
        b->MaxChannels = (long)what2do;
        md.Expiry = pWS->lExpireValue;
    }
    LogData(LOGDEBUG, "Updating status Q started....", pWS);
    if (b->CurrChannels+1 > b->limCurrChl)
    {
        sprintf(pWS->szTmpBufff,"Channel limit reached lim=%ld curr=%ld",b->limCurrChl,b->CurrChannels);
        LogData(LOGDATA, pWS->szTmpBufff, pWS);
        lRc=-1;
    }
    else {
        b->CurrChannels++;  /* Adjust counter */
        lRc=0;
        if (b->CurrChannels > b->MaxChannels) {
            b->MaxChannels = b->CurrChannels;
        }
    }
    messlen=sizeof(currChl_type);
    pmo.Options = MQPMO_NO_SYNCPOINT;  /* no cyncpointing.. */
    LogData(LOGDEBUG, "MQPUT next....", pWS);
    MQPUT(pWS->hConn,
          pWS->HobjStat,            /* object handle                     */
          &md,                 /* message descriptor                */
          &pmo,                /* put message options               */
          messlen,              /* buffer length                     */
          buffer,              /* message buffer                    */
          &compCode,           /* completion code                   */
          &reason);            /* reason code                       */
    sprintf(pWS->szTmpBufff,"MQPUT status queue Reason=%ld",reason);
    LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
    return lRc;
}


long	deductStatusQ(MQCD *pChannelDefinition, workingStorage_type *pWS){
    long	lRc = -1;
    long        buflen;
    long        messlen;

    MQLONG	reason;
    MQLONG	compCode;
    char        buffer[30];
    currChl_type *b;

    MQGMO	gmo = {MQGMO_DEFAULT};
    MQPMO	pmo = {MQPMO_DEFAULT};
    MQMD	md = {MQMD_DEFAULT};

    LogData(LOGDEBUG, "Deduct status Q entered....", pWS);

#if defined UNIX
    /*lCurrChl =*/ UpdateShrMem(pChannelDefinition,(long*)-1,pWS);
#else
#if defined WIN32_SHM
    /*lCurrChl =*/ UpdateShrMem(pChannelDefinition,(long*)-1,pWS);
#else
    if (connectToQMgr(pWS) != MXC_RC_NONE)
        return 0;
    if (openStatusQ(pWS) != MXC_RC_NONE)
        return 0;

    gmo.Options = MQGMO_NO_WAIT |         /* wait for new messages           */
                  MQGMO_NO_SYNCPOINT;  /* no cyncpointing.. */
    gmo.WaitInterval = 50;             /* nil second limit for waiting     */

    memcpy(md.MsgId, pChannelDefinition->ChannelName, sizeof(pChannelDefinition->ChannelName));
    memcpy(md.CorrelId, MQCI_NONE, sizeof(md.CorrelId));
    buflen = sizeof(buffer) - 1; /* buffer size available for GET   */

    LogData(LOGDEBUG, "Before MQ Get....", pWS);
    MQGET(pWS->hConn,               /* connection handle                 */
          pWS->HobjStat,            /* object handle                     */
          &md,                 /* message descriptor                */
          &gmo,                /* get message options               */
          buflen,              /* buffer length                     */
          buffer,              /* message buffer                    */
          &messlen,            /* message length                    */
          &compCode,           /* completion code                   */
          &reason);            /* reason code                       */
    sprintf(pWS->szTmpBufff,"MQGET status queue. Reason=%ld [%s]",reason,buffer);
    LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
    if (reason == 0)
    {
        sprintf(pWS->szTmpBufff,"MQGET status queue success. Reason=%ld",reason);
        LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
    }
    else
    {
        if (closeStatusQ(pWS) != MXC_RC_NONE)
        {
            return 0;
        }
        return 0;
    }
    /*    b=&buffer;*/
    b=(struct CURRCHLDATA*)&buffer;
    sprintf(pWS->szTmpBufff,"Buffer was lim=%ld curr=%ld",b->limCurrChl,b->CurrChannels);
    LogData(LOGDEBUG, pWS->szTmpBufff, pWS);

    if (b->CurrChannels>0)
        b->CurrChannels--;  /* Adjust counter */
    messlen=sizeof(currChl_type);
    pmo.Options = MQPMO_NO_SYNCPOINT;  /* no cyncpointing.. */
    LogData(LOGDEBUG, "MQPUT next....", pWS);
    MQPUT(pWS->hConn,
          pWS->HobjStat,       /* object handle                     */
          &md,                 /* message descriptor                */
          &pmo,                /* put message options               */
          messlen,             /* buffer length                     */
          buffer,              /* message buffer                    */
          &compCode,           /* completion code                   */
          &reason);            /* reason code                       */
    sprintf(pWS->szTmpBufff,"MQPUT status queue Reason=%ld",reason);
    LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
    if (closeStatusQ(pWS) != MXC_RC_NONE)
    {
        return 0;
    }
#endif
#endif
    return 0;
}

#endif

#if defined(__MVS__)

/*
**********************************************************************************************************
* Update status queue
*
**********************************************************************************************************
*/
long	UpdateStatusQ(MQCD *pChannelDefinition,  long* what2do, workingStorage_type *pWS){
    long	lRc = -1;
    bi2ds_type  dsdata;
    bi2ds_type  * pdsdata;

    pdsdata = &dsdata;

    memset(pdsdata -> channel,' ',MQ_CHANNEL_NAME_LENGTH);
    memcpy(pdsdata -> channel,
           pChannelDefinition->ChannelName,
           MQ_CHANNEL_NAME_LENGTH) ;
    memset(pdsdata->function,'+',1);

    pdsdata->limit   = pWS->lMaxChannelActive;
    pdsdata->current = (long)what2do;  /* insert monitored value */
    pdsdata->maxrunning = (long)what2do;

    BLOCKIPS(pdsdata);
    sprintf(pWS->szTmpBufff,"BLOCKIPS gave= [%.*s] %ld",8, pdsdata->status,strcmp(pdsdata->status,"OK"));
    LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
    if (strcmp(pdsdata->status,"OK") == -1) {
        sprintf(pWS->szTmpBufff,"Channel limit reached lim=%ld curr=%ld",pdsdata->limit,pdsdata->current);
        LogData(LOGDATA, pWS->szTmpBufff, pWS);
        return lRc;
    }
    return pdsdata->current;
}


long	deductStatusQ(MQCD *pChannelDefinition, workingStorage_type *pWS){
    bi2ds_type  dsdata;
    bi2ds_type  * pdsdata;

    pdsdata = &dsdata;

    memset(pdsdata -> channel,' ',MQ_CHANNEL_NAME_LENGTH);
    memcpy(pdsdata -> channel,
           pChannelDefinition->ChannelName,
           MQ_CHANNEL_NAME_LENGTH) ;
    memset(pdsdata->function,'-',1);
    BLOCKIPS(pdsdata);
    return 0;
}
#endif

/*
**********************************************************************************************************
* Interrogate a fixed size array. Replace the first space with a null delimiter.  If no space exists, set the byte following the array to null.
**********************************************************************************************************
*/
void	nullDelimit(char arg_szString[],char arg_acArray[],long arg_lMaxBufferSize ){
    char* pcTemp;

    memcpy(arg_szString,arg_acArray,arg_lMaxBufferSize-1);
    pcTemp = strstr(arg_szString," ");
    if (pcTemp){
        *pcTemp=0;
    }else{
        arg_szString[arg_lMaxBufferSize-1] = 0;
    }
}

/*
**********************************************************************************************************
* Obtain the userid executing the task on z/OS
**********************************************************************************************************
*/
#if defined(__MVS__)

void GetZOSUserId(char *buffer) {
    typedef struct
    {
        char UidLen;
        char UserId[8];
    } AceeUid;

    char * loc;
    long * addr;
    long * addr2;
    AceeUid * UID;

    addr=   *((long*)(0X00000224));
    addr2= addr + (0X0000006C/4);
    addr= *addr2 ;
    addr2= addr + (0X000000C8/4);
    addr= *addr2 ;
    loc = addr;
    loc= loc+20;
    UID=loc;
    memcpy(buffer,UID->UserId,8);
}
#endif

char * MQExitReason(PMQCXP pChannelExitParms) {
    switch(pChannelExitParms->ExitReason) {
    case(MQXR_INIT):            return("MQXR_INIT");
    case(MQXR_TERM):            return("MQXR_TERM");
    case(MQXR_MSG):             return("MQRX_MSG");
    case(MQXR_XMIT):            return("MQXR_XMIT");
    case(MQXR_SEC_MSG):         return("MQXR_SEC_MSG");
    case(MQXR_INIT_SEC):        return("MQXR_INIT_SEC");
    case(MQXR_RETRY):           return("MQXR_RETRY");
    case(MQXR_AUTO_CLUSSDR):    return("MQXR_AUTO_CLUSSDR");
    case(MQXR_AUTO_RECEIVER):   return("MQXR_AUTO_RECEIVER");
    case(MQXR_AUTO_SVRCONN):    return("MQXR_AUTO_SVRCONN");
    case(MQXR_AUTO_CLUSRCVR):   return("MQXR_AUTO_CLUSRCVR");
#ifdef MQXR_SEC_PARMS
    case(MQXR_SEC_PARMS): 	     return("MQXR_SEC_PARMS");
#endif
    default:                    return("Unknown ExitReason");
    } /* end switch */
}

char * MQExitId(PMQCXP pChannelExitParms) {
    switch(pChannelExitParms->ExitId) {
    case(MQXT_CHANNEL_SEC_EXIT):         return("MQXT_CHANNEL_SEC_EXIT");
    case(MQXT_CHANNEL_MSG_EXIT):         return("MQXT_CHANNEL_MSG_EXIT");
    case(MQXT_CHANNEL_SEND_EXIT):        return("MQXT_CHANNEL_SEND_EXIT");
    case(MQXT_CHANNEL_RCV_EXIT):         return("MQXT_CHANNEL_RCV_EXIT");
    case(MQXT_CHANNEL_MSG_RETRY_EXIT):   return("MQXT_CHANNEL_MSG_RETRY_EXIT");
    case(MQXT_CHANNEL_AUTO_DEF_EXIT):    return("MQXT_CHANNEL_AUTO_DEF_EXIT");
    default:                             return("Unknown ExitID");
    } /* end switch */
}

char * MQChannelType(PMQCD pChannelDefinition) {
    switch(pChannelDefinition->ChannelType) {
    case(MQCHT_SENDER):         return("MQCHT_SENDER");
    case(MQCHT_SERVER):         return("MQCHT_SERVER");
    case(MQCHT_RECEIVER):       return("MQCHT_RECEIVER");
    case(MQCHT_REQUESTER):      return("MQCHT_REQUESTER");
    case(MQCHT_CLNTCONN):       return("MQCHT_CLNTCONN");
    case(MQCHT_SVRCONN):        return("MQCHT_SVRCONN");
    case(MQCHT_CLUSSDR):        return("MQCHT_CLUSSDR");
    case(MQCHT_CLUSRCVR):       return("MQCHT_CLUSRCVR");
    default:                    return("Unknown ChannelType");
    } /* end switch */
}


#ifdef WIN32
/*
 *   Update the shadred table entry for the passed channel.
 */
long UpdateShrMem(MQCD *pChannelDefinition,  long* what2do, workingStorage_type *pWS)
{
    char filename[128] = "";
    char okrc[9];

    int channelix=-1;
    int found=-1;
    int iId=676735;
    HANDLE hShrMemFile;
    LPCTSTR pBuf;
    long CurrentAc;

    HANDLE MutexBlockIP2S;

    shmem_type * shm;

    #define BUFSIZE 4096

    STARTUPINFO si;
    PROCESS_INFORMATION pi;
    BOOL fSuccess;

    char szCommand2do[256];

    HANDLE   semaphore;
    DWORD    retVal;
    SECURITY_ATTRIBUTES  sec_attr;
    char szSemKey[256];
    long  lCount;

    LogData(LOGDEBUG,"UpdateShrMem() entered", pWS);

    CurrentAc=-1;

    /* Generate the name of the file */
    strcpy(filename,"Global\\BlockIP2-");
    strcat(filename,pWS->szQMgr);
    strcat(filename,"-shm");              /* shared mem */

    sprintf(pWS->szTmpBufff,"UpdateShrMem() shmem filename [%s] CurrentAc %ld", filename,CurrentAc);
    LogData(LOGDEBUG, pWS->szTmpBufff, pWS);

    hShrMemFile = OpenFileMapping(FILE_MAP_ALL_ACCESS,     /* read/write access        */
                                  FALSE,                   /* Do not inherit the name  */
                                  filename);               /* Name of mapping object   */

    sprintf(pWS->szTmpBufff,"UpdateShrMem() hShrMemFile [0x%lx]", hShrMemFile);
    LogData(LOGDEBUG, pWS->szTmpBufff, pWS);

    if (hShrMemFile == NULL) {
        /* This part must be encapsulated in a mutex/semaphore */
        /*Set mutex here and evt. semaphore here */

        OpenMTXBlockIP2S(pWS, &MutexBlockIP2S);
        LogData(LOGDEBUG,"After OpenMTXBlockIP2S() entered", pWS);

        LockBlockIP2S(pWS, &MutexBlockIP2S);
        LogData(LOGDEBUG,"After LockBlockIP2S() entered", pWS);

        /* See if another thread have started the SHared Memory Server(SMS) */

        hShrMemFile = OpenFileMapping(FILE_MAP_ALL_ACCESS,     /* read/write access        */
                                      FALSE,                   /* Do not inherit the name  */
                                      filename);               /* Name of mapping object   */

        if (hShrMemFile == NULL) {
            /* Start the shared memory server */
            if (LoadRegistry(pWS) == 0) {
                LogData(LOGDEBUG,"UpdateShrMem() LoadRegistry() OK",pWS);

                /* Initialise si and pi */
                ZeroMemory( &si, sizeof(si) );
                si.cb = sizeof(si);
                ZeroMemory( &pi, sizeof(pi) );

                strcpy(szCommand2do,pWS->ExitPath);
                strcat(szCommand2do,"\\BlockIP2S.exe ");
                strcat(szCommand2do,pWS->szQMgr);

                sprintf(pWS->szTmpBufff,"UpdateShrMem() CreateProcess [%s]", szCommand2do);
                LogData(LOGDEBUG, pWS->szTmpBufff, pWS);

                /* create sem2. here */

                sec_attr.nLength              = sizeof( SECURITY_ATTRIBUTES );
                sec_attr.lpSecurityDescriptor = NULL;
                sec_attr.bInheritHandle       = TRUE;

                strcpy(szSemKey,"Global\\BlockIP2S-");
                strcat(szSemKey,pWS->szQMgr);
                strcat(szSemKey,"-ctrl");

                sprintf(pWS->szTmpBufff,"Semaphore key is [%s].",  szSemKey);
                LogData(LOGDEBUG, pWS->szTmpBufff, pWS);

                semaphore = CreateSemaphore( &sec_attr, 0, 65536, szSemKey );
                if( semaphore == (HANDLE) NULL )
                    return -1;

                sprintf(pWS->szTmpBufff,"UpdateShrMem() Semaphore opened. (%lx)", semaphore);
                LogData(LOGDEBUG, pWS->szTmpBufff, pWS);

                /* Create a childProcess */
                if( !CreateProcess( NULL,             /* No module name (use command line).*/
                                    szCommand2do,     /* Command to execute                */
                                    NULL,             /* Process handle not inheritable.   */
                                    NULL,             /* Thread handle not inheritable.    */
                                    FALSE,            /* Set handle inheritance to FALSE.  */
                                    DETACHED_PROCESS, /* fdwCreate                         */
                                    NULL,             /* Use parent's environment block.   */
                                    NULL,             /* Use parent's starting directory.  */
                                    &si,              /* Pointer to STARTUPINFO structure. */
                                    &pi )             /* Pointer to PROCESS_INFORMATION structure. */
                  )
                {
                    LogData(LOGDATA,"UpdateShrMem() CreateProcess failed.",pWS);
                }
                else {
                    LogData(LOGDATA,"UpdateShrMem() BlockIP2S Shared memory controller detached.",pWS);
                }
                lCount = 10L;
                while( (( retVal = WaitForSingleObject( semaphore, 250 ) ) == WAIT_TIMEOUT) && lCount > 0) {
                    lCount--;
                    LogData(LOGDEBUG, "Timeout (250ms).", pWS);
                }
                if (lCount ==0) {
                    LogData(LOGDATA,"UpdateShrMem() BlockIP2S didn't respond correctly.",pWS);
                    if( ! ReleaseSemaphore(semaphore, 1, NULL) )      /* clean up */
                        LogData(LOGDATA, "ReleaseSemaphore() Failed.", pWS);
                    CloseHandle(semaphore);
                    return -1;
                }

                if( ! ReleaseSemaphore(semaphore, 1, NULL) ) {
                    LogData(LOGDATA, "ReleaseSemaphore() Failed.", pWS);
                    CloseHandle(semaphore);  /* Clean up */
                    return -1;
                }
                CloseHandle(semaphore);
                LogData(LOGDEBUG, "CloseHandle.", pWS);

                /* Obtain file again */

                hShrMemFile = OpenFileMapping(FILE_MAP_ALL_ACCESS,     /* read/write access        */
                                              FALSE,                   /* Do not inherit the name  */
                                              filename);               /* Name of mapping object   */

                sprintf(pWS->szTmpBufff,"UpdateShrMem() hShrMemFile [0x%lx]", hShrMemFile);
                LogData(LOGDEBUG, pWS->szTmpBufff, pWS);

                if (hShrMemFile == NULL) {
                    LogData(LOGDEBUG, "UpdateShrMem() Shared memory was not allocated yet.",pWS);
                    /* release mutex here */
                    FreeBlockIP2S(pWS, &MutexBlockIP2S);
                    LogData(LOGDEBUG,"UpdateShrMem() leaving", pWS);
                    return -1;
                }
            }
        }
        /* release mutex here */
        FreeBlockIP2S(pWS, &MutexBlockIP2S);
        LogData(LOGDEBUG,"After FreeBlockIP2S() entered", pWS);
    }

    pBuf=MapViewOfFile(hShrMemFile,                        /* handle to mapping object */
                       FILE_MAP_ALL_ACCESS,                /* Read/write access        */
                       0,
                       0,
                       SHMSZ);
    if (pBuf == NULL) {

        /* her skal BlockIP2S startes */

        sprintf(pWS->szTmpBufff,"UpdateShrMem() Could not map view of file [%s] error %d\n",filename,GetLastError());
        LogData(LOGDATA, pWS->szTmpBufff, pWS);
        LogData(LOGDEBUG,"UpdateShrMem() leaving", pWS);
        return -1;
    }

    shm = pBuf;

    sprintf(pWS->szTmpBufff,"UpdateShrMem() Memory table contains %ld channel entries",shm->iNumOfChannels);
    LogData(LOGDEBUG, pWS->szTmpBufff, pWS);

    /*
     *  We're ready to locate the channel entry
     */
    for (channelix=0; channelix < shm->iNumOfChannels; channelix++) {
        if (!strcmp(shm->chnl[channelix].szChannelName, pWS->szChannelName)) {
            found=channelix;
            sprintf(pWS->szTmpBufff,"UpdateShrMem() Channel [%s] located at %ld", pWS->szChannelName,found);
            LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
            channelix=shm->iNumOfChannels;   /* Stop loop */
        }
    }

    /*
     *  if we didn't locate the channel insert channel entry
     */
    if (found==-1) {
        found=channelix;
        if (shm->iNumOfChannels<SHMMAX_CHL) {
            LockChannelSem(pWS);
            /* see if another have inserted before us (just to be shure) */
            found=-1;
            for (channelix=0; channelix < shm->iNumOfChannels; channelix++) {
                if (!strcmp(shm->chnl[channelix].szChannelName, pWS->szChannelName)) {
                    found=channelix;
                    sprintf(pWS->szTmpBufff,"UpdateShrMem() Channel [%s] located at %ld", pWS->szChannelName,found);
                    LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
                    channelix=shm->iNumOfChannels;   /* Stop loop */
                }
            }
            if (found == -1) {
                channelix=shm->iNumOfChannels++;
                shm->chnl[channelix].iNextChl=0;
                shm->chnl[channelix].iActiveChannelLimit=pWS->lMaxChannelActive;
                shm->chnl[channelix].iCurrentActiveConnections=0;
                shm->chnl[channelix].iMaxActiveConnections=0;
                strcpy(shm->chnl[channelix].szChannelName,pWS->szChannelName);
                sprintf(pWS->szTmpBufff,"UpdateShrMem() Channel [%s] added to channel tabel at %ld", pWS->szChannelName,channelix);
                LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
            }
            FreeChannelSem(pWS);
        }
        else {
            UnmapViewOfFile(pBuf);         /* detaches the shared memory segment */
            CloseHandle(hShrMemFile);
            sprintf(pWS->szTmpBufff,"UpdateShrMem() No more room in channel table limit is: %ld",SHMMAX_CHL);
            LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
            LogData(LOGDEBUG,"UpdateShrMem() leaving", pWS);
            return CurrentAc;
        }

    }

    /*
     *  now it's time to see what we're doing.....
     */

    channelix=found; /* Setup pointer */

    if(shm->chnl[channelix].iActiveChannelLimit != pWS->lMaxChannelActive) {
        LockChannelSem(pWS);
        shm->chnl[channelix].iActiveChannelLimit=pWS->lMaxChannelActive;
        sprintf(pWS->szTmpBufff,"UpdateShrMem() Channel [%s] limit changed to %ld", pWS->szChannelName,
                pWS->lMaxChannelActive);
        LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
        FreeChannelSem(pWS);
    }

    if (shm->chnl[channelix].iCurrentActiveConnections+1>shm->chnl[channelix].iActiveChannelLimit) {
        sprintf(pWS->szTmpBufff,"UpdateShrMem() Channel [%s] max lim was reached %ld", pWS->szChannelName,
                shm->chnl[channelix].iActiveChannelLimit);
        LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
    }
    else
    {
        CurrentAc=0;      /* value below limit */
    }

    LockChannelSem(pWS);
    if (what2do != (long*)-1) {
        shm->chnl[channelix].iCurrentActiveConnections++;
        if (shm->chnl[channelix].iCurrentActiveConnections > shm->chnl[channelix].iMaxActiveConnections)
            shm->chnl[channelix].iMaxActiveConnections=shm->chnl[channelix].iCurrentActiveConnections;
        LogData(LOGDEBUG, "UpdateShrMem() Increment counter", pWS);
    }
    else {
        if (shm->chnl[channelix].iCurrentActiveConnections > 0)
            shm->chnl[channelix].iCurrentActiveConnections--;
        LogData(LOGDEBUG, "UpdateShrMem() Decrement counter", pWS);
    }
    pWS->iCurrentActiveConnections =shm->chnl[channelix].iCurrentActiveConnections;
    FreeChannelSem(pWS);
    sprintf(pWS->szTmpBufff,"UpdateShrMem() status return %ld (0=below limit)", CurrentAc);
    LogData(LOGDEBUG, pWS->szTmpBufff, pWS);

    UnmapViewOfFile(pBuf);

    CloseHandle(hShrMemFile);

    LogData(LOGDEBUG,"UpdateShrMem() leaving", pWS);
    return CurrentAc;
}

#endif


#if defined (UNIX)

/*
 *   Update the shadred table entry for the passed channel.
 */
long UpdateShrMem(MQCD *pChannelDefinition,  long* what2do, workingStorage_type *pWS)
{
    long CurrentAc=-1;
    char filename[128] = "";
    char okrc[9];
    FILE *fptr;
    int semid, shmid;
    key_t key,keyShm;

    int channelix=-1;
    int found=-1;
    int iId=676735;

    shmem_type * shm;

    if (pWS->KeyShmem==-1) {    /* key not obtained yet */

/*        strcpy(filename,"/var/mqm/qmgrs/");
        strcat(filename,pWS->szQMgr);*/

        strcpy(filename,pWS->szQmgrPrefix);     /* 2.45 change*/
        strcat(filename,"/qmgrs/");             /* 2.45 change*/
        strcat(filename,pWS->szQmgrDirectory);  /* 2.45 change*/

        strcat(filename,"/shmem/BlockIP2.2");              /* shared mem */

        sprintf(pWS->szTmpBufff,"UpdateShrMem() - generate shm shmem [%s]", filename);
        LogData(LOGDEBUG, pWS->szTmpBufff, pWS);

        if((pWS->KeyShmem=ftok(filename,iId)) == -1) {
            if((fptr = fopen( filename,"a+"))!=NULL)
                fclose(fptr);
            if((pWS->KeyShmem=ftok(filename,iId)) == -1) {
                LogData(LOGDATA,"ftok error -1", pWS);
                perror("ftok");
                return -1;
            }
        }
    }

    if ((shmid = shmget(pWS->KeyShmem, SHMSZ, 0660)) < 0) {
        LockAllSem(pWS);
        LogData(LOGDEBUG,"UpdateShrMem() Create shared memory area...", pWS);
        if ((shmid = shmget(pWS->KeyShmem, SHMSZ, IPC_CREAT | 0660)) < 0) {
            LogData(LOGDATA,"Creation failed(shmget) -1", pWS);
            FreeAllSem(pWS);
            return -1;
        }

        shm = shmat(shmid, NULL, 0);
        if ((size_t)shm  == (size_t) -1) {
            LogData(LOGDATA,"UpdateShrMem() Addressing of shmem failed(shmat) -1", pWS);
            FreeAllSem(pWS);
            perror("shmat");
            return(-1);
        }
        shm->iPid= iId;         /* Say it's mine  */
        shm->iNumOfChannels=0;  /* No channes yet */
        LogData(LOGDEBUG,"UpdateShrMem() Shared memory initialized.", pWS);


        shmdt((char*)shm); 	/* detaches the shared memory segment */
        FreeAllSem(pWS);
    }
    LogData(LOGDEBUG,"UpdateShrMem() Working",pWS);
    /*
     * Now we attach the segment to our data space.
     */
    shm = shmat(shmid, NULL, 0);
    if ((size_t)shm == (size_t) -1) {
        LogData(LOGDATA,"UpdateShrMem() Addressing of shmem failed(shmat) -1", pWS);
        perror("shmat");
        return(-1);
    }

    /*
     *  See if it's my own
     */
    if (shm->iPid != iId) {    /* Nope, then it's time to init */
        LockAllSem(pWS);
        if (shm->iPid != iId) {    /* Just to be shure.... */
            shm->iPid= iId;         /* Say it's mine  */
            shm->iNumOfChannels=0;  /* No channes yet */
            LogData(LOGDEBUG,"UpdateShrMem() Shared memory initialized.", pWS);
        }
        FreeAllSem(pWS);
    }

    /*
     *  We're ready to locate the channel entry
     */

    for (channelix=0; channelix < shm->iNumOfChannels; channelix++) {
        if (!strcmp(shm->chnl[channelix].szChannelName, pWS->szChannelName)) {
            found=channelix;
            sprintf(pWS->szTmpBufff,"UpdateShrMem() Channel [%s] located at %ld", pWS->szChannelName,found);
            LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
            channelix=shm->iNumOfChannels;   /* Stop loop */
        }
    }

    /*
     *  if we didn't locate the channel insert channel entry
     */
    if (found==-1) {
        found=channelix;
        if (shm->iNumOfChannels<SHMMAX_CHL) {
            LockChannelSem(pWS);
            channelix=shm->iNumOfChannels++;
            shm->chnl[channelix].iNextChl=0;
            shm->chnl[channelix].iActiveChannelLimit=pWS->lMaxChannelActive;
            shm->chnl[channelix].iCurrentActiveConnections=0;
            shm->chnl[channelix].iMaxActiveConnections=0;
            strcpy(shm->chnl[channelix].szChannelName,pWS->szChannelName);
            sprintf(pWS->szTmpBufff,"UpdateShrMem() Channel [%s] added to channel tabel at %ld", pWS->szChannelName,channelix);
            LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
            FreeChannelSem(pWS);
        }
        else {
            shmdt((char*)shm);             /* detaches the shared memory segment */
            sprintf(pWS->szTmpBufff,"UpdateShrMem() No more room in channel table limit is: %ld",SHMMAX_CHL);
            LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
            LogData(LOGDEBUG,"UpdateShrMem() leaving", pWS);
            return CurrentAc;
        }
    }

    /*
     *  now it's time to see what we're doing.....
     */

    channelix=found; /* Setup pointer */

    if(shm->chnl[channelix].iActiveChannelLimit != pWS->lMaxChannelActive) {
        LockChannelSem(pWS);
        shm->chnl[channelix].iActiveChannelLimit=pWS->lMaxChannelActive;
        sprintf(pWS->szTmpBufff,"UpdateShrMem() Channel [%s] limit changed to %ld", pWS->szChannelName,
                pWS->lMaxChannelActive);
        LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
        FreeChannelSem(pWS);
    }

    if (shm->chnl[channelix].iCurrentActiveConnections+1>shm->chnl[channelix].iActiveChannelLimit) {
        sprintf(pWS->szTmpBufff,"UpdateShrMem() Channel [%s] max lim was reached %ld", pWS->szChannelName,
                shm->chnl[channelix].iActiveChannelLimit);
        LogData(LOGDEBUG, pWS->szTmpBufff, pWS);
    }
    else
    {
        CurrentAc=0;      /* value below limit */
    }

    LockChannelSem(pWS);
    if (what2do != (long*)-1) {
        shm->chnl[channelix].iCurrentActiveConnections++;
        if (shm->chnl[channelix].iCurrentActiveConnections > shm->chnl[channelix].iMaxActiveConnections)
            shm->chnl[channelix].iMaxActiveConnections=shm->chnl[channelix].iCurrentActiveConnections;
        LogData(LOGDEBUG, "UpdateShrMem() Increment counter", pWS);
    }
    else {
        if (shm->chnl[channelix].iCurrentActiveConnections > 0)
            shm->chnl[channelix].iCurrentActiveConnections--;
        LogData(LOGDEBUG, "UpdateShrMem() Decrement counter", pWS);
    }
    pWS->iCurrentActiveConnections =shm->chnl[channelix].iCurrentActiveConnections;
    FreeChannelSem(pWS);
    sprintf(pWS->szTmpBufff,"UpdateShrMem() status return %ld (0=below limit)", CurrentAc);
    LogData(LOGDEBUG, pWS->szTmpBufff, pWS);

    shmdt((char*)shm); 	/* detaches the shared memory segment */
    return CurrentAc;
}
/*
    key_t     KeySemaphores;
    key_t     KeyShmem;
    LockPWS1Sem();
#define SEM_SHM1 0
#define SEM_CHL1 1
#define SEM_PWS1 2
    */

int LockAllSem(workingStorage_type *pWS)
{
    int nsops; /* number of operations to do */
    struct sembuf *sops = (struct sembuf *) malloc(6*sizeof(struct sembuf));
    int rc, workint;
    char filename[128] = "";
    FILE *fptr;
    int semid;
    int iId=676735;


    rc=16;

    LogData(LOGDEBUG, "LockAllSem() Entered", pWS);

    if (pWS->KeySemaphores==-1) {    /* key not obtained yet */
        /*        iId     = getpid(); JHP060203 */ /* Get parent id */
/*        strcpy(filename,"/var/mqm/qmgrs/");
        strcat(filename,pWS->szQMgr);*/

        strcpy(filename,pWS->szQmgrPrefix);     /* 2.45 change*/
        strcat(filename,"/qmgrs/");             /* 2.45 change*/
        strcat(filename,pWS->szQmgrDirectory);  /* 2.45 change*/

        strcat(filename,"/shmem/BlockIP2.1");

        sprintf(pWS->szTmpBufff,"LockAllSem() - generate semaphores shmem [%s]", filename);
        LogData(LOGDEBUG, pWS->szTmpBufff, pWS);

        if((pWS->KeySemaphores=ftok(filename,iId)) == -1) {
            if((fptr = fopen( filename,"a+"))!=NULL)
                fclose(fptr);
            if((pWS->KeySemaphores=ftok(filename,iId)) == -1) {
                LogData(LOGDATA,"ftok error -1", pWS);
                perror("ftok");
                return rc;
            }
        }
        LogData(LOGDEBUG, "ftok1 %ld ok", pWS);

        /* open semaphore set with 6 semaphore: */
        if ((semid = semget(pWS->KeySemaphores, 6, 0660 )) == -1) {
            /* create a semaphore set with 6 semaphore: */
            LogData(LOGDEBUG,"Create semaphores...", pWS);

            if ((semid = semget(pWS->KeySemaphores, 6, 0660 | IPC_CREAT)) == -1) {
                LogData(LOGDATA,"semget error -1", pWS);
                return rc;
            }
            LogData(LOGDEBUG,"Create semaphores... Done", pWS);
        }
    }

    nsops = 6;

    sops[0].sem_num = 0; /* We only use one track */
    sops[0].sem_op = 0; /* wait for semaphore flag to become zero */
    sops[0].sem_flg = SEM_UNDO; /* take off semaphore asynchronous  */

    sops[1].sem_num = 0;
    sops[1].sem_op = 1; /* increment semaphore -- take control of track */
    sops[1].sem_flg = SEM_UNDO | IPC_NOWAIT; /* take off semaphore */

    sops[2].sem_num = 1; /* We only use one track */
    sops[2].sem_op = 0; /* wait for semaphore flag to become zero */
    sops[2].sem_flg = SEM_UNDO; /* take off semaphore asynchronous  */


    sops[3].sem_num = 1;
    sops[3].sem_op = 1; /* increment semaphore -- take control of track */
    sops[3].sem_flg = SEM_UNDO | IPC_NOWAIT; /* take off semaphore */

    sops[4].sem_num = 2; /* We only use one track */
    sops[4].sem_op = 0; /* wait for semaphore flag to become zero */
    sops[4].sem_flg = SEM_UNDO; /* take off semaphore asynchronous  */


    sops[5].sem_num = 2;
    sops[5].sem_op = 1; /* increment semaphore -- take control of track */
    sops[5].sem_flg = SEM_UNDO | IPC_NOWAIT; /* take off semaphore */

    LogData(LOGDEBUG,"executing semop...", pWS);
    /* Make the semop() call and report the results. */
    if ((workint = semop(semid, sops, nsops)) == -1) {
        LogData(LOGDATA,"LockAllSem() semop: semop failed", pWS);
    }
    else
    {
        rc=0;
    }

    LogData(LOGDEBUG, "LockAllSem() leaved", pWS);
    return rc;
}  /* LockAllSem() */

int FreeAllSem(workingStorage_type *pWS)
{
    int nsops; /* number of operations to do */
    struct sembuf *sops = (struct sembuf *) malloc(3*sizeof(struct sembuf));
    int iId, rc, workint;
    char filename[128] = "";
    FILE *fptr;
    int semid;

    rc=16;
    LogData(LOGDEBUG, "FreeAllSem() Entered", pWS);
    /* open semaphore set with 6 semaphore: */
    if ((semid = semget(pWS->KeySemaphores, 6, 0660 )) == -1) {
        /* create a semaphore set with 6 semaphore: */
        LogData(LOGDEBUG,"FreeAllSem() semget failed ...", pWS);
        return rc;
    }

    nsops = 3;

    /* wait for semaphore to reach zero */
    sops[0].sem_num = 0;
    sops[0].sem_op = -1; /* Give UP COntrol of track */
    sops[0].sem_flg = SEM_UNDO | IPC_NOWAIT; /* take off semaphore, asynchronous  */

    sops[1].sem_num = 1;
    sops[1].sem_op = -1; /* Give UP COntrol of track */
    sops[1].sem_flg = SEM_UNDO | IPC_NOWAIT; /* take off semaphore, asynchronous  */

    sops[2].sem_num = 2;
    sops[2].sem_op = -1; /* Give UP COntrol of track */
    sops[2].sem_flg = SEM_UNDO | IPC_NOWAIT; /* take off semaphore, asynchronous  */

    if ((workint = semop(semid, sops, nsops)) == -1) {
        LogData(LOGDATA,"FreeAllSem() semop: semop failed", pWS);
        return rc;
    }
    else
        rc=0;
    LogData(LOGDEBUG, "FreeAllSem() leaved", pWS);
    return rc;
}  /* FreeAllSem() */


int LockSpecSem(int sem_number, workingStorage_type *pWS)
{
    int nsops; /* number of operations to do */
    struct sembuf *sops = (struct sembuf *) malloc(2*sizeof(struct sembuf));
    int rc, workint;
    char filename[128] = "";
    FILE *fptr;
    int semid;
    int iId=676735;

    LogData(LOGDEBUG, "LockSpecSem() entered", pWS);

    rc=16;

    /*iId     = getpid(); JHP060203*/ /* Get parent id */
/*    strcpy(filename,"/var/mqm/qmgrs/");
    strcat(filename,pWS->szQMgr);*/

    strcpy(filename,pWS->szQmgrPrefix);     /* 2.45 change*/
    strcat(filename,"/qmgrs/");             /* 2.45 change*/
    strcat(filename,pWS->szQmgrDirectory);  /* 2.45 change*/

    strcat(filename,"/shmem/BlockIP2.1");

    sprintf(pWS->szTmpBufff,"LockAllSem() - generate sem shmem [%s]", filename);
    LogData(LOGDEBUG, pWS->szTmpBufff, pWS);

    if((pWS->KeySemaphores=ftok(filename,iId)) == -1) {
        if((fptr = fopen( filename,"a+"))!=NULL)
            fclose(fptr);
        if((pWS->KeySemaphores=ftok(filename,iId)) == -1) {
            LogData(LOGDATA,"ftok error -1", pWS);
            perror("ftok");
            return rc;
        }
    }
    LogData(LOGDEBUG, "ftok1 %ld ok", pWS);

    /* open semaphore set with 6 semaphore: */
    if ((semid = semget(pWS->KeySemaphores, 6, 0660 )) == -1) {
        /* create a semaphore set with 6 semaphore: */
        LogData(LOGDEBUG,"Create semaphores...", pWS);

        if ((semid = semget(pWS->KeySemaphores, 6, 0660 | IPC_CREAT)) == -1) {
            LogData(LOGDATA,"semget error -1", pWS);
            return rc;
        }
    }

    nsops = 2;

    sops[0].sem_num = sem_number;  /* We only use one track */
    sops[0].sem_op = 0;            /* wait for semaphore flag to become zero */
    sops[0].sem_flg = SEM_UNDO;    /* take off semaphore asynchronous  */

    sops[1].sem_num = sem_number;
    sops[1].sem_op = 1;            /* increment semaphore -- take control of track */
    sops[1].sem_flg = SEM_UNDO | IPC_NOWAIT; /* take off semaphore */

    /* Make the semop() call and report the results. */
    if ((workint = semop(semid, sops, nsops)) == -1) {
        sprintf(pWS->szTmpBufff,"LockSpecSem() semop: semop failed # %ld", sem_number);
        LogData(LOGDATA, pWS->szTmpBufff, pWS);
    }
    else
    {
        rc=0;
    }
    LogData(LOGDEBUG, "LockSpecSem() leaved", pWS);
    return rc;
}  /* LockSpecSem() */

int FreeSpecSem(int sem_number, workingStorage_type *pWS)
{
    int nsops; /* number of operations to do */
    struct sembuf *sops = (struct sembuf *) malloc(1*sizeof(struct sembuf));
    int iId, rc, workint;
    char filename[128] = "";
    FILE *fptr;
    int semid;

    LogData(LOGDEBUG, "FreeSpecSem() entered", pWS);
    rc=16;
    /* open semaphore set with 6 semaphore: */
    if ((semid = semget(pWS->KeySemaphores, 6, 0660 )) == -1) {
        /* create a semaphore set with 6 semaphore: */
        sprintf(pWS->szTmpBufff,"FreeSpecSem() semget failed # %ld", sem_number);
        LogData(LOGDATA, pWS->szTmpBufff, pWS);
        return rc;
    }

    nsops = 1;

    /* wait for semaphore to reach zero */
    sops[0].sem_num = sem_number;
    sops[0].sem_op = -1; /* Give UP COntrol of track */
    sops[0].sem_flg = SEM_UNDO | IPC_NOWAIT; /* take off semaphore, asynchronous  */

    if ((workint = semop(semid, sops, nsops)) == -1) {
        sprintf(pWS->szTmpBufff,"FreeSpecSem() semop failed # %ld", sem_number);
        LogData(LOGDATA, pWS->szTmpBufff, pWS);
        return rc;
    }
    else
        rc=0;
    LogData(LOGDEBUG, "FreeSpecSem() leaved", pWS);
    return rc;
}  /* FreeSpecSem() */

int LockChannelSem(workingStorage_type *pWS)
{
    LogData(LOGDEBUG, "LockChannelSem() entered", pWS);
    return LockSpecSem(SEM_CHL, pWS);
}

int FreeChannelSem(workingStorage_type *pWS)
{
    LogData(LOGDEBUG, "FreeChannelSem() entered", pWS);
    return FreeSpecSem(SEM_CHL, pWS);
}

#endif    /* exit for UNIX */

#ifdef WIN32
void OpenMTX(workingStorage_type *pWS, char *szLogFile)
{
    FILE *fptrx;			/* Declare file handles for input and output   */
    char szFile[256];
    char szFile2[256];

    strcpy(szFile2,szLogFile);
    translate2(szFile2,"","");          /* remove \\, : and ' ' */
    strcpy(szFile,"Global\\Ms0o-");
    strcat(szFile,szFile2);
    if (pWS->hMutex == ERROR_INVALID_HANDLE | strcmp(szFile,pWS->szMutexKey)) {
        strcpy(pWS->szMutexKey,szFile);
        pWS->hMutex = CreateMutex( NULL, FALSE, szFile );
        if (pWS->hMutex == ERROR_INVALID_HANDLE)
        {
            if((fptrx = fopen( "c:\\BlockIP2x.log",LOGMODE))!=NULL)
            {
                fprintf(fptrx,"CreateMutex failed Error code %ld\n", GetLastError());
                fclose(fptrx);
            }
        }
    }
}

void OpenMTXShm(workingStorage_type *pWS)
{
    FILE *fptrx;			/* Declare file handles for input and output   */
    char szMutexKey[256];
    char szMutexKey2[256];

    if (pWS->hMutexShm == ERROR_INVALID_HANDLE) {
        strcpy(szMutexKey2,pWS->szQMgr);
        strcpy(szMutexKey,"Global\\BlockIP2-");
        strcat(szMutexKey,szMutexKey2);
        strcat(szMutexKey,"-shmMutex");
        pWS->hMutexShm = CreateMutex( NULL, FALSE, szMutexKey);
        if (pWS->hMutexShm == ERROR_INVALID_HANDLE)
        {
            sprintf(pWS->szTmpBufff,"CreateMutex failed Error code %ld\n", GetLastError());
            LogData(LOGDATA, pWS->szTmpBufff, pWS);
        }
    }
}

int LockChannelSem(workingStorage_type *pWS)
{
    long lCount=0;
    DWORD dwWaitResult;

    LogData(LOGDEBUG, "LockChannelSem() entered", pWS);
    OpenMTXShm(pWS);
    lCount=0;
    while( ( dwWaitResult = WaitForSingleObject( pWS->hMutexShm, 5L ) ) == WAIT_TIMEOUT)
        lCount++;

    return 0;
}

int FreeChannelSem(workingStorage_type *pWS)
{
    LogData(LOGDEBUG, "FreeChannelSem() entered", pWS);
    ReleaseMutex( pWS->hMutexShm );
    return 0;
}

void OpenMTXBlockIP2S(workingStorage_type *pWS, HANDLE *MutexBlockIP2S)
{
    FILE *fptrx;			/* Declare file handles for input and output   */
    char szMutexKey[256];
    char szMutexKey2[256];

    if (MutexBlockIP2S == ERROR_INVALID_HANDLE) {
        strcpy(szMutexKey2,pWS->szQMgr);
        strcpy(szMutexKey,"Global\\BlockIP2-");
        strcat(szMutexKey,szMutexKey2);
        strcat(szMutexKey,"-BlockIP2S");
        MutexBlockIP2S = CreateMutex( NULL, FALSE, szMutexKey);
        if (MutexBlockIP2S == ERROR_INVALID_HANDLE)
        {
            sprintf(pWS->szTmpBufff,"CreateMutex failed Error code %ld\n", GetLastError());
            LogData(LOGDATA, pWS->szTmpBufff, pWS);
        }
    }
}

int LockBlockIP2S(workingStorage_type *pWS, HANDLE *MutexBlockIP2S)
{
    long lCount=0;
    DWORD dwWaitResult;

    LogData(LOGDEBUG, "LockBlockIP2S() entered", pWS);
    OpenMTXShm(pWS);
    lCount=0;
    while( ( dwWaitResult = WaitForSingleObject( pWS->hMutexShm, 5L ) ) == WAIT_TIMEOUT)
        lCount++;

    return 0;
}

int FreeBlockIP2S(workingStorage_type *pWS, HANDLE *MutexBlockIP2S)
{
    LogData(LOGDEBUG, "FreeBlockIP2S() entered", pWS);
    ReleaseMutex( MutexBlockIP2S );
    return 0;
}


#define DMKEY "SOFTWARE\\IBM\\MQSeries\\CurrentVersion\\Configuration\\QueueManager\\"
#define SUBKEY "ExitsDefaultPath"


int LoadRegistry(workingStorage_type *pWS) {
    DWORD rc;
    DWORD len;
    HKEY dmKey;
    DWORD dwType;
    char  key[256];

    strcpy(key,DMKEY);
    strcat(key,pWS->szQMgr);
    strcat(key,"\\ExitPath\\");

    len = sizeof(pWS->ExitPath);

    rc = RegOpenKeyEx( HKEY_LOCAL_MACHINE,  key,   0,   KEY_QUERY_VALUE,   &dmKey   );
    if (rc != ERROR_SUCCESS) {
        LogData(LOGDEBUG,"LoadRegistry RegOpenKeyEx failed",pWS);
        return 4;
    }

    RegQueryValueEx( dmKey, SUBKEY, 0, &dwType, (LPBYTE)pWS->ExitPath, &len );

    sprintf(pWS->szTmpBufff,"ExitsDefaultPath for QMGR is [%s] in Registry", pWS->ExitPath);
    LogData(LOGDEBUG,pWS->szTmpBufff,pWS);

    if (RegQueryValueEx(dmKey, SUBKEY, 0, &dwType, (LPBYTE)pWS->ExitPath, &len )!=ERROR_SUCCESS) {
        LogData(LOGDEBUG,"LoadRegistry RegQueryValueEx failed-2",pWS);
    }

    return 0;
}

#endif /* WIN32 */

#if defined (__EXT_LOG_FILENAME__)

long getFileSize(char* szFileName, workingStorage_type *pWS) {

   struct stat buf;
   FILE *fp;
   int fh,result;
   long  lFileSize;

   fp = fopen(szFileName, "r");
   if (fp == NULL) {
       return(-1);
   }
   if (0 == fstat(fileno(fp), &buf)) {
      lFileSize= buf.st_size ;
   }
   else
      lFileSize=-1;
   fclose(fp);
   return lFileSize;
}


long CycleFiles(workingStorage_type *pWS) {
char Buffer[256];
char LogFile[256];
char szPath[256];
char szFile[256];
char szFile1[256];
char szDrive[256];
char szFileName[128];
char szExt[32];
int i;

if (pWS->bFileSizeCheck==0) {

    strcpy(szFile,GenerateLogFileName(Buffer,pWS));
   _Lsplitpath(szFile, szDrive, szPath, szFileName, szExt);

   szFileName[strlen(szFileName)-3] = '\00';
   sprintf(pWS->szTmpBufff,"CycleFiles: File info is drive[%s] path[%s] prefix [%s]", szDrive, szPath, szFileName);
   LogData(LOGDEBUG,pWS->szTmpBufff,pWS);
   sprintf(szFile,"%s%s%s%03d%s",szDrive,szPath,szFileName,pWS->lLogFileCount,szExt);
   sprintf(pWS->szTmpBufff,"CycleFiles: File to delete [%s]", szFile);
   LogData(LOGDEBUG,pWS->szTmpBufff,pWS);

   if (remove(szFile)==0) {
      LogData(LOGDEBUG,"File removed.",pWS);
      }
   else {
      LogData(LOGDEBUG,"File was not removed.",pWS);
      }
   for (i=pWS->lLogFileCount; i>1; i--) {
       sprintf(szFile,"%s%s%s%03d%s",szDrive,szPath,szFileName,i,szExt);
       sprintf(szFile1,"%s%s%s%03d%s",szDrive,szPath,szFileName,i-1,szExt);
       if (rename(szFile1,szFile)==0) {
          sprintf(pWS->szTmpBufff,"CycleFiles: File renamed [%s%03d%s]", szFileName,i-1,szExt);
          LogData(LOGDEBUG,pWS->szTmpBufff,pWS);
          }
       }

   pWS->bFileSizeCheck=1;
   return 0;
   }
else
  return 0;    /* Allways return ok when inactive */

}

long HandleCycleFiles(workingStorage_type *pWS) {
    char Buffer[256];
    char szFile[256];
    long lFileSize;


    LogData(LOGDEBUG, "Check for cycle files started.",pWS);
    strcpy(szFile,GenerateLogFileName(Buffer,pWS));
    lFileSize=getFileSize(szFile,pWS);
    sprintf(pWS->szTmpBufff,"File size of %s is %ld (limit %ld)", szFile, lFileSize,pWS->lMaxlogFileSize);
    LogData(LOGDEBUG, pWS->szTmpBufff,pWS);
    if (lFileSize > pWS->lMaxlogFileSize) {
        LogData(LOGDATA, "Cycle of logfiles is required.",pWS);
        if (CycleFiles(pWS)==0)
            LogData(LOGDATA,"LogFiles was changed.",pWS);
        else
            LogData(LOGDATA,"LogFile change Failed.",pWS);
        }
    return 0;
}


#endif /* __EXT_LOG_FILENAME__ */


void _Lsplitpath(const char* path, char* drv, char* dir, char* name, char* ext)
{
        const char* end; /* end of processed string */
        const char* p;   /* search pointer */
        const char* s;   /* copy pointer */

        /* extract drive name */
        if (path[0] && path[1]==':') {
                if (drv) {
                        *drv++ = *path++;
                        *drv++ = *path++;
                        *drv = '\0';
                }
        } else if (drv)
                *drv = '\0';

        /* search for end of string or stream separator */
        for(end=path; *end && *end!=':'; )
                end++;

        /* search for begin of file extension */
        for(p=end; p>path && *--p!='\\' && *p!='/'; )
                if (*p == '.') {
                       end = p;
                       break;
                }

        if (ext)
                for(s=end; (*ext=*s++); )
                        ext++;

        /* search for end of directory name */
        for(p=end; p>path; )
                if (*--p=='\\' || *p=='/') {
                        p++;
                        break;
                }

        if (name) {
                for(s=p; s<end; )
                        *name++ = *s++;

                *name = '\0';
        }

        if (dir) {
                for(s=path; s<p; )
                        *dir++ = *s++;

                *dir = '\0';
        }
}


/* End of file */
